import fs from "fs";
import {
  checkEnginesFromString,
  validatePackageJson,
  validatePackageLock,
} from "@smarlhens/npm-check-engines";

export function updatePackageEngines(packagePath, packageLockPath) {
  let packageJsonString = fs.readFileSync(packagePath); // load content of package.json as stringified JSON
  let packageLockString = fs.readFileSync(packageLockPath); // load content of package-lock.json as stringified JSON

  validatePackageJson({ packageJsonString }); // can throw Errors if unexpected format
  validatePackageLock({ packageLockString }); // can throw Errors if unexpected format

  const pkg = JSON.parse(packageJsonString);

  // packageJson is the content of your package.json with updated engines
  // enginesRangeToSet contains changes if you want to display them
  const { enginesRangeToSet, packageJson } = checkEnginesFromString({
    packageJsonString,
    packageLockString,
  });

  let nodeVersions = enginesRangeToSet.find(
    (el) => el.engine == "node"
  )?.rangeToSet;
  let npmVersions = enginesRangeToSet.find(
    (el) => el.engine == "npm"
  )?.rangeToSet;

  function updateEngine(pkg, engineName, engineVersion) {
    if (!pkg.engines) {
      pkg.engines = {};
    }

    pkg.engines[engineName] = engineVersion;
  }

  if (nodeVersions) {
    updateEngine(pkg, "node", nodeVersions);
  }

  if (npmVersions) {
    updateEngine(pkg, "npm", npmVersions);
  }

  console.log("current engines.");
  console.log(pkg.engines);

  fs.writeFileSync(packagePath, JSON.stringify(pkg, null, 2));
}
