import { WebDavStore } from "@/stores/WebDavStore.js";
import UrlResolveService from "@/services/UrlResolveService.js";
import Router from "@/router/index.js";
import webDavSettings from "@/WebDavSettings.js";

export default {
  run() {
    var connection = new WebSocket(this._getConnectionUrl());
    var self = this;
    connection.onmessage = function(event) {
      var notifyObject = JSON.parse(event.data);

      // Removing domain and trailing slash.
      var currentLocation = location.pathname.replace(/^\/|\/$/g, "");
      // Checking message type after receiving.
      if (
        notifyObject.EventType === "updated" ||
        notifyObject.EventType === "locked" ||
        notifyObject.EventType === "unlocked"
      ) {
        if (
          notifyObject.ItemPath.substring(
            0,
            notifyObject.ItemPath.lastIndexOf("/")
          ).toUpperCase() === currentLocation.toUpperCase()
        ) {
          WebDavStore.commit(
            "updateItem",
            new URL(UrlResolveService.getWebDavServerUrl()).origin +
              "/" +
              notifyObject.ItemPath
          );
        }
      } else if (notifyObject.EventType === "created") {
        // Refresh folder structure if any item in this folder is updated or new item is created.
        if (
          notifyObject.ItemPath.substring(
            0,
            notifyObject.ItemPath.lastIndexOf("/")
          ).toUpperCase() === currentLocation.toUpperCase()
        ) {
          WebDavStore.commit("updateGrid");
        }
      } else if (notifyObject.EventType === "moved") {
        // Refresh folder structure if file or folder is moved.
        if (
          notifyObject.ItemPath.substring(
            0,
            notifyObject.ItemPath.lastIndexOf("/")
          ).toUpperCase() === currentLocation.toUpperCase() ||
          notifyObject.TargetPath.substring(
            0,
            notifyObject.TargetPath.lastIndexOf("/")
          ).toUpperCase() === currentLocation.toUpperCase()
        ) {
          WebDavStore.commit("updateGrid");
        }
      } else if (notifyObject.EventType === "deleted") {
        if (
          notifyObject.ItemPath.substring(
            0,
            notifyObject.ItemPath.lastIndexOf("/")
          ).toUpperCase() === currentLocation.toUpperCase()
        ) {
          // Refresh folder structure if any item in this folder is deleted.
          WebDavStore.commit("updateGrid");
        } else if (
          currentLocation
            .toUpperCase()
            .indexOf(notifyObject.ItemPath.toUpperCase()) === 0
        ) {
          var rootFolder = UrlResolveService.getWebDavServerUrl();
          // Redirect client to the root folder if current path is being deleted.
          Router.push(rootFolder ? rootFolder : "/");
        }
      }
    };

    connection.onerror = function(err) {
      // eslint-disable-next-line no-console
      console.error(
        "Socket encountered error: ",
        err.message,
        "Closing socket"
      );
      connection.close();
    };

    connection.onclose = function(e) {
      // eslint-disable-next-line no-console
      console.log(
        "Socket is closed. Reconnect will be attempted in 5 seconds.",
        e.reason
      );
      setTimeout(function() {
        self.run();
      }, 5000);
    };
  },
  _getConnectionUrl() {
    if (webDavSettings.WebSocketPath) {
      return webDavSettings.WebSocketPath;
    } else {
      var socketSource;

      var oAppPath = new URL(UrlResolveService.getWebDavServerUrl());
      var host = oAppPath.host;

      if (location.protocol === "https:") {
        socketSource = "wss://" + host + "/dav";
      } else {
        socketSource = "ws://" + host + "/dav";
      }

      return socketSource;
    }
  }
};
