import { mount, createLocalVue } from "@vue/test-utils";
import Toolbar from "@/components/toolbar/Toolbar.vue";
import ToolbarConfig from "@/settings/toolbar.js";
import Vuex from "vuex";
import MockStore from "./mockStore.js";
import { oneFile, oneFolder } from "./testData.js";

const localVue = createLocalVue();
localVue.use(Vuex);

let store = MockStore;
let stubs = {
  UploadButton: {
    template: "<div class='stub'></div>"
  }
};

describe("toolbar without selected items", () => {
  let wrapper;
  let disabledButtons = [
    ToolbarConfig.copyButton,
    ToolbarConfig.cutButton,
    ToolbarConfig.pasteButton,
    ToolbarConfig.deleteButton,
    ToolbarConfig.downloadButton,
    ToolbarConfig.printButton,
    ToolbarConfig.renameButton
  ];
  let enabledButtons = [ToolbarConfig.createFolderButton];

  beforeEach(() => {
    wrapper = mount(Toolbar, {
      computed: {
        selectedItems() {
          return [];
        },
        storedItems() {
          return [];
        },
        isDavProtocolSupported() {
          return false;
        }
      },
      store,
      localVue,
      stubs: stubs
    });
  });

  test("renders", () => {
    expect(wrapper.html()).toMatchSnapshot();
  });

  test.each(disabledButtons)("is disabled %# button", ({ id }) => {
    expect(wrapper.find("#" + id).attributes("disabled")).toBe("disabled");
  });

  test.each(enabledButtons)("is enabled %# button", ({ id }) => {
    expect(wrapper.find("#" + id).attributes("disabled")).toBeUndefined();
  });
});

describe("toolbar with 1 selected file", () => {
  let wrapper;
  let disabledButtons = [ToolbarConfig.pasteButton];
  let enabledButtons = [
    ToolbarConfig.createFolderButton,
    ToolbarConfig.copyButton,
    ToolbarConfig.cutButton,
    ToolbarConfig.deleteButton,
    ToolbarConfig.downloadButton,
    ToolbarConfig.renameButton,
    ToolbarConfig.printButton
  ];
  beforeEach(() => {
    wrapper = mount(Toolbar, {
      computed: {
        selectedItems() {
          return [oneFile];
        },
        storedItems() {
          return [];
        },
        isDavProtocolSupported() {
          return true;
        }
      },
      store,
      localVue,
      stubs: stubs
    });
  });

  test.each(disabledButtons)("is disabled %# button", ({ id }) => {
    expect(wrapper.find("#" + id).attributes("disabled")).toBe("disabled");
  });

  test.each(enabledButtons)("is enabled %# button", ({ id }) => {
    expect(wrapper.find("#" + id).attributes("disabled")).toBeUndefined();
  });
});

describe("toolbar with 1 selected folder", () => {
  let wrapper;
  let disabledButtons = [
    ToolbarConfig.pasteButton,
    ToolbarConfig.printButton,
    ToolbarConfig.downloadButton
  ];
  let enabledButtons = [
    ToolbarConfig.createFolderButton,
    ToolbarConfig.copyButton,
    ToolbarConfig.cutButton,
    ToolbarConfig.deleteButton,
    ToolbarConfig.renameButton
  ];
  beforeEach(() => {
    wrapper = mount(Toolbar, {
      computed: {
        selectedItems() {
          return [oneFolder];
        },
        storedItems() {
          return [];
        },
        isDavProtocolSupported() {
          return true;
        }
      },
      store,
      localVue,
      stubs: stubs
    });
  });

  test.each(disabledButtons)("is disabled %# button", ({ id }) => {
    expect(wrapper.find("#" + id).attributes("disabled")).toBe("disabled");
  });

  test.each(enabledButtons)("is enabled %# button", ({ id }) => {
    expect(wrapper.find("#" + id).attributes("disabled")).toBeUndefined();
  });
});
