//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (Params.Hosting != HostingEnvironment.IIS)  return string.Empty; #>
using System;
using System.Collections.Generic;
using System.Configuration;
using System.IO;
//!<# if (Params.ACL && Params.Storage == StorageType.Ntfs) { #>
using System.Security.Principal;
//!<#}#>
using System.Web;
using System.Xml;
using System.Xml.Serialization;
using System.Threading.Tasks;

using ITHit.Server;
using ITHit.WebDAV.Server;
//!<# if (Params.ACL && Params.Storage == StorageType.Ntfs) { #>
using ITHit.WebDav.Server.SampleModel.__NtfsStorage__;
using ITHit.WebDav.Server.SampleModel.__NtfsStorage__.Acl;
//!<#}#>
//!<# if (Params.GSuite) { #>
using ITHit.GSuite.Server;
//!<#}#>

namespace ITHit.WebDav.Server.SampleModel
{
    //$<IHttpHandler
    /// <summary>
    /// This handler processes WebDAV requests.
    /// </summary>
    public class DavHandler :/*!<# if (Params.TargetFramework45OrLater) { #>!*/ HttpTaskAsyncHandler/*!<# }else{ #>!*/ IHttpHandler/*!<# } #>!*/
    {
        /// <summary>
        /// This license file is used to activate:
        ///  - IT Hit WebDAV Server Engine for .NET
        ///  - IT Hit iCalendar and vCard Library if used in a project
        /// </summary>
        //!<# if (!string.IsNullOrEmpty(Params.License)) { #>
        //!        private static string license = @"<#=Params.License == null ? null : Params.License.Replace("\"", "\"\"")#>";
        //!<# } else { #>
        private readonly string license = File.ReadAllText(HttpContext.Current.Request.PhysicalApplicationPath + "License.lic");
        //!<# } #>

        //!<# if (Params.GSuite) { #>
        /// <summary>
        /// Google Service Account ID (client_email field from JSON file).
        /// </summary>
        private static readonly string googleServiceAccountID = ConfigurationManager.AppSettings["GoogleServiceAccountID"];

        /// <summary>
        /// Google Service private key (private_key field from JSON file).
        /// </summary>
        private static readonly string googleServicePrivateKey = ConfigurationManager.AppSettings["GoogleServicePrivateKey"];

        /// <summary>
        /// Relative Url of "Webhook" callback. It handles the API notification messages that are triggered when a resource changes.
        /// </summary>
        private static readonly string googleNotificationsRelativeUrl = ConfigurationManager.AppSettings["GoogleNotificationsRelativeUrl"];

        /// <summary>
        /// This license file is used to activate G Suite Documents Editing for IT Hit WebDAV Server
        /// </summary>
        //!<# if (!string.IsNullOrEmpty(Params.License)) { #>
        //!        private static string gSuiteLicense = @"<#=Params.License == null ? null : Params.License.Replace("\"", "\"\"")#>";
        //!<# } else { #>
        private readonly string gSuiteLicense = File.Exists(HttpContext.Current.Request.PhysicalApplicationPath + "GSuiteLicense.lic") ? File.ReadAllText(HttpContext.Current.Request.PhysicalApplicationPath + "GSuiteLicense.lic") : string.Empty;
        //!<# } #>
        //!<#}#>

        /// <summary>
        /// If debug logging is enabled reponses are output as formatted XML,
        /// all requests and response headers and most bodies are logged.
        /// If debug logging is disabled only errors are logged.
        /// </summary>
        private static readonly bool debugLoggingEnabled =
            "true".Equals(
                ConfigurationManager.AppSettings["DebugLoggingEnabled"],
                StringComparison.InvariantCultureIgnoreCase);
        //!<# if (false && Params.ACL && Params.Storage == StorageType.Ntfs) { #>

        /// <summary>
        /// Domain(or computer name) agains which to perform authentication.
        /// </summary>
        private static readonly string domain = ConfigurationManager.AppSettings["Domain"];

        /// <summary>
        /// Name of user to be used with anonymous access to webdav.
        /// </summary>
        private static readonly string anonymousUserLogin = ConfigurationManager.AppSettings["AnonymousUserLogin"];

        /// <summary>
        /// Password for <see cref="anonymousUserLogin"/>.
        /// </summary>
        private static readonly string anonymousUserPassword = 
            ConfigurationManager.AppSettings["AnonymousUserPassword"];

        /// <summary>
        /// <see cref="WindowsIdentity"/> which corresponds to the anonymous user.
        /// </summary>
        private WindowsIdentity anonymousUser;
        //!<#}#>
        //!<# if (Params.DeltaV) { #>

        private static readonly bool autoPutUnderVersionControl =
            Convert.ToBoolean(ConfigurationManager.AppSettings["AutoPutUnderVersionControl"]);
        //!<#}#>
        //!<# if (Params.ACL && Params.Storage == StorageType.Ntfs) { #>

        /// <summary>
        /// Initializes serialization assemblies.
        /// </summary>
        static DavHandler()
        {
            //create serialization assemblies here.
            //Otherwise it fails with permissions error on first access in impersonated context.
            await using (MemoryStream ms = new MemoryStream())
            {
                //!<# if (Params.Class2) { #>
                new XmlSerializer(typeof(List<DateLockInfo>)).Serialize(ms, new List<DateLockInfo>());
                //!<# } #>
                new XmlSerializer(typeof(List<PropertyValue>)).Serialize(ms, new List<PropertyValue>());
                new XmlSerializer(typeof(long)).Serialize(ms, 0L);
                new XmlSerializer(typeof(int)).Serialize(ms, 0);
            }
        }
        //!<# } #>

        /// <summary>
        /// Gets a value indicating whether another request can use the
        /// <see cref="T:System.Web.IHttpHandler"/> instance.
        /// </summary>
        /// <returns>
        /// true if the <see cref="T:System.Web.IHttpHandler"/> instance is reusable; otherwise, false.
        /// </returns>
        //!<# if (!Params.TargetFramework45OrLater) { #>
        public bool IsReusable
        {
            get { return true; }
        }
        //!<#}else{#>
        public override bool IsReusable
        {
            get { return true; }
        }
        //!<#}#>
      

        /// <summary>
        /// Enables processing of HTTP Web requests.
        /// </summary>
        /// <param name="context">An <see cref="T:System.Web.HttpContext"/> object that provides references to the
        /// intrinsic server objects (for example, Request, Response, Session, and Server) used to service
        /// HTTP requests. 
        /// </param>
        //!<# if (!Params.TargetFramework45OrLater) { #>
        public void ProcessRequest(HttpContext context)
        //!<#}else{#>
        public override async Task ProcessRequestAsync(HttpContext context)
        //!<#}#>
        {
            DavEngineAsync webDavEngine = getOrInitializeWebDavEngine(context);

            context.Response.BufferOutput = false;

            //!<# if (Params.Storage == StorageType.Ntfs) { #>
            DavContext ntfsDavContext = new __NtfsStorage__.DavContext(context);
            //!<#}#>
            //!<# if (false && Params.Storage == StorageType.Ntfs && Params.ACL) { #>

            ntfsDavContext.AnonymousUser = anonymousUser;
            webDavEngine.Logger.LogDebug(
                string.Format(
                "Logged in as:{0}, anonymous user is:{1}",
                ntfsDavContext.WindowsIdentity.Name, anonymousUser == null ? "null" : anonymousUser.Name));
            //!<#}#>
            //!<# if (Params.GSuite) { #>
            GSuiteEngineAsync gSuiteEngine = getOrInitializeGSuiteEngine(context);

            //!<#}#>
            //!<# if (Params.Storage == StorageType.Ntfs) { #>
            await webDavEngine.RunAsync(ntfsDavContext);
            //!<# if (Params.GSuite) { #>
            if (gSuiteEngine != null)
            {
                await gSuiteEngine.RunAsync(ContextConverter.ConvertToGSuiteContext(ntfsDavContext));
            }
            //!<#}#>
            //!<#}#>

            //!<# if (Params.Storage == StorageType.Db) { #>

            using (var sqlDavContext = new __SqlStorage__.DavContext(context))
            {
                await webDavEngine.RunAsync(sqlDavContext);
                //!<# if (Params.GSuite) { #>
                if (gSuiteEngine != null)
                {
                    await gSuiteEngine.RunAsync(ContextConverter.ConvertToGSuiteContext(sqlDavContext));
                }
                //!<#}#>
            }
            //!<#}#>

            //!<# if (Params.Storage == StorageType.Blank) { #>
            var davContext = new __DavImpl__.DavContext(context);
            await webDavEngine.RunAsync(davContext);
            //!<# if (Params.GSuite) { #>
            if (gSuiteEngine != null)
            {
                await gSuiteEngine.RunAsync(ContextConverter.ConvertToGSuiteContext(davContext));
            }
            //!<#}#>
            //!<#}#>
        }

        /// <summary>
        /// Initializes engine.
        /// </summary>
        /// <param name="context">Instance of <see cref="HttpContext"/>.</param>
        /// <returns>Initialized <see cref="DavEngine"/>.</returns>
        private DavEngineAsync initializeWebDavEngine(HttpContext context)
        {
            //!<# if (false && Params.ACL && Params.Storage == StorageType.Ntfs) { #>
            try
            {
                anonymousUser = LogonUtil.GetUser(anonymousUserLogin, domain, anonymousUserPassword);
            }
            catch (Exception ex)
            {
                Logger.Instance.LogError("Failed to retrieve anonymous user identity", ex);
                throw;
            }
            //!<#}#>

            ILogger logger = ITHit.WebDav.Server.SampleModel.Logger.Instance;
            //!<# if (Params.CalDav || Params.CardDav) { #>
            logger.LogFlags = LogFlagsEnum.LogGetResponseBody | LogFlagsEnum.LogPutRequestBody;
            //!<# } #>
            DavEngineAsync webDavEngine = new DavEngineAsync
            {
                Logger = logger

                // Use idented responses if debug logging is enabled.
                , OutputXmlFormatting = true
                //!<# if (Params.DeltaV) { #>

                , AutoPutUnderVersionControl = autoPutUnderVersionControl
                //!<# } #>
                //!<# if (Params.CalDav || Params.CardDav) { #>

                // Do not emit CORS headers, typically not required in case of CalDAV/CardDAV server.
                , CorsAllowedFor = null

                // Use full URIs only in case of a regular WebDAV server. Microsoft Mini-redirector does not support URIs without domain.
                , UseFullUris = false 
                //!<# } #>
            };

            webDavEngine.License = license;
            //!<# if (Params.AddCustomGetHandler) { #>

            //!<# if (Params.Hosting == HostingEnvironment.IIS) { #>
            string contentRootPath = HttpContext.Current.Request.MapPath("/");
            //!<# } #>

            // Set custom handler to process GET and HEAD requests to folders and display 
            // info about how to connect to server. We are using the same custom handler 
            // class (but different instances) here to process both GET and HEAD because 
            // these requests are very similar. Some WebDAV clients may fail to connect if HEAD 
            // request is not processed.
            MyCustomGetHandler handlerGet  = new MyCustomGetHandler(contentRootPath);
            MyCustomGetHandler handlerHead = new MyCustomGetHandler(contentRootPath);
            handlerGet.OriginalHandler  = webDavEngine.RegisterMethodHandler("GET",  handlerGet);
            handlerHead.OriginalHandler = webDavEngine.RegisterMethodHandler("HEAD", handlerHead);
            //!<# } #>
            //!<# if (Params.CalDav || Params.CardDav) { #>

            // Set your iCalendar & vCard library license before calling any members.
            // iCalendar & vCard library accepts:
            // - WebDAV Server Engine license with iCalendar & vCard modules. Verify your license file to see if these modules are specified.
            // - or iCalendar and vCard Library license.
            ITHit.Collab.LicenseValidator.SetLicense(license);
            //!<# } #>

            return webDavEngine;
        }

        /// <summary>
        /// Initializes or gets engine singleton.
        /// </summary>
        /// <param name="context">Instance of <see cref="HttpContext"/>.</param>
        /// <returns>Instance of <see cref="DavEngineAsync"/>.</returns>
        private DavEngineAsync getOrInitializeWebDavEngine(HttpContext context)
        {
            //we don't use any double check lock pattern here because nothing wrong
            //is going to happen if we created occasionally several engines.
            const string ENGINE_KEY = "$DavEngine$";
            if (context.Application[ENGINE_KEY] == null)
            {
                context.Application[ENGINE_KEY] = initializeWebDavEngine(context);
            }

            return (DavEngineAsync)context.Application[ENGINE_KEY];
        }

        //!<# if (Params.GSuite) { #>
        /// <summary>
        /// Initializes or gets engine singleton.
        /// </summary>
        /// <param name="context">Instance of <see cref="HttpContext"/>.</param>
        /// <returns>Instance of <see cref="GSuiteEngineAsync"/>.</returns>
        private GSuiteEngineAsync getOrInitializeGSuiteEngine(HttpContext context)
        {
            //we don't use any double check lock pattern here because nothing wrong
            //is going to happen if we created occasionally several engines.
            const string ENGINE_KEY = "$GSuiteEngine$";
            if (string.IsNullOrEmpty(googleServiceAccountID) || string.IsNullOrEmpty(googleServicePrivateKey))
            {
                return null;
            }

            if (context.Application[ENGINE_KEY] == null)
            {
                var gSuiteEngine = new GSuiteEngineAsync(googleServiceAccountID, googleServicePrivateKey, googleNotificationsRelativeUrl)
                {
                    License = gSuiteLicense, 
                    Logger = ITHit.WebDav.Server.SampleModel.Logger.Instance
                };

                context.Application[ENGINE_KEY] = gSuiteEngine;
            }

            return (GSuiteEngineAsync)context.Application[ENGINE_KEY];
        }
        //!<#}#>
    }
    //$>
}
