//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (!Params.AddCustomGetHandler)  return string.Empty; #>
using System;
using System.Collections.Generic;
using System.IO;
using System.Reflection;
using System.Linq;
using System.Text;
//!<# if (Params.Hosting == HostingEnvironment.IIS) { #>
using System.Web;
using System.Web.UI;
//!<# } else if (Params.Hosting == HostingEnvironment.NetCore) {#>
using Microsoft.AspNetCore.StaticFiles;
//!<# } else { #>
using Microsoft.Win32;
//!<# } #>
//!<# if (Params.CalDav || Params.CardDav) { #>
using System.Text.RegularExpressions;
//!<# } #>
using System.Threading.Tasks;

using ITHit.WebDAV.Server;
using ITHit.WebDAV.Server.Class1;
using ITHit.WebDAV.Server.Extensibility;
//!<# if (Params.CalDav) { #>
using ITHit.WebDAV.Server.CalDav;
//!<# } #>
//!<# if (Params.CardDav) { #>
using ITHit.WebDAV.Server.CardDav;
//!<# } #>
//!<# if (false) { #>
// only required to compile project at dev time
using ITHit.WebDav.Server.SampleModel.__NtfsStorage__;
//!<# } #>
using ITHit.Server.Extensibility;
using ITHit.Server;

namespace ITHit.WebDav.Server.SampleModel
{
    //$<IMethodHandler
    /// <summary>
    /// This handler processes GET and HEAD requests to folders returning custom HTML page.
    /// </summary>
    internal class MyCustomGetHandler : IMethodHandler<IHierarchyItem>
    {
        /// <summary>
        /// Handler for GET and HEAD request registered with the engine before registering this one.
        /// We call this default handler to handle GET and HEAD for files, because this handler
        /// only handles GET and HEAD for folders.
        /// </summary>
        public IMethodHandler<IHierarchyItem> OriginalHandler { get; set; }

        /// <summary>
        /// Gets a value indicating whether output shall be buffered to calculate content length.
        /// Don't buffer output to calculate content length.
        /// </summary>
        public bool EnableOutputBuffering
        {
            get { return false; }
        }

        /// <summary>
        /// Gets a value indicating whether engine shall log response data (even if debug logging is on).
        /// </summary>
        public bool EnableOutputDebugLogging
        {
            get { return false; }
        }

        /// <summary>
        /// Gets a value indicating whether the engine shall log request data.
        /// </summary>
        public bool EnableInputDebugLogging
        {
            get { return false; }
        }

        /// <summary>
        /// Path to the folder where HTML files are located.
        /// </summary>
        private readonly string htmlPath;

        /// <summary>
        /// Creates instance of this class.
        /// </summary>
        /// <param name="contentRootPathFolder">Path to the folder where HTML files are located.</param>
        public MyCustomGetHandler(string contentRootPathFolder)
        {
            this.htmlPath = contentRootPathFolder;
        }

        /// <summary>
        /// Handles GET and HEAD request.
        /// </summary>
        /// <param name="context">Instace of <see cref="ContextAsync{IHierarchyItem}"/>.</param>
        /// <param name="item">Instance of <see cref="IHierarchyItem"/> which was returned by
        /// <see cref="ContextAsync{IHierarchyItem}.GetHierarchyItemAsync"/> for this request.</param>
        public async Task ProcessRequestAsync(ContextAsync<IHierarchyItem> context, IHierarchyItem item)
        {
            string urlPath = context.Request.RawUrl.Substring(context.Request.ApplicationPath.TrimEnd('/').Length);

            if (item is IItemCollection)
            {
                // In case of GET requests to WebDAV folders we serve a web page to display 
                // any information about this server and how to use it.

                // Remember to call EnsureBeforeResponseWasCalledAsync here if your context implementation
                // makes some useful things in BeforeResponseAsync.
                await context.EnsureBeforeResponseWasCalledAsync();
                //!<# if (Params.CardDav || Params.CalDav) { #>

                // Request to iOS/OS X CalDAV/CardDAV profile.
                if (context.Request.RawUrl.EndsWith("?connect"))
                {
                    await WriteProfileAsync(context, item, htmlPath);
                    return;
                }
                //!<# } #>

                //!<# if (Params.Hosting == HostingEnvironment.IIS) { #>
                IHttpAsyncHandler page = (IHttpAsyncHandler)System.Web.Compilation.BuildManager.CreateInstanceFromVirtualPath(
                    "~/MyCustomHandlerPage.aspx", typeof(MyCustomHandlerPage));
                //!<# if (Params.TargetFramework45OrLater) { #>

                if(Type.GetType("Mono.Runtime") != null)
                {
                    page.ProcessRequest(HttpContext.Current);
                }
                else
                {
                    // Here we call BeginProcessRequest instead of ProcessRequest to start an async page execution and be able to call RegisterAsyncTask if required. 
                    // To call APM method (Begin/End) from TAP method (Task/async/await) the Task.FromAsync must be used.
                    await Task.Factory.FromAsync(page.BeginProcessRequest, page.EndProcessRequest, HttpContext.Current, null);
                }
                //!<# } else { #>
                page.ProcessRequest(HttpContext.Current);
                //!<# } #>
                //!<# } #>
                //!<# if ((Params.Hosting == HostingEnvironment.Listener) || (Params.Hosting == HostingEnvironment.NetCore)) { #>

                string htmlName = "MyCustomHandlerPage.html";
                using (TextReader reader = File.OpenText(Path.Combine(htmlPath, htmlName)))
                {
                    string html = await reader.ReadToEndAsync();
                    //!<# if (Params.CalDav || Params.CardDav) { #>
                    html = html.Replace("_webDavServerUrl_", context.Request.UrlPrefix + context.Request.ApplicationPath);
                    //!<# } #>
                    //!<# if (Params.CalDav) { #>
                    html = html.Replace("_CALENDARS_", await AllUserCalendarsUrlAsync(context));
                    //!<# } #>
                    //!<# if (Params.CardDav) { #>
                    html = html.Replace("_BOOKS_", await AllUserAddressbooksUrlAsync(context));
                    //!<# } #>
                    //!<# if (string.IsNullOrEmpty(Params.DavUrlPrefix)) { #>
                    html = html.Replace("_webDavServerRoot_", context.Request.ApplicationPath.TrimEnd('/'));
                    //!<# } #>
                    //!<# if (!string.IsNullOrEmpty(Params.DavUrlPrefix)) { #>
                    html = html.Replace("_webDavServerRoot_", "");
                    //!<# } #>
                    html = html.Replace("_webDavServerVersion_",
                        typeof(DavEngineAsync).GetTypeInfo().Assembly.GetName().Version.ToString());
                    //!<# if (Params.Hosting == HostingEnvironment.NetCore && Params.AuthenticationType.HasFlag(AuthenticationType.Cookies)) { #>

                    // Set list of cookie names for ajax lib.
                    if (context.Request.Headers.ContainsKey("Cookie"))
                    {
                        html = html.Replace("_webDavAuthCookieNames_", string.Join(",", context.Request.Headers["Cookie"]
                                    .TrimEnd(';').Split(';').Select(p => p.Split(new[] { '=' }, 2)[0].Trim())
                                    .Where(p => p.StartsWith(".AspNetCore.Identity.Application") || p.StartsWith(".AspNetCore.Cookies") || p.StartsWith(".AspNetCore.AzureADCookie"))));
                    }
                    //!<# } #>

                    await WriteFileContentAsync(context, html, htmlName);
                }
                //!<# } #>
            }
            //!<# if (Params.Hosting == HostingEnvironment.Listener) { #>
            else if (urlPath.StartsWith("/AjaxFileBrowser/") || urlPath.StartsWith("/wwwroot/"))
            {
                // The "/AjaxFileBrowser/" are not a WebDAV folders. They can be used to store client script files, 
                // images, static HTML files or any other files that does not require access via WebDAV.
                // Any request to the files in this folder will just serve them to the client. 

                await context.EnsureBeforeResponseWasCalledAsync();
                string filePath = Path.Combine(htmlPath, urlPath.TrimStart('/').Replace('/', Path.DirectorySeparatorChar));

                // Remove query string.
                int queryIndex = filePath.LastIndexOf('?');
                if (queryIndex > -1)
                {
                    filePath = filePath.Remove(queryIndex);
                }

                if (!File.Exists(filePath))
                {
                    throw new DavException("File not found: " + filePath, DavStatus.NOT_FOUND);
                }

                Encoding encoding = context.Engine.ContentEncoding; // UTF-8 by default
                context.Response.ContentType = string.Format("{0}; charset={1}", MimeType.GetMimeType(Path.GetExtension(filePath)) ?? "application/octet-stream", encoding.WebName);

                // Return file content in case of GET request, in case of HEAD just return headers.
                if (context.Request.HttpMethod == "GET")
                {
                    await using (FileStream fileStream = File.OpenRead(filePath))
                    {
                        context.Response.ContentLength = fileStream.Length;
                        await fileStream.CopyToAsync(context.Response.OutputStream);
                    }
                }
            }
            //!<# } #>
            else
            {
                await OriginalHandler.ProcessRequestAsync(context, item);
            }
        }
        //!<# if ((Params.Hosting == HostingEnvironment.Listener) || (Params.Hosting == HostingEnvironment.NetCore)) { #>

        /// <summary>
        /// Writes HTML to the output stream in case of GET request using encoding specified in Engine. 
        /// Writes headers only in case of HEAD request.
        /// </summary>
        /// <param name="context">Instace of <see cref="ContextAsync{IHierarchyItem}"/>.</param>
        /// <param name="content">String representation of the content to write.</param>
        /// <param name="filePath">Relative file path, which holds the content.</param>
        private async Task WriteFileContentAsync(ContextAsync<IHierarchyItem> context, string content, string filePath)
        {
            Encoding encoding = context.Engine.ContentEncoding; // UTF-8 by default
            context.Response.ContentLength = encoding.GetByteCount(content);     
            context.Response.ContentType = string.Format("{0}; charset={1}", MimeType.GetMimeType(Path.GetExtension(filePath)) ?? "application/octet-stream", encoding.WebName);

            // Return file content in case of GET request, in case of HEAD just return headers.
            if (context.Request.HttpMethod == "GET")
            {               
                await using (var writer = new StreamWriter(context.Response.OutputStream, encoding))
                {
                    await writer.WriteAsync(content);
                }
            }
        }
        //!<# } #>

        /// <summary>
        /// This handler shall only be invoked for <see cref="IFolder"/> items or if original handler (which
        /// this handler substitutes) shall be called for the item.
        /// </summary>
        /// <param name="item">Instance of <see cref="IHierarchyItem"/> which was returned by
        /// <see cref="ContextAsync{IHierarchyItem}.GetHierarchyItemAsync"/> for this request.</param>
        /// <returns>Returns <c>true</c> if this handler can handler this item.</returns>
        public bool AppliesTo(IHierarchyItem item)
        {
            return item is IFolder || OriginalHandler.AppliesTo(item);
        }
        //!<# if (((Params.Hosting == HostingEnvironment.Listener) || (Params.Hosting == HostingEnvironment.NetCore)) && Params.CalDav) { #>

        /// <summary>
        /// Gets all user calendars URLs.
        /// </summary>
        private async Task<string> AllUserCalendarsUrlAsync(ContextAsync<IHierarchyItem> context)
        {
            Discovery discovery = new Discovery(context as DavContext);
            List<IHierarchyItem> items = new List<IHierarchyItem>();

            // get list of folders that contain user calendars and enumerate calendars in each folder
            foreach (IItemCollection folder in await discovery.GetCalendarHomeSetAsync())
            {
                IEnumerable<IHierarchyItem> children = (await folder.GetChildrenAsync(new PropertyName[0], null, null, null)).Page;
                items.AddRange(children.Where(x => x is ICalendarFolder));
            }
            IEnumerable<string> nameAndUrls = items.Select(x => string.Format("<tr><td><span class=\"glyphicon glyphicon - book\"></span></td><td>{0}</td><td>{1}</td><td><a href=\"{1}?connect\" class=\"btn btn-default\">Connect</a></td></tr>", x.Name, AddAppPath(context, x.Path)));
            return String.Join(string.Empty, nameAndUrls.ToArray());
        }
        //!<# } #>
        //!<# if (((Params.Hosting == HostingEnvironment.Listener) || (Params.Hosting == HostingEnvironment.NetCore)) && Params.CardDav) { #>

        /// <summary>
        /// Gets all user address books URLs.
        /// </summary>
        private async Task<string> AllUserAddressbooksUrlAsync(ContextAsync<IHierarchyItem> context)
        {
            Discovery discovery = new Discovery(context as DavContext);
            List<IHierarchyItem> items = new List<IHierarchyItem>();

            // get list of folders that contain user address books and enumerate address books in each folder
            foreach (IItemCollection folder in await discovery.GetAddressbookHomeSetAsync())
            {
                IEnumerable<IHierarchyItem> children = (await folder.GetChildrenAsync(new PropertyName[0], null, null, null)).Page;
                items.AddRange(children.Where(x => x is IAddressbookFolder));
            }
            IEnumerable<string> nameAndUrls = items.Select(x => string.Format("<tr><td><span class=\"glyphicon glyphicon - book\"></span></td><td>{0}</td><td>{1}</td><td><a href=\"{1}?connect\" class=\"btn btn-default\">Connect</a></td></tr>", x.Name, AddAppPath(context, x.Path)));
            return String.Join(string.Empty, nameAndUrls.ToArray());
        }
        //!<# } #>
        //!<# if (((Params.Hosting == HostingEnvironment.Listener) || (Params.Hosting == HostingEnvironment.NetCore)) && (Params.CalDav || Params.CardDav)) { #>

        private static string AddAppPath(ContextAsync<IHierarchyItem> context, string path)
        {
            string applicationPath = context.Request.UrlPrefix + context.Request.ApplicationPath;
            return string.Format("{0}/{1}", applicationPath.TrimEnd(new[] { '/' }), path.TrimStart(new[] { '/' }));
        }
        //!<# } #>
        //!<# if (Params.CardDav || Params.CalDav) { #>

        /// <summary>
        /// Writes iOS / OS X CalDAV/CardDAV profile.
        /// </summary>
        /// <param name="context">Instace of <see cref="ContextAsync{IHierarchyItem}"/>.</param>
        /// <param name="item">ICalendarFolder or IAddressbookFolder item.</param>
        /// <returns></returns>
        private async Task WriteProfileAsync(ContextAsync<IHierarchyItem> context, IHierarchyItemBase item, string htmlPath)
        {
            string mobileconfigFileName = null;
            string decription = null;
            //!<# if (Params.CalDav) { #>
            if (item is ICalendarFolder)
            {
                mobileconfigFileName = "CalDAV.AppleProfileTemplete.mobileconfig";
                decription = (item as ICalendarFolder).CalendarDescription;
            }
            //!<# } #>
            //!<# if (Params.CardDav) { #>
            if (item is IAddressbookFolder)
            {
                mobileconfigFileName = "CardDAV.AppleProfileTemplete.mobileconfig";
                decription = (item as IAddressbookFolder).AddressbookDescription;
            }
            //!<# } #>

            decription = !string.IsNullOrEmpty(decription) ? decription : item.Name;

            string templateContent = null;
            using (TextReader reader = new StreamReader(Path.Combine(htmlPath, mobileconfigFileName)))
            {
                templateContent = await reader.ReadToEndAsync();
            }

            Uri url = new Uri(context.Request.UrlPrefix);

            string payloadUUID = item.Path.Split(new[] { '/' }, StringSplitOptions.RemoveEmptyEntries).Last(); // PayloadUUID

            string profile = string.Format(templateContent
                , url.Host // host name
                , item.Path // CalDAV / CardDAV Principal URL. Here we can return (await (item as ICurrentUserPrincipal).GetCurrentUserPrincipalAsync()).Path if needed.
                            //!<# if (Params.Storage == StorageType.Ntfs) { #>
                , (context as DavContext).UserName // user name
                //!<# } else {#>
                , (context as DavContext).Identity.Name // user name
                //!<# } #>
                , url.Port // port                
                , (url.Scheme == "https").ToString().ToLower() // SSL
                , decription // CardDAV / CardDAV Account Description
                , Assembly.GetAssembly(this.GetType()).GetName().Version.ToString()
                , Assembly.GetAssembly(typeof(DavEngineAsync)).GetName().Version.ToString()
                , payloadUUID
                );

            byte[] profileBytes = SignProfile(context, profile);

            context.Response.ContentType = "application/x-apple-aspen-config";
            context.Response.AddHeader("Content-Disposition", "attachment; filename=profile.mobileconfig");
            context.Response.ContentLength = profileBytes.Length;
            await context.Response.OutputStream.WriteAsync(profileBytes, 0, profileBytes.Length);
        }

        /// <summary>
        /// Signs iOS / OS X payload profile with SSL certificate.
        /// </summary>
        /// <param name="context">Instace of <see cref="ContextAsync{IHierarchyItem}"/>.</param>
        /// <param name="profile">Profile to sign.</param>
        /// <returns>Signed profile.</returns>
        private byte[] SignProfile(ContextAsync<IHierarchyItem> context, string profile)
        {
            // Here you will sign your profile with SSL certificate to avoid "Unsigned" warning on iOS and OS X.
            // For demo purposes we just return the profile content unmodified.
            return context.Engine.ContentEncoding.GetBytes(profile);
        }
        //!<# } #>
    }
    //$>
}