//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (Params.Hosting != HostingEnvironment.Listener)  return string.Empty; #>
using System;
using System.Configuration;
using System.IO;
//!<# if (Params.Storage == StorageType.Db) {#>
using System.Data.SqlClient;
//!<# } #>
//!<# if (Params.DASL) { #>
using System.Linq;
//!<# } #>
using System.Net;
using System.Reflection;
using System.Security.Principal;
//!<# if (Params.RunAsService) { #>
using System.ServiceProcess;
//!<# } #>
//!<# if (Params.AuthenticationType.HasFlag(AuthenticationType.Basic) || Params.AuthenticationType.HasFlag(AuthenticationType.Digest)) { #>
using System.Web.Security;
//!<# } #>
using System.Text;
using System.Xml;
//!<# if (Params.DeltaV) {#>
using System.Collections.Generic;
//!<# } #>
using System.Threading.Tasks;

//!<# if (Params.DeltaV) {#>
using ITHit.WebDAV.Server.DeltaV;
//!<# } #>
using ITHit.WebDAV.Server.Logger;
using ITHit.WebDAV.Server;
using ITHit.WebDav.Server.SampleModel.__NtfsStorage__;
//!<# if (Params.WebSockets) {#>
using System.Net.WebSockets;
using System.Threading;
//!<# } #>
//!<# if (Params.Storage == StorageType.Ntfs) { #>
using ITHit.WebDav.Server.SampleModel.__NtfsStorage__.ExtendedAttributes;
//!<# } #>

using ITHit.Server;
//!<# if (Params.GSuite) { #>
using ITHit.GSuite.Server;
//!<# } #>

namespace ITHit.WebDav.Server.SampleModel
{
    /// <summary>
    /// WebDAV engine host.
    /// </summary>
    internal class Program
    {
        public static bool Listening { get; set; }
        //!<# if (Params.AuthenticationType.HasFlag(AuthenticationType.Digest)) { #>

        private static readonly __Authentication__.DigestAuthenticationProvider digestProvider =
            new __Authentication__.DigestAuthenticationProvider(GetPasswordAndRoles);
        //!<#}#>

        private static DavEngineAsync webDavEngine;
        //!<# if (Params.GSuite) { #>

        private static GSuiteEngineAsync gSuiteEngine;

        /// <summary>
        /// Google Service Account ID (client_email field from JSON file).
        /// </summary>
        private static readonly string googleServiceAccountID = ConfigurationManager.AppSettings["GoogleServiceAccountID"];

        /// <summary>
        /// Google Service private key (private_key field from JSON file).
        /// </summary>
        private static readonly string googleServicePrivateKey = ConfigurationManager.AppSettings["GoogleServicePrivateKey"];

        /// <summary>
        /// Relative Url of "Webhook" callback. It handles the API notification messages that are triggered when a resource changes.
        /// </summary>
        private static readonly string googleNotificationsRelativeUrl = ConfigurationManager.AppSettings["GoogleNotificationsRelativeUrl"];

        //!<# } #>
        //!<# if (Params.Storage == StorageType.Ntfs) { #>

        private static readonly string repositoryPath =
            ConfigurationManager.AppSettings["RepositoryPath"].TrimEnd(Path.DirectorySeparatorChar);
        //!<#}#>
        //!<# if (false && Params.ACL && Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm)) { #>

        private static readonly string domain =
            ConfigurationManager.AppSettings["Domain"];

        private static readonly string anonymousUserLogin =
            ConfigurationManager.AppSettings["AnonymousUserLogin"];

        private static readonly string anonymousUserPassword =
            ConfigurationManager.AppSettings["AnonymousUserPassword"];

        private static WindowsIdentity anonymousUser;
        //!<#}#>
        //!<# if (Params.DeltaV) { #>

        private static readonly bool autoPutUnderVersionControl =
            Convert.ToBoolean(ConfigurationManager.AppSettings["AutoPutUnderVersionControl"]);
        //!<#}#>

        /// <summary>
        /// Whether requests/responses shall be logged.
        /// </summary>
        private static readonly bool debugLoggingEnabled =
            ConfigurationManager.AppSettings["DebugLoggingEnabled"].Equals(
                "true",
                StringComparison.InvariantCultureIgnoreCase);

        /// <summary>
        /// Logger instance.
        /// </summary>
        private static readonly DefaultLoggerImpl logger = new DefaultLoggerImpl();
        //!<# if (Params.RunAsService && Params.RunAsConsole) { #>

        /// <summary>
        /// Gets a value indicating whether the program is runing as a Windows service or standalone application.
        /// </summary>
        private static bool IsServiceMode
        {
            get { return !Environment.UserInteractive; }
        }
        //!<#}#>

        /// <summary>
        /// Entry point.
        /// </summary>
        /// <param name="args">Command line arguments.</param>
        public static void Main(string[] args)
        {
            try
            {
                CheckConfigErrors();
                Init();

                //!<# if (Params.RunAsService && Params.RunAsConsole) { #>
                if (IsServiceMode)
                {
                //!<#}#>
                    //!<# if (Params.RunAsService) { #>
                    Listening = false;
                    ServiceBase.Run(new Service());
                    //!<#}#>
                //!<# if (Params.RunAsService && Params.RunAsConsole) { #>
                }
                else
                {
                //!<#}#>
                    //!<# if (Params.RunAsConsole) { #>
                    Listening = true;
                    //!<# if (!Params.TargetFramework45OrLater) { #>                    
                    ThreadProc();
                    //!<#} else {#>
                    ThreadProcAsync();
                    Console.ReadKey();
                    //!<#}#>
                    //!<#}#>
                //!<# if (Params.RunAsService && Params.RunAsConsole) { #>
                }
                //!<#}#>
            }
            catch (Exception ex)
            {
                //!<# if (Params.RunAsService && Params.RunAsConsole) { #>
                if (IsServiceMode)
                {
                //!<#}#>
                    //!<# if (Params.RunAsService) { #>
                    string errorMessage = "Could not start service" + Environment.NewLine + " " + ex.Message;
                    logger.LogError(errorMessage, null);
                    //!<#}#>
                //!<# if (Params.RunAsService && Params.RunAsConsole) { #>
                }
                else
                {
                //!<#}#>
                    //!<# if (Params.RunAsConsole) { #>
                    Console.WriteLine(ex.Message);
                    Console.WriteLine(Environment.NewLine + "Press any key...");
                    Console.ReadKey();
                    //!<#}#>
                //!<# if (Params.RunAsService && Params.RunAsConsole) { #>
                }
                //!<#}#>
            }
        }

        static void Init()
        {
            string contentRootPath = Directory.GetParent(Path.GetDirectoryName(new Uri(Assembly.GetExecutingAssembly().EscapedCodeBase).LocalPath)).FullName;
            string logPath = Path.Combine(contentRootPath, @"App_Data\WebDav\Logs");
            logger.LogFile = Path.Combine(logPath, "WebDAVlog.txt");
            logger.IsDebugEnabled = debugLoggingEnabled;
            //!<# if (Params.CalDav || Params.CardDav) { #>
            logger.LogFlags = LogFlagsEnum.LogGetResponseBody | LogFlagsEnum.LogPutRequestBody;
            //!<# } #>

            webDavEngine = new DavEngineAsync
            {
                Logger = logger

                // Use idented responses if debug logging is enabled.
                , OutputXmlFormatting = true
                //!<# if (Params.DeltaV) { #>

                , AutoPutUnderVersionControl = autoPutUnderVersionControl
                //!<# } #>
                //!<# if (Params.CalDav || Params.CardDav) { #>

                // Do not emit CORS headers, typically not required in case of CalDAV/CardDAV server.
                , CorsAllowedFor = null

                // Use full URIs only in case of a regular WebDAV server. Microsoft Mini-redirector does not support URIs without domain.
                , UseFullUris = false 
                //!<# } #>
            };

            /// This license lile is used to activate:
            ///  - IT Hit WebDAV Server Engine for .NET
            ///  - IT Hit iCalendar and vCard Library if used in a project
            //!<# if (!string.IsNullOrEmpty(Params.License)) { #>
            //!            string license = @"<#=Params.License == null ? null : Params.License.Replace("\"", "\"\"")#>";
            //!<# } else { #>
            string license = File.ReadAllText(Path.Combine(contentRootPath, "License.lic"));
            //!<# } #>

            webDavEngine.License = license;
            //!<# if (Params.GSuite) { #>

            /// This license file is used to activate G Suite Documents Editing for IT Hit WebDAV Server
            //!<# if (!string.IsNullOrEmpty(Params.License)) { #>
            //!        string gSuiteLicense = @"<#=Params.License == null ? null : Params.License.Replace("\"", "\"\"")#>";
            //!<# } else { #>
            string gSuiteLicense = File.Exists(Path.Combine(contentRootPath, "GSuiteLicense.lic")) ? File.ReadAllText(Path.Combine(contentRootPath,"GSuiteLicense.lic")) : string.Empty;
            //!<# } #>
            if (!string.IsNullOrEmpty(gSuiteLicense))
            {
                gSuiteEngine = new GSuiteEngineAsync(googleServiceAccountID, googleServicePrivateKey, googleNotificationsRelativeUrl)
                {
                    License = gSuiteLicense,
                    Logger = logger
                };
            }

            //!<# } #>
            //!<# if (Params.AddCustomGetHandler) { #>

            // Set custom handler to process GET and HEAD requests to folders and display 
            // info about how to connect to server. We are using the same custom handler 
            // class (but different instances) here to process both GET and HEAD because 
            // these requests are very similar. Some WebDAV clients may fail to connect if HEAD 
            // request is not processed.
            MyCustomGetHandler handlerGet  = new MyCustomGetHandler(contentRootPath);
            MyCustomGetHandler handlerHead = new MyCustomGetHandler(contentRootPath);
            handlerGet.OriginalHandler  = webDavEngine.RegisterMethodHandler("GET",  handlerGet);
            handlerHead.OriginalHandler = webDavEngine.RegisterMethodHandler("HEAD", handlerHead);
            //!<# } #>
            //!<# if (Params.CalDav || Params.CardDav) { #>

            // Set your iCalendar & vCard library license before calling any members.
            // iCalendar & vCard library accepts:
            // - WebDAV Server Engine license with iCalendar & vCard modules. Verify your license file to see if these modules are specified.
            // - or iCalendar and vCard Library license.
            ITHit.Collab.LicenseValidator.SetLicense(license);
            //!<# } #>

            //!<# if (Params.Storage == StorageType.Ntfs) { #>
            string attrStoragePath = (ConfigurationManager.AppSettings["AttrStoragePath"] ?? string.Empty).TrimEnd(Path.DirectorySeparatorChar);

            if (!string.IsNullOrEmpty(attrStoragePath))
            {
                FileSystemInfoExtension.UseFileSystemAttribute(new FileSystemExtendedAttribute(attrStoragePath, repositoryPath));
            }
            //!<# } #>
        }

        public static async void ThreadProcAsync()
        {
            string uriPrefix = ConfigurationManager.AppSettings["ListenerPrefix"];
            using (System.Net.HttpListener listener = new System.Net.HttpListener())
            {
                listener.Prefixes.Add(uriPrefix);
                //!<# if (Params.AuthenticationType.HasFlag(AuthenticationType.None)) { #>

                listener.AuthenticationSchemes = AuthenticationSchemes.Anonymous;
                //!<#}#>
                //!<# if (Params.AuthenticationType.HasFlag(AuthenticationType.Basic)) { #>

                // We do not use AuthenticationSchemes.Basic here because OPTIONS request must be processed without authentication. 
                // Instead this sample provides its own Basic authentication implementation.
                // IMPORTANT: Microsoft Office requires SSL with Basic authentication, it would not open documents for editing with non-SSL connection.
                listener.AuthenticationSchemes = AuthenticationSchemes.Anonymous;
                //!<#}#>
                //!<# if (Params.AuthenticationType.HasFlag(AuthenticationType.Digest)) { #>

                // We do not use AuthenticationSchemes.Digest here because OPTIONS request must be processed without authentication. 
                // Instead this sample provides its own Digest authentication implementation.
                listener.AuthenticationSchemes = AuthenticationSchemes.Anonymous;
                //!<#}#>
                //!<# if (Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm)) { #>

                listener.AuthenticationSchemes = AuthenticationSchemes.IntegratedWindowsAuthentication;
                listener.UnsafeConnectionNtlmAuthentication = true; // Required to avoid buffering on a client side in case NTLM is used. Otherwise "This request requires buffering data to succeed" exception may be thrown on a client side. Also required to support upload in web browsers when NTLM is used.
                //!<#}#>

                listener.IgnoreWriteExceptions = true;
                //!<# if (Params.RunAsConsole) { #>

                // For the sake of the development convenience, this code opens default web browser
                // with this server url when project is started in the debug mode as a console app.
#if DEBUG
                //!<# if (Params.RunAsService) { #>
                if (!IsServiceMode)
                {
                    System.Diagnostics.Process.Start(uriPrefix.Replace("+", "localhost"), null);
                }
                //!<# } else { #>
                System.Diagnostics.Process.Start(uriPrefix.Replace("+", "localhost"), null);
                //!<# } #>
#endif
                //!<# } #>


                listener.Start();

                Console.WriteLine("Start listening on " + uriPrefix);
                Console.WriteLine("Press Control-C to stop listener...");

                while (Listening)
                {
                    //!<# if (Params.TargetFramework45OrLater) { #>
                    HttpListenerContext context = await listener.GetContextAsync();
#pragma warning disable 4014
                    Task.Factory.StartNew(() => ProcessRequestAsync(listener, context));
#pragma warning restore 4014
                    //!<#} else {#>
                    IAsyncResult result = listener.BeginGetContext(ListenerCallback, listener);
                    result.AsyncWaitHandle.WaitOne();
                    //!<#}#>
                }
            }
        }

        //$<Listener.DavContextBase
        //!<# if (!Params.TargetFramework45OrLater) { #>
        public static void ListenerCallback(IAsyncResult result)
        {
            System.Net.HttpListener listener = (System.Net.HttpListener)result.AsyncState;
            HttpListenerContext context = listener.EndGetContext(result);
            ProcessRequest(listener, context);
        }
        //!<#}#>

        //!<# if (Params.WebSockets) { #>
        private static async Task ProcessWebSocketRequestAsync(HttpListenerContext context)
        {
            WebSocketsService socketService = WebSocketsService.Service;
            WebSocketContext webSocketContext = await context.AcceptWebSocketAsync(null);
            WebSocket client = webSocketContext.WebSocket;

            // Adding client to connected clients collection.
            Guid clientId = socketService.AddClient(client, context.Request.Headers["InstanceId"]);

            byte[] buffer = new byte[1024 * 4];
            WebSocketReceiveResult result = null;

            while (client.State == WebSocketState.Open)
            {
                try
                {
                    // Must receive client results.
                    result = await client.ReceiveAsync(new ArraySegment<byte>(buffer), CancellationToken.None);
                }
                catch (WebSocketException)
                {
                    break;
                }

                if (result.MessageType == WebSocketMessageType.Close)
                {
                    await client.CloseAsync(result.CloseStatus.Value, result.CloseStatusDescription, CancellationToken.None);
                }
            }

            // Remove client from connected clients dictionary after disconnecting.
            socketService.RemoveClient(clientId);
        }
        //!<# } #>

        private static async Task ProcessRequestAsync(System.Net.HttpListener listener, HttpListenerContext context)
        {
            try
            { 
                MacOsXPreprocessor.Process(context.Request); // fixes headers for Mac OS X v10.5.3 or later

                IPrincipal principal = null;
                //!<# if (Params.AuthenticationType.HasFlag(AuthenticationType.Basic) || Params.AuthenticationType.HasFlag(AuthenticationType.Digest)) { #>

                principal = AuthenticateRequest(context.Request);

                // OPTIONS request must be processed without authentication. 
                // Some clients including web browser preflight requests and Microsoft Mini-redirector 
                // does not attach Authorization header to OPTIONS request but require correct 
                // response to such request.
                if (!context.Request.HttpMethod.Equals("OPTIONS", StringComparison.InvariantCultureIgnoreCase))
                {
                    if (principal == null)
                    {
                        context.Response.StatusCode = 401;
                        ShowLoginDialog(context, context.Response);
                        return;
                    }
                }
                else
                {
                    if (principal == null)
                    {
                        principal = new GenericPrincipal(new GenericIdentity(""), null); // Empty user name sets IsAuthenticated = false
                    }
                }
                //!<#}#>
                //!<# if (Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm)) { #>
                principal = context.User;
                //!<#}#>

                //!<# if (Params.WebSockets) { #>
                if (context.Request.IsWebSocketRequest && context.Request.RawUrl.StartsWith("/dav"))
                {
                    // If current request is web socket request.
                    await ProcessWebSocketRequestAsync(context);
                    return;
                }
                //!<# } #>

                context.Response.SendChunked = false;
                //!<# if (Params.Storage == StorageType.Ntfs) { #>

                var ntfsDavContext =
                    new __NtfsStorage__.DavContext(context, listener.Prefixes, principal, repositoryPath, webDavEngine.Logger);

                if ((principal != null) && principal.Identity.IsAuthenticated)
                {
                    //!<# if (Params.CalDav) { #>
                    await Provisioning.CreateCalendarFoldersAsync(ntfsDavContext);
                    //!<#}#>
                    //!<# if (Params.CardDav) { #>
                    await Provisioning.CreateAddressbookFoldersAsync(ntfsDavContext);
                    //!<#}#>
                }

                //!<# if (false && Params.ACL && Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm)) { #>
                ntfsDavContext.AnonymousUser = anonymousUser;
                //!<#}#>
                await webDavEngine.RunAsync(ntfsDavContext);
                //!<# if (Params.GSuite) { #>
                if (gSuiteEngine != null)
                {
                    await gSuiteEngine.RunAsync(ContextConverter.ConvertToGSuiteContext(ntfsDavContext));
                }
                //!<# } #>
                //!<#}#>
                //!<# if (Params.Storage == StorageType.Db) { #>

                using (var sqlDavContext = new __SqlStorage__.DavContext(context, listener.Prefixes, principal, webDavEngine.Logger))
                {
                    //!<# if (Params.CalDav || Params.CardDav) { #>
                    if ((principal != null) && principal.Identity.IsAuthenticated)
                    {
                        //!<# if (Params.CalDav) { #>
                        await Provisioning.CreateCalendarFoldersAsync(sqlDavContext);
                        //!<#}#>
                        //!<# if (Params.CardDav) { #>
                        await Provisioning.CreateAddressbookFoldersAsync(sqlDavContext);
                        //!<#}#>
                    }
                    //!<#}#>
                    await webDavEngine.RunAsync(sqlDavContext);
                    //!<# if (Params.GSuite) { #>
                    if (gSuiteEngine != null)
                    {
                        await gSuiteEngine.RunAsync(ContextConverter.ConvertToGSuiteContext(sqlDavContext));
                    }
                    //!<# } #>
                }
                //!<#}#>
                //!<# if (Params.Storage == StorageType.Blank) { #>

                var davContext = new __DavImpl__.DavContext(context, listener.Prefixes, principal, webDavEngine.Logger);
                await webDavEngine.RunAsync(davContext);
                //!<# if (Params.GSuite) { #>
                if (gSuiteEngine != null)
                {
                    await gSuiteEngine.RunAsync(ContextConverter.ConvertToGSuiteContext(davContext));
                }
                //!<# } #>
                //!<#}#>
                //!<# if (!Params.AuthenticationType.HasFlag(AuthenticationType.None)) { #>

                if (context.Response.StatusCode == 401)
                {
                    ShowLoginDialog(context, context.Response);
                }
                //!<#}#>
            }
            finally
            {
                if (context != null && context.Response != null)
                {
                    try
                    {
                        context.Response.Close();
                    }
                    catch
                    {
                        // client closed connection before the content was sent
                    }
                }
            }
        }
        //$>

        //!<# if (Params.AuthenticationType.HasFlag(AuthenticationType.Digest)) { #>
        private static __Authentication__.DigestAuthenticationProvider.PasswordAndRoles GetPasswordAndRoles(
            string username)
        {
            if (Membership.GetUser(username) != null)
            {
                string password = Membership.Provider.GetPassword(username, null);
                return new __Authentication__.DigestAuthenticationProvider.PasswordAndRoles(password, new[] { "admin" });
            }

            return null;
        }
        //!<#}#>

        //!<# if (!Params.AuthenticationType.HasFlag(AuthenticationType.None)) { #>
        private static void ShowLoginDialog(HttpListenerContext context, HttpListenerResponse response)
        {
            //!<# if (Params.AuthenticationType.HasFlag(AuthenticationType.Basic)) { #>
            context.Response.AddHeader("WWW-Authenticate", "Basic Realm=\"My WebDAV Server\"");
            //!<#}#>
            //!<# if (Params.AuthenticationType.HasFlag(AuthenticationType.Digest)) { #>
            context.Response.AddHeader("WWW-Authenticate", digestProvider.GetChallenge());
            //!<#}#>
            //!<# if (Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm)) { #>
            response.AddHeader("WWW-Authenticate", "Negotiate");
            //!<#}#>

            if (!context.Request.HttpMethod.Equals("OPTIONS", StringComparison.InvariantCultureIgnoreCase)
                && !context.Request.HttpMethod.Equals("HEAD", StringComparison.InvariantCultureIgnoreCase))
            {
                byte[] message = new UTF8Encoding().GetBytes("Access is denied.");
                context.Response.ContentLength64 = message.Length;
                context.Response.OutputStream.Write(message, 0, message.Length);
            }
        }
        //!<#}#>

        //!<# if (Params.AuthenticationType.HasFlag(AuthenticationType.Basic) || Params.AuthenticationType.HasFlag(AuthenticationType.Digest)) { #>
        private static IPrincipal AuthenticateRequest(HttpListenerRequest request)
        {
            //!<# if (Params.AuthenticationType.HasFlag(AuthenticationType.Basic)) { #>
            string authName = "Basic";

            // check if Authorization header is present
            string auth = request.Headers["Authorization"];
            if ((auth == null) || !auth.Substring(0, authName.Length).Equals(authName, StringComparison.InvariantCultureIgnoreCase))
                return null;

            // decode username and password
            string base64Credentials = auth.Substring(authName.Length).TrimStart();
            byte[] bytesCredentials = Convert.FromBase64String(base64Credentials);
            string[] credentials = new UTF8Encoding().GetString(bytesCredentials).Split(':');
            string userName = credentials[0];
            string password = credentials[1];

            // Windows may submit the user name in the following format: Machine\User.
            int ind = userName.LastIndexOf('\\');
            if (ind > 0)
            {
                userName = userName.Remove(0, ind + 1);
            }

            if (Membership.ValidateUser(userName, password))
            { // authenticated succesefully
                return new GenericPrincipal(new GenericIdentity(userName), null);
            }
            else
            { // invalid credentials
                return null;
            }
            //!<#}#>
            //!<# if (Params.AuthenticationType.HasFlag(AuthenticationType.Digest)) { #>
            return digestProvider.AuthenticateRequest(request.Headers, request.HttpMethod);
            //!<#}#>
        }
        //!<#}#>

        /// <summary>
        /// Checks configuration errors.
        /// </summary>
        private static void CheckConfigErrors()
        {
            //!<# if (Params.DeltaV) {#>
            string autoput = ConfigurationManager.AppSettings["AutoPutUnderVersionControl"];
            bool v;
            if (!bool.TryParse(autoput, out v))
            {
                string message = "AutoPutUnderVersionControl section is missing or invalid!" +
                    " Should contain 'true/false' value only.";
                throw new Exception(message);
            }
            //!<# } #>

            //!<# if (Params.Storage == StorageType.Db) {#>
            string connStr = ConfigurationManager.ConnectionStrings["WebDAV"].ConnectionString;
            if (string.IsNullOrEmpty(connStr))
            {
                throw new Exception("SqlConnectionString section is missing or invalid!");
            }

            string exePath = Path.GetDirectoryName(new Uri(Assembly.GetExecutingAssembly().EscapedCodeBase).LocalPath);
            string dataPath = Path.Combine(Directory.GetParent(exePath).FullName, "App_Data");
            connStr = connStr.Replace("|DataDirectory|", dataPath);
            // test database connection
            using (SqlConnection conn = new SqlConnection(connStr))
            {
                conn.Open();
            }
            //!<# } #>

            //!<# if (Params.Storage == StorageType.Ntfs) {#>
            string repPath = ConfigurationManager.AppSettings["RepositoryPath"];
            if (repPath == null || !Directory.Exists(repPath))
            {
                throw new Exception("Invalid RepositoryPath configuration parameter value.");
            }
            //!<# } #>

            //!<# if (Params.DeltaV) {#>
            string autoVerMode = ConfigurationManager.AppSettings["AutoVersionMode"];
            if (autoVerMode == null)
            {
                throw new Exception("AutoVersionMode is not specified.");
            }

            if (!new List<string>(Enum.GetNames(typeof(AutoVersion))).Contains(autoVerMode))
            {
                throw new Exception(
                        "Unsupported AutoVersionMode configuration parameter value." + Environment.NewLine +
                        "Possible values are: " +
                        "(NoAutoVersioning | CheckOutCheckIn | CheckOutUnlockedCheckIn | CheckOut | LockedCheckOut).");
            }
            //!<# } #>

            string uriPrefix = ConfigurationManager.AppSettings["ListenerPrefix"];
            if (string.IsNullOrEmpty(uriPrefix))
            {
                throw new Exception("ListenerPrefix section is missing or invalid!");
            }
        }
    }
}
