//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (Params.Hosting != HostingEnvironment.IIS)  return string.Empty; #>
//!<# if (!Params.AuthenticationType.HasFlag(AuthenticationType.Basic))  return string.Empty; #>
//!<# if (!Params.IsOwinAuthMode) return string.Empty; #>

using System;
using Microsoft.AspNet.Identity;
using Microsoft.Owin;
using Owin;

namespace ITHit.WebDav.Server.SampleModel.__Authentication__.BasicAuthentication
{
    /// <summary>
    /// Provides extension methods to add Basic Authenticaiton middleware to ASP.NET MVC project
    /// using app.UseXXX() patern.
    /// </summary>
    public static class BasicAuthenticationMiddlewareExtensions
    {
        /// <summary>
        /// Adds Basic authentication middleware to the OWIN project.
        /// </summary>
        /// <param name="app"></param>
        /// <param name="realm">Realm string, typically displayed in log-in dialog of the client application.</param>
        /// <param name="loginPath">Specifies the path to login redirect URL (typically "/Account/Login") which is used 
        /// by Cookie authentication middleware.</param>
        /// <param name="validationFunction">Method used to validate user credentials.</param>
        /// <returns>The same instance of iAppBuilder as passed to this method.</returns>
        public static IAppBuilder UseBasicAuthentication<TKey, TUser, TManager>(this IAppBuilder app,
            string realm, string loginPath, ValidateUserDelegate validationFunction)
            where TKey : IEquatable<TKey>
            where TUser : class, IUser<TKey>
            where TManager : UserManager<TUser, TKey>
        {
            BasicAuthenticationOptions<TKey, TUser, TManager> options = new BasicAuthenticationOptions<TKey, TUser, TManager>(realm)
            {
                LoginPath = new PathString(loginPath),
                OnValidateUser = validationFunction
            };
            return app.UseBasicAuthentication(options);
        }

        /// <summary>
        /// Adds Basic authentication middleware to the OWIN project
        /// (using passed <see cref="BasicAuthenticationOptions{TKey, TUser, TManager}"/> class).
        /// </summary>
        public static IAppBuilder UseBasicAuthentication<TKey, TUser, TManager>(
            this IAppBuilder app, BasicAuthenticationOptions<TKey, TUser, TManager> options)
            where TKey : IEquatable<TKey>
            where TUser : class, IUser<TKey>
            where TManager : UserManager<TUser, TKey>
        {
            return app.Use<BasicAuthenticationMiddleware<TKey, TUser, TManager>>(options);
        }
    }
}
