//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (Params.Hosting != HostingEnvironment.IIS)  return string.Empty; #>
//!<# if (!Params.AuthenticationType.HasFlag(AuthenticationType.Basic))  return string.Empty; #>
//!<# if (!Params.IsOwinAuthMode) return string.Empty; #>
using System;
using Microsoft.AspNet.Identity;
using Microsoft.Owin;
using Microsoft.Owin.Security.Infrastructure;

namespace ITHit.WebDav.Server.SampleModel.__Authentication__.BasicAuthentication
{
    /// <summary>
    /// Implements Basic Authentication Middleware.
    /// </summary>
    /// <remarks>
    /// To use this middleware, call app.UseBasicAuthentication() extension method
    /// for IAppBuilder instance in Startup.Auth.cs.
    /// 
    /// Basic is supported by most desktop applications such as Microsoft Mini-redirector, 
    /// OS X Finder, Microsoft Office, etc.
    /// Basic can not be used alone to authenticate in third party log-in providers such as 
    /// such as Facebook, Twitter, Google, etc. Use MS-OFBA or Basic + MS-OFBA instead for 
    /// this purposes.
    /// </remarks>
    internal class BasicAuthenticationMiddleware<TKey, TUser, TManager>
        : AuthenticationMiddleware<BasicAuthenticationOptions<TKey, TUser, TManager>>
        where TKey : IEquatable<TKey>
        where TUser : class, IUser<TKey>
        where TManager : UserManager<TUser, TKey>
    {
        public BasicAuthenticationMiddleware(OwinMiddleware nextMiddleware,
            BasicAuthenticationOptions<TKey, TUser, TManager> options)
            : base(nextMiddleware, options)
        {
        }

        /// <summary>
        /// Factory method that creates new instance of <see cref="BasicAuthenticationHandler{TKey, TUser, TManager}"/>
        /// </summary>
        protected override AuthenticationHandler<BasicAuthenticationOptions<TKey, TUser, TManager>> CreateHandler()
        {
            return new BasicAuthenticationHandler<TKey, TUser, TManager>(Options);
        }
    }
}
