//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (Params.Hosting != HostingEnvironment.IIS)  return string.Empty; #>
//!<# if (!Params.AuthenticationType.HasFlag(AuthenticationType.Basic))  return string.Empty; #>
//!<# if (!Params.IsOwinAuthMode) return string.Empty; #>
using System;
using System.Collections.Generic;
using System.Security.Claims;
using System.Threading.Tasks;
using Microsoft.AspNet.Identity;
using Microsoft.Owin;
using Microsoft.Owin.Security;
using Microsoft.Owin.Security.Cookies;


namespace ITHit.WebDav.Server.SampleModel.__Authentication__.BasicAuthentication
{
    public delegate Task<IEnumerable<Claim>> ValidateUserDelegate(string id, string secret, IOwinContext context);

    /// <summary>
    /// Provides options to configure Basic Authentication Middleware.
    /// </summary>
    public class BasicAuthenticationOptions<TKey, TUser, TManager> : AuthenticationOptions
        where TKey : IEquatable<TKey>
        where TUser : class, IUser<TKey>
        where TManager : UserManager<TUser, TKey>
    {
        /// <summary>
        /// Method used to validate user credentials.
        /// </summary>
        public ValidateUserDelegate OnValidateUser { get; set; }

        /// <summary>
        /// Realm string, typically displayed in log-in dialog of the client application.
        /// </summary>
        public string Realm { get; set; }

        /// <summary>
        /// Specifies the path to log-in redirect URL (typically "/Account/Login") which is used 
        /// by Cookie authentication middleware.
        /// </summary>
        /// <remarks>
        /// This property is used to detect the case when Cookie authentication fails in a client 
        /// that doesn't support it (i.e. MS Office application), and thus we need to issue Basic 
        /// authentication.
        /// </remarks>
        public PathString LoginPath { get; set; }

        /// <summary>
        /// The parameter name used to pass the return URL.
        /// </summary>
        /// <value>String that represents return URL parameter name.</value>
        public string ReturnUrlParameter { get; set; }

        /// <param name="realm">Realm string, typically displayed in log-in dialog of the client application.</param>
        public BasicAuthenticationOptions(string realm = "My WebDAV Server")
            : base("Basic")
        {
            Realm = realm;
            LoginPath = CookieAuthenticationDefaults.LoginPath;
            ReturnUrlParameter = CookieAuthenticationDefaults.ReturnUrlParameter;
            OnValidateUser = BasicAuthenticationHandler<TKey, TUser, TManager>.ValidateUser;
        }
    }
}