//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (Params.Hosting != HostingEnvironment.IIS)  return string.Empty; #>
//!<# if (!Params.AuthenticationType.HasFlag(AuthenticationType.Digest))  return string.Empty; #>
//!<# if (!Params.IsOwinAuthMode) return string.Empty; #>
using System;
using Microsoft.AspNet.Identity;
using Microsoft.AspNet.Identity.EntityFramework;
using Owin;
using Microsoft.Owin;

namespace ITHit.WebDav.Server.SampleModel.__Authentication__.DigestAuthentication
{
    /// <summary>
    /// Provides extension methods to add Digest Authenticaiton middleware to ASP.NET MVC project
    /// using app.UseXXX() patern.
    /// </summary>
    public static class DigestAuthenticationMiddlewareExtensions
    {
        /// <summary>
        /// Provides static accessor to <see cref="DigestAuthenticationOptions"/> class 
        /// so that <see cref="ApplicationUserManagerBase{TUser, TKey}"/> class can access them.
        /// </summary>
        internal static DigestAuthenticationOptions DigestAuthenticationOptions { get; private set; }

        /// <summary>
        /// Adds Digest authentication middleware to the OWIN project.
        /// </summary>
        /// <param name="app">The application object.</param>
        /// <param name="realm">Realm string, typically displayed in log-in dialog of the client application.</param>
        /// <param name="loginPath">Specifies the path to log-in redirect URL (typically "/Account/Login") which is used 
        /// by Cookie authentication middleware.</param>
        /// <typeparam name="TKey">Type of the primary key used by ASP.NET MVC / Entity Framework.
        /// string type is used by default, unless you've configured your project to use int primary keys.</typeparam>
        /// <typeparam name="TUser">Application user class.</typeparam>
        /// <typeparam name="TManager">Application user manager class.</typeparam>
        /// <typeparam name="TLogin">ASP.NET Identity Log-in class.</typeparam>
        /// <typeparam name="TRole">ASP.NET Identity Role class.</typeparam>
        /// <typeparam name="TClaim">ASP.NET Identity Claim class.</typeparam>
        public static IAppBuilder UseDigestAuthentication<TKey, TUser, TManager, TLogin, TRole, TClaim>
            (this IAppBuilder app, string realm, string loginPath)
            where TKey : IEquatable<TKey>
            where TUser : class, IUser<TKey>
            where TManager : UserManager<TUser, TKey>
            where TLogin : IdentityUserLogin<TKey>
            where TRole : IdentityUserRole<TKey>
            where TClaim : IdentityUserClaim<TKey>
        {
            DigestAuthenticationOptions options = new DigestAuthenticationOptions(realm)
            {
                LoginPath = new PathString(loginPath),
            };

            return app.UseDigestAuthentication<TKey, TUser, TManager, TLogin, TRole, TClaim>(options);
        }

        /// <summary>
        /// Adds Basic authentication middleware to the OWIN project
        /// (using passed <see cref="DigestAuthenticationOptions"/> class).
        /// </summary>
        /// <param name="app">The application object.</param>
        /// <param name="options">Options class used by Digest Authentication Middleware</param>
        /// <typeparam name="TKey">Type of the primary key used by ASP.NET MVC / Entity Framework.
        /// string type is used by default, unless you've configured your project to use int primary keys.</typeparam>
        /// <typeparam name="TUser">Application user class.</typeparam>
        /// <typeparam name="TLogin">ASP.NET Identity Log-in class.</typeparam>
        /// <typeparam name="TRole">ASP.NET Identity Role class.</typeparam>
        /// <typeparam name="TClaim">ASP.NET Identity Claim class.</typeparam>
        public static IAppBuilder UseDigestAuthentication<TKey, TUser, TManager, TLogin, TRole, TClaim>
            (this IAppBuilder app, DigestAuthenticationOptions options)
            where TKey : IEquatable<TKey>
            where TUser : class, IUser<TKey>
            where TManager : UserManager<TUser, TKey>
            where TLogin : IdentityUserLogin<TKey>
            where TRole : IdentityUserRole<TKey>
            where TClaim : IdentityUserClaim<TKey>
        {
            DigestAuthenticationOptions = options;
            return app.Use<DigestAuthenticationMiddleware<TKey, TUser, TManager, TLogin, TRole, TClaim>>(options);
        }
    }
}