//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (Params.Hosting != HostingEnvironment.IIS)  return string.Empty; #>
//!<# if ((!Params.AuthenticationType.HasFlag(AuthenticationType.Digest)))  return string.Empty; #>
//!<# if (!Params.IsOwinAuthMode) return string.Empty; #>
using System;
using Microsoft.AspNet.Identity;
using Microsoft.AspNet.Identity.EntityFramework;
using Microsoft.Owin;
using Microsoft.Owin.Security.Infrastructure;

namespace ITHit.WebDav.Server.SampleModel.__Authentication__.DigestAuthentication
{
    /// <summary>
    /// Implements Digest Authentication Middleware for ASP.NET MVC applications.
    /// </summary>
    /// <remarks>
    /// To use this middleware, please call app.UseDigestAuthentication() extension method
    /// for IAppBuilder instance in Startup.Auth.cs.
    /// 
    /// Digest is supported by most desktop applications such as Microsoft Mini-redirector, 
    /// OS X Finder, Microsoft Office, etc.
    /// Digest can not be used alone to authenticate in third party log-in providers such as 
    /// such as Facebook, Twitter, Google, etc. Use MS-OFBA or Digest + MS-OFBA instead for 
    /// this purposes. 
    /// </remarks>
    internal class DigestAuthenticationMiddleware<TKey, TUser, TManager, TLogin, TRole, TClaim>
        : AuthenticationMiddleware<DigestAuthenticationOptions>
        where TKey : IEquatable<TKey>
        where TUser : class, IUser<TKey>
        where TManager : UserManager<TUser, TKey>
        where TLogin : IdentityUserLogin<TKey>
        where TRole : IdentityUserRole<TKey>
        where TClaim : IdentityUserClaim<TKey>
    {
        public DigestAuthenticationMiddleware(OwinMiddleware nextMiddleware, DigestAuthenticationOptions options)
            : base(nextMiddleware, options)
        {
        }

        /// <summary>
        /// Factory method that creates new instance of 
        /// <see cref="DigestAuthenticationHandler{TKey, TUser, TManager, TLogin, TRole, TClaim}"/>
        /// </summary>
        protected override AuthenticationHandler<DigestAuthenticationOptions> CreateHandler()
        {
            return new DigestAuthenticationHandler<TKey, TUser, TManager, TLogin, TRole, TClaim>(Options);
        }
    }
}

