//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (Params.Hosting != HostingEnvironment.IIS)  return string.Empty; #>
//!<# if ((!Params.AuthenticationType.HasFlag(AuthenticationType.Digest)))  return string.Empty; #>
//!<# if (!Params.IsOwinAuthMode) return string.Empty; #>
using Microsoft.Owin;
using Microsoft.Owin.Security;
using Microsoft.Owin.Security.Cookies;

namespace ITHit.WebDav.Server.SampleModel.__Authentication__.DigestAuthentication
{
    /// <summary>
    /// Provides options to configure Digest Authentication Middleware.
    /// </summary>
    public class DigestAuthenticationOptions : AuthenticationOptions
    {
        public const string DefaultRealmString = "MyWebDAVServer";

        /// <summary>
        /// Realm string, typically displayed in log-in dialog of the client application.
        /// </summary>
        public string Realm { get; set; }

        /// <summary>
        /// Specifies the path to log-in redirect URL (typically "/Account/Login") which is used 
        /// by Cookie authentication middleware.
        /// This property is used to detect the case when Cookie authentication fails in a client 
        /// that doesn't support it (for example in Microsoft Mini-redirector, OS X Finder, etc), 
        /// and thus we need to issue Digest authentication.
        /// </summary>
        public PathString LoginPath { get; set; }

        /// <summary>
        /// The parameter name used to pass the return URL.
        /// </summary>
        /// <value>String that represents return URL parameter name.</value>
        public string ReturnUrlParameter { get; set; }

        public DigestAuthenticationOptions(string realm = DefaultRealmString)
            : base("Digest")
        {
            Realm = realm;
            LoginPath = CookieAuthenticationDefaults.LoginPath;
            ReturnUrlParameter = CookieAuthenticationDefaults.ReturnUrlParameter;
        }
    }
}