//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (Params.Hosting != HostingEnvironment.IIS)  return string.Empty; #>
//!<# if (!Params.AuthenticationType.HasFlag(AuthenticationType.Digest))  return string.Empty; #>
//!<# if (!Params.IsOwinAuthMode) return string.Empty; #>
using Microsoft.AspNet.Identity;

namespace ITHit.WebDav.Server.SampleModel.__Authentication__.DigestAuthentication
{
    /// <summary>
    /// Custom OWIN password hasher class that computes Digest HA1 hash
    /// (which can be later used to verify credentials during digest authentication).
    /// </summary>
    public class DigestHA1PasswordHasher : IPasswordHasher
    {
        /// <summary>
        /// Hash a password.
        /// </summary>
        /// <param name="userRealmAndPassword">String formatted as "username:realm:password"</param>
        /// <returns>Digest HA1 hash string</returns>
        /// <remarks>
        /// Using this password hasher with Digest Authentication Middleware requires
        /// that <see cref="ApplicationUserManager"/> class is derived from 
        /// <see cref="ApplicationUserManagerBase{TUser, TKey}"/> superclass
        /// (which takes care of passing "username:realm:password" string as password parameter
        /// in identity verification methods).
        /// </remarks>
        public string HashPassword(string userRealmAndPassword)
        {
            return DigestAuthenticationProvider.CreateMD5HashBinHex(userRealmAndPassword);
        }

        /// <summary>
        /// Verify that a password matches the hashedPassword
        /// </summary>
        /// <param name="hashedPassword">
        /// HA1 password hash (typically retrieved from user store in database),
        /// which will be used to verify user provided password.
        /// </param>
        /// <param name="providedPassword">
        /// User provided password string (without adding username and realm parts)
        /// </param>
        /// <returns>Result of the password verification. 
        /// See <see cref="PasswordVerificationResult"/> enumeration for possible return values.
        /// </returns>
        public PasswordVerificationResult VerifyHashedPassword(string hashedPassword, string providedPassword)
        {
            return DigestAuthenticationProvider.CreateMD5HashBinHex(providedPassword).Equals(hashedPassword)
                ? PasswordVerificationResult.Success
                : PasswordVerificationResult.Failed;
        }
    }
}