//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (!Params.CalDav && !Params.CardDav)  return string.Empty; #>
//!<# if (Params.Storage != StorageType.Db)  return string.Empty; #>
using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Threading.Tasks;

using ITHit.WebDAV.Server;
using ITHit.WebDAV.Server.Paging;

using ITHit.WebDav.Server.SampleModel.__CalDavSql__.Acl;
//!<# if (Params.CalDav) { #>
using ITHit.WebDav.Server.SampleModel.__CalDavSql__.CalDav;
//!<# } #>
//!<# if (Params.CardDav) { #>
using ITHit.WebDav.Server.SampleModel.__CalDavSql__.CardDav;
//!<# } #>

namespace ITHit.WebDav.Server.SampleModel.__CalDavSql__
{
    /// <summary>
    /// Logical folder which contains /acl/, /calendars/ and /addressbooks/ folders.
    /// Represents a folder with the following path: [DAVLocation]
    /// </summary>
    /// <example>
    /// [DavLocation]
    ///  |-- acl
    ///  |-- calendars
    ///  |-- addressbooks
    /// </example>
    public class DavLocationFolder : LogicalFolder
    {
        /// <summary>
        /// Path to this folder.
        /// </summary>
        /// <value>Returns first non-root path from DavLocation section from config file or "/" if no DavLocation section is found.</value>
        public static string DavLocationFolderPath
        {
            get
            {
                //!<# if (Params.Hosting != HostingEnvironment.NetCore) { #>   
                NameValueCollection davLocationsSection = (NameValueCollection)System.Configuration.ConfigurationManager.GetSection("davLocations");

                if (davLocationsSection != null)
                {
                    foreach (string path in davLocationsSection.AllKeys)
                    {
                        // Typically you will enable WebDAV on site root ('/') to allow CalDAV/CardDAV 
                        // discovery. We skip site root WebDAV location to find first non-root location.
                        if (!string.IsNullOrEmpty(path.Trim('/')))
                            return path.TrimEnd('/') + '/';
                    }
                }
                //!<# } #>
                // If no davLocation section is found or no non-root WebDAV location is specified in 
                // configuration file asume the WebDAV is on web site root.
                return "/";
            }
        }

        /// <summary>
        /// Initializes a new instance of this class.
        /// </summary>
        /// <param name="context">Instance of <see cref="DavContext"/></param>
        public DavLocationFolder(DavContext context)
            : base(context, DavLocationFolderPath)
        {
        }

        /// <summary>
        /// Retrieves children of this folder: /acl/, /calendars/ and /addressbooks/ folders.
        /// </summary>
        /// <param name="propNames">Properties requested by client application for each child.</param>
        /// <param name="offset">The number of children to skip before returning the remaining items. Start listing from from next item.</param>
        /// <param name="nResults">The number of items to return.</param>
        /// <param name="orderProps">List of order properties requested by the client.</param>
        /// <returns>Children of this folder.</returns>
        public override async Task<PageResults> GetChildrenAsync(IList<PropertyName> propNames, long? offset, long? nResults, IList<OrderProperty> orderProps)
        {
            // In this samle we list users folder only. Groups and groups folder is not implemented.
            return new PageResults(new IHierarchyItem[] 
            { 
                  new AclFolder(Context)
//!<# if (Params.CalDav) { #>
                , new CalendarsRootFolder(Context)
//!<# } #>
//!<# if (Params.CardDav) { #>
                , new AddressbooksRootFolder(Context)
//!<# } #>
            }, null);
        }
    }
}
