//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (!(Params.CalDav || Params.CardDav)) return string.Empty; #>
//!<# if (Params.Storage != StorageType.Db)  return string.Empty; #>
using System;
using System.Configuration;
using System.IO;
using System.Web;
using System.Security.Principal;
using System.Security.AccessControl;
using System.Threading.Tasks;

using ITHit.WebDAV.Server.Acl;
//!<# if (Params.CalDav) { #>
using ITHit.WebDav.Server.SampleModel.__CalDavSql__.CalDav;
//!<# } #>
//!<# if (Params.CardDav) { #>
using ITHit.WebDav.Server.SampleModel.__CalDavSql__.CardDav;
//!<# } #>
//!<# if (false) { #>
// only required to compile project at dev time
using ITHit.WebDav.Server.SampleModel.__CalDavSql__;
//!<# } #>

namespace ITHit.WebDav.Server.SampleModel.__CalDavSql__
{
    /// <summary>
    /// This class creates initial calendar(s) and address book(s) for user during first log-in.
    /// </summary>
    public class Provisioning /*!<# if (Params.Hosting == HostingEnvironment.IIS) { #>!*/: IHttpModule /*!<# } #>!*/
    {
        //!<# if (Params.Hosting == HostingEnvironment.IIS) { #>

        public void Dispose()
        {
        }

        public void Init(HttpApplication application)
        {
            //!<# if (Params.TargetFramework45OrLater) { #>
            EventHandlerTaskAsyncHelper postAuthAsyncHelper = new EventHandlerTaskAsyncHelper(App_OnPostAuthenticateRequestAsync);
            application.AddOnPostAuthenticateRequestAsync(postAuthAsyncHelper.BeginEventHandler, postAuthAsyncHelper.EndEventHandler);
            //!<# } else { #>
            application.PostAuthenticateRequest += App_OnPostAuthenticateRequest;
            //!<# } #>
        }

        private async Task App_OnPostAuthenticateRequestAsync(object source, EventArgs eventArgs)
        {
            HttpContext httpContext = HttpContext.Current;
            if ((httpContext.User == null) || !httpContext.User.Identity.IsAuthenticated)
                return;

            using (DavContext context = new DavContext(httpContext))
            {
                //!<# if (Params.CalDav) { #>
                // Create calendars for the user during first log-in.
                await CreateCalendarFoldersAsync(context);
                //!<# } #>
                //!<# if (Params.CardDav) { #>

                // Create addressboks for the user during first log-in.
                await CreateAddressbookFoldersAsync(context);
                //!<# } #>

                // Closes transaction. Calls ContextAsync{IHierarchyItem}.BeforeResponseAsync only first time this method is invoked.
                // This method must be called manually if ContextAsync{IHierarchyItem} is used outside of DavEngine. 
                await context.EnsureBeforeResponseWasCalledAsync();
            }
        }
        //!<# } #>

        //!<# if (Params.CalDav) { #>

        /// <summary>
        /// Creates initial calendars for users.
        /// </summary>
        internal static async Task CreateCalendarFoldersAsync(DavContext context)
        {
            // If user does not have access to any calendars - create new calendars.
            string sql = @"SELECT ISNULL((SELECT TOP 1 1 FROM [cal_Access] WHERE [UserId] = @UserId) , 0)";
            if (await context.ExecuteScalarAsync<int>(sql, "@UserId", context.UserId) < 1)
            {
                await CalendarFolder.CreateCalendarFolderAsync(context, "Cal 1", "Calendar 1");
                await CalendarFolder.CreateCalendarFolderAsync(context, "Cal 2", "Calendar 2");
                await CalendarFolder.CreateCalendarFolderAsync(context, "Cal 3", "Calendar 3");
            }
        }
        //!<# } #>
        //!<# if (Params.CardDav) { #>

        /// <summary>
        /// Creates initial address books for user.
        /// </summary>
        internal static async Task CreateAddressbookFoldersAsync(DavContext context)
        {
            // If user does not have access to any address books - create new address books.
            string sql = @"SELECT ISNULL((SELECT TOP 1 1 FROM [card_Access] WHERE [UserId] = @UserId) , 0)";
            if (await context.ExecuteScalarAsync<int>(sql, "@UserId", context.UserId) < 1)
            {
                await AddressbookFolder.CreateAddressbookFolderAsync(context, "Book 1", "Address Book 1");
                await AddressbookFolder.CreateAddressbookFolderAsync(context, "Book 2", "Address Book 2");
                await AddressbookFolder.CreateAddressbookFolderAsync(context, "Book 3", "Address Book 3");
            }
        }
        //!<# } #>
    }
}
