//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (!Params.ACL)  return string.Empty; #>
//!<# if (Params.Storage != StorageType.Blank)  return string.Empty; #>
using System;
using System.Collections.Generic;
using System.Threading.Tasks;

using ITHit.WebDAV.Server;
using ITHit.WebDAV.Server.Paging;

namespace ITHit.WebDav.Server.SampleModel.__DavImpl__
{
    /// <summary>
    /// Folder named 'acl' in the root of the repository.
    /// It contains folders 'users' and 'groups'
    /// </summary>
    public class AclFolder : LogicalFolder
    {
        /// <summary>
        /// Initializes a new instance of the AclFolder class.
        /// </summary>
        /// <param name="context">Instance of <see cref="DavContext"/> class.</param>
        public AclFolder(DavContext context) : base(context, "acl", "acl/")
        {
        }

        /// <summary>
        /// Returns child folders inside '/acl/' folder.
        /// For example two folders 'users' and 'groups'.
        /// </summary>
        /// <param name="propNames">List of properties to retrieve with the children. They will be queried by the engine later.</param>
        /// <param name="offset">The number of children to skip before returning the remaining items. Start listing from from next item.</param>
        /// <param name="nResults">The number of items to return.</param>
        /// <param name="orderProps">List of order properties requested by the client.</param>
        /// <returns>Items requested by the client and a total number of items.</returns>
        public override async Task<PageResults> GetChildrenAsync(IList<PropertyName> propNames, long? offset, long? nResults, IList<OrderProperty> orderProps)
        {
            IList<IHierarchyItem> children = new List<IHierarchyItem>();
            children.Add(new UserFolder(Context));
            children.Add(new GroupFolder(Context));

            return new PageResults(children, null);
        }
    }
}
