//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (Params.Storage != StorageType.Blank)  return string.Empty; #>
using System;
using System.Collections.Generic;
using System.IO;
using System.Threading.Tasks;

using ITHit.WebDAV.Server;
//!<# if (Params.ACL) { #>
using ITHit.WebDAV.Server.Acl;
//!<# } #>
using ITHit.WebDAV.Server.Class1;
//!<# if (Params.Class2) { #>
using ITHit.WebDAV.Server.Class2;
//!<# } #>
//!<# if (Params.DeltaV) { #>
using ITHit.WebDAV.Server.DeltaV;
//!<# } #>
//!<# if (Params.MicrosoftExtensions) { #>
using ITHit.WebDAV.Server.Extensibility;
using ITHit.WebDAV.Server.MicrosoftExtensions;
//!<# } #>
//!<# if (Params.Quota) { #>
using ITHit.WebDAV.Server.Quota;
//!<# } #>
//!<# if (Params.ResumableUpload || Params.IEUpload) { #>
using ITHit.WebDAV.Server.ResumableUpload;
//!<# } #>
using IPrincipal = ITHit.WebDAV.Server.Acl.IPrincipal;

namespace ITHit.WebDav.Server.SampleModel.__DavImpl__
{
    /// <summary>
    /// Represents a file in WebDAV repository.
    /// </summary>
    public class DavFile : IFile/*!<# if (Params.Class2) { #>!*/, ILock/*!<# } #>!*//*!<# if (Params.ResumableUpload || Params.IEUpload) { #>!*/, IResumableUpload, IUploadProgress/*!<# } #>!*//*!<# if (Params.DeltaV) { #>!*/, IVersionableItem/*!<# } #>!*//*!<# if (Params.ACL) { #>!*/, IAclHierarchyItem/*!<# } #>!*//*!<# if (Params.Quota) { #>!*/, IQuota/*!<# } #>!*//*!<# if (Params.MicrosoftExtensions) { #>!*/, IMsItem/*!<# } #>!*/
    {
        /// <summary>
        /// Initializes a new instance of the DavFile class.
        /// </summary>
        /// <param name="context">Instance of <see cref="DavContext"/>.</param>
        public DavFile(DavContext context)
        {
        }

        /// <summary>
        /// Gets the media type of the file.
        /// </summary>
        /// <remarks>
        /// <para>
        /// The mime-type provided by this property is returned in a Content-Type header with GET request.
        /// </para>
        /// <para>
        /// When deciding which action to perform when downloading a file some WebDAV clients and browsers (such as
        /// Internet Explorer) rely on file extension, while others (such as Firefox) rely on Content-Type header
        /// returned by server. For identical behavior in all browsers and WebDAV clients your server must return
        /// a correct mime-type with a requested file.
        /// </para>
        /// </remarks>
        /// <value>The MIME type of the file.</value>
        public string ContentType
        {
            get { throw new NotImplementedException(); }
        }

        /// <summary>
        /// Gets the size of the file content in bytes.
        /// </summary>
        /// <value>Length of the file content in bytes.</value>
        public long ContentLength
        {
            get { throw new NotImplementedException(); }
        }

        /// <summary>
        /// Writes the content of the file to the specified stream.
        /// </summary>
        /// <param name="output">Output stream.</param>
        /// <param name="startIndex">The zero-bazed byte offset in file content at which to begin copying bytes to the
        /// output stream.</param>
        /// <param name="count">The number of bytes to be written to the output stream.</param>
        /// <remarks>
        /// <para>
        /// By default ASP.NET buffers content on server side before sending output. You must turn off buffering to
        /// eliminate keeping entire file content in memory before sending:
        /// <code>
        /// HttpContext.Current.Response.BufferOutput = false;
        /// </code>
        /// </para>
        /// <para>
        /// Client application can request only a part of a file specifying <b>Range</b> header.
        /// Download managers may use this header to download single file using several threads at a time.
        /// </para>
        /// </remarks>
        public async Task ReadAsync(Stream output, long startIndex, long count)
        {
            throw new NotImplementedException();
        }

        //!<# if (Params.ResumableUpload || Params.IEUpload) { #>
        /// <summary>
        /// In this method implementation you can delete partially uploaded file.
        /// </summary>
        /// <remarks>
        /// <para>
        /// Often during long-continued upload you will keep the old file 
        /// content to be returned by GET requests and store the new file content in a
        /// temporary file (or temporary field in database, etc).  To delete this partially
        /// uploaded content client can submit CANCELUPLOAD command, the Engine will call this method in this case.
        /// </para>
        /// <para>If the item was automatically checked-out by the Engine when upload started it will be automatically
        /// checked-in by the Engine after this call.</para>
        /// </remarks>
        public async Task CancelUploadAsync()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Gets date and time when the last chunk of file was saved in your storage.
        /// </summary>
        /// <remarks>
        /// <para>Requested by the Engine during a call to 
        /// <see cref="IUploadProgress.GetUploadProgressAsync"/>.</para>
        /// </remarks>
        public DateTime LastChunkSaved
        {
            get { throw new NotImplementedException(); }
        }

        /// <summary>
        /// Gets amount of bytes successfully saved to your storage.
        /// </summary>
        /// <remarks>
        /// <para>Client will use value returned by this property to restore broken upload.
        /// This value shall always reflect number of bytes already stored to persistent medium.
        /// </para>
        /// <para>Requested by the Engine during a call to 
        /// <see cref="IUploadProgress.GetUploadProgressAsync"/>.</para>
        /// </remarks>
        public long BytesUploaded
        {
            get { throw new NotImplementedException(); }
        }

        /// <summary>Gets total file size that is being uploaded.</summary>
        /// <remarks>
        /// <para>This value is passed to <see cref="IContent.WriteAsync"/> method.
        ///  Usually AJAX/HTML based clients will use value returned by this property to display upload progress.</para>
        /// <para>Requested by the Engine during a call to <see cref="IUploadProgress.GetUploadProgressAsync"/>.</para>
        /// </remarks>
        /// <returns>Total file size in bytes.</returns>
        public long TotalContentLength
        {
            get { throw new NotImplementedException(); }
        }

        /// <summary>Array of items that are being uploaded to this item subtree.</summary>
        /// <remarks>
        /// <para>
        /// Return array with a single item if implemented on file items. Return all items that are being uploaded to this subtree if implemented on folder items.
        /// </para>
        /// <para>
        /// Engine calls <see cref="IHierarchyItem.Path"/>, 
        /// <see cref="IResumableUpload.LastChunkSaved"/>, 
        /// <see cref="IResumableUpload.BytesUploaded"/>, 
        /// <see cref="IResumableUpload.TotalContentLength"/> and returns this information to
        /// client.
        /// </para>
        /// </remarks>
        public async Task<IEnumerable<IResumableUpload>> GetUploadProgressAsync()
        {
            throw new NotImplementedException();
        }

        //!<# } #>
        /// <summary>
        /// Saves the content of the file from the specified stream to the WebDAV repository.
        /// </summary>
        /// <param name="content">Stream to read the content of the file from.</param>
        /// <param name="contentType">Indicates the media type of the file.</param>
        /// <param name="startIndex">Start offset to which content shall be saved.</param>
        /// <param name="totalLength">Entire length of the file. Is is not less then length of
        /// <paramref name="content"/> stream.</param>
        /// <returns>Boolean value indicating whether entire stream was written.
        /// This value is used by engine to take decision whether autocheckin shall be performed.</returns>
        /// <remarks>
        /// <para>IIS and ASP.NET does not support files upload larger than 2Gb.
        /// If you need to upload files larger than 2Gb you must develop HttpListener-based WebDAV server or
        /// implement resumable upload interfaces.</para>
        /// <para>
        /// If you are creating HttpHandler-based WebDAV server you must specify the file 
        /// maximum upload size in web.config of your web application. By default maximum 
        /// upload size is set to 4096 KB (4 MB) by ASP.NET. This limit is used to 
        /// prevent denial of service attacks caused by users posting large files to the 
        /// server. To increase the upload limit add &lt;httpRuntime&gt; section to your web application web.config
        /// file and specify the limit in kilobytes: 
        /// </para>
        /// <code>
        ///   <![CDATA[
        ///     <configuration>
        ///        <system.web>
        ///           ...
        ///           <httpRuntime maxRequestLength="2097151" /> //2Gb
        ///           ...
        ///        </system.web>
        ///     </configuration>
        /// ]]>
        /// </code>
        /// <para>
        /// When client uploads file to IIS, ASP.NET first creates the file in a the temporary upload directory. Only when the entire file is uploaded to server you can read its content from stream. By default ASP.NET uploads files to <b>%FrameworkInstallLocation%\Temporary ASP.NET Files</b> folder. You must make sure you have enough disk space to keep temporary files uploaded to your server. To change this folder location add the following section to your web.config file:
        /// <code>
        ///   <![CDATA[
        ///    <configuration>
        ///        <system.web>
        ///           ...
        ///           <compilation tempDirectory="temporary files directory" />
        ///           ...
        ///        </system.web>
        ///    </configuration>
        ///   ]]>
        /// </code>
        /// To avoid temporary file creation and pass content directly to engine set the
        /// <see cref="ITHit.WebDAV.Server.ResumableUpload.PutUploadProgressAndResumeModule"/> module in
        /// your web.config file. Unlike IIS/ASP.NET, HttpListener-based server does not create any temporary
        /// files when handling uploads.
        /// </para>
        /// </remarks>
        public async Task<bool> WriteAsync(Stream content, string contentType, long startIndex, long totalLength)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Gets entity tag - string that identifies current state of resource's content.
        /// More information about etags is available here: http://en.wikipedia.org/wiki/HTTP_ETag
        /// You can return here either cheksum or hash or counter which increases with every
        /// modification.
        /// </summary>
        /// <remarks>
        /// This property shall return different value if content changes.
        /// Return <c>null</c> to indicate that server doesn't support etags.
        /// </remarks>
        public string Etag
        {
            get { throw new NotImplementedException(); }
        }

        /// <summary>
        /// Gets the name of the item in repository.
        /// </summary>
        public string Name
        {
            get { throw new NotImplementedException(); }
        }

        /// <summary>
        /// Gets the creation date of the item in repository expressed as the coordinated universal time (UTC).
        /// </summary>
        public DateTime Created
        {
            get { throw new NotImplementedException(); }
        }

        /// <summary>
        /// Gets the last modification date of the item in repository expressed as the coordinated universal time (UTC).
        /// </summary>
        public DateTime Modified
        {
            get { throw new NotImplementedException(); }
        }

        /// <summary>
        /// Creates a copy of this item with a new name in the destination folder.
        /// </summary>
        /// <param name="destFolder">Destination folder.</param>
        /// <param name="destName">Name of the destination item.</param>
        /// <param name="deep">Indicates whether to copy entire subtree.</param>
        /// <param name="multistatus">If some items fail to copy but operation in whole shall be continued, add
        /// information about the error into <paramref name="multistatus"/> using 
        /// <see cref="MultistatusException.AddInnerException(string,ITHit.WebDAV.Server.DavException)"/>.
        /// </param>
        /// <remarks>
        /// <para>
        /// If error occurred while copying file located in a subtree, the server 
        /// should try to continue copy operation and copy all other items. In this case 
        /// you must add that error <paramref name="multistatus"/> container.
        /// </para>
        /// <para>
        /// A CopyTo method invocation must not copy any locks active on the source item.
        /// However, if this method copies the item into a folder that has a deep lock,
        /// then the destination item must be added to the lock.
        /// </para>
        /// </remarks>
        public async Task CopyToAsync(IItemCollection destFolder, string destName, bool deep, MultistatusException multistatus)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Moves this item to the destination folder under a new name.
        /// </summary>
        /// <param name="destFolder">Destination folder.</param>
        /// <param name="destName">Name of the destination item.</param>
        /// <param name="multistatus">If some items fail to copy but operation in whole shall be continued, add
        /// information about the error into <paramref name="multistatus"/> using 
        /// <see cref="MultistatusException.AddInnerException(string,ITHit.WebDAV.Server.DavException)"/>.
        /// </param>
        /// <remarks>
        /// <papa>
        /// If the item is locked the server must not move any locks with the item. However, items must be added to an
        /// existing lock at the destination.
        /// </papa>
        /// </remarks>
        public async Task MoveToAsync(IItemCollection destFolder, string destName, MultistatusException multistatus)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Deletes this item.
        /// </summary>
        /// <param name="multistatus">If some items fail to delete but operation in whole shall be continued, add
        /// information about the error into <paramref name="multistatus"/> using
        /// <see cref="MultistatusException.AddInnerException(string,ITHit.WebDAV.Server.DavException)"/>.
        /// </param>
        public async Task DeleteAsync(MultistatusException multistatus)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Gets values of all properties or selected properties for this item.
        /// </summary>
        /// <param name="props">
        /// Array of property names which values are requested. 
        /// If a property does not exist for this hierarchy item then the property value shall not be returned.
        /// </param>
        /// <param name="allprop">
        /// If it is <c>true</c> it means that besides properties listed in <paramref name="props"/> you need to 
        /// return all properties you think may be useful to client.
        /// </param>
        /// <returns>
        /// Enumerable with property values.
        /// </returns>
        public async Task<IEnumerable<PropertyValue>> GetPropertiesAsync(IList<PropertyName> props, bool allprop)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Gets names of all properties for this item.
        /// </summary>
        /// <returns>
        /// Enumerable with available property names.
        /// </returns>
        /// <remarks>
        /// <para>Most WebDAV clients never request list of property names, so your implementation can just return
        /// empty enumerable.</para>
        /// </remarks>
        public async Task<IEnumerable<PropertyName>> GetPropertyNamesAsync()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Modifies and removes properties for this item.
        /// </summary>
        /// <param name="setProps">Array of properties to be set.</param>
        /// <param name="delProps">
        /// Property names to be removed. Properties that don't exist shall be skipped.
        /// </param>
        /// <param name="multistatus">The standard requires this operation to be transactional.
        /// If some properties fail to update but there is no possibility to rollback the transaction
        /// in <see cref="ContextAsync{IHierarchyItem}.BeforeResponseAsync"/>, add
        /// information about the error into <paramref name="multistatus"/> 
        /// using <see cref="MultistatusException.AddInnerException(string,ITHit.WebDAV.Server.PropertyName,ITHit.WebDAV.Server.DavException)"/>.
        /// In this case engine will report correct statuses for all properties at least
        /// (although this is against standard).
        /// </param>
        public async Task UpdatePropertiesAsync(
            IList<PropertyValue> setProps,
            IList<PropertyName> delProps,
            MultistatusException multistatus)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Unique item path in the repository relative to storage root.
        /// </summary>
        /// <remarks>
        /// <para>
        /// The URL returned by this property is relative to storage root.
        /// If your server root is located at http://example.webdavsystem.com:8080/myserver/ and the item URL is
        /// http://example.webdavsystem.com:8080/myserver/myfolder/myitem.doc this property implementation must
        /// return myfolder/myitem.doc. To calculate the entire item URL the engine will
        /// call <see cref="DavRequest.ApplicationPath"/> property and attach it to url returned by
        /// <see cref="IHierarchyItem.Path"/> property.
        /// </para>
        /// <para>
        /// Every part of the path (between '/' characters) shall be encoded,
        /// for example using <see cref="EncodeUtil"/>.
        /// </para>
        /// <para>Examples:
        /// <list type="bullet">
        /// <item><description>File: myfolder/my%20doc.docx</description></item>
        /// <item><description>Folder: myfolder/folder/</description></item>
        /// <item><description>History item: myfolder/mydoc.docx?history</description></item>
        /// <item><description>Version: myfolder/mydoc.docx?version=5</description></item>
        /// </list>
        /// </para>
        /// </remarks>
        /// <value><c>String</c> representing relative item path in the repository.</value>
        public string Path
        {
            get { throw new NotImplementedException(); }
        }

        //!<# if (Params.ACL) { #>
        /// <summary>
        /// Retrieves a particular principal as being the "owner" of the item. Since the owner of a
        /// resource often has special access control capabilities (e.g., the owner frequently has permanent
        /// <see cref="Privilege.WriteAcl"/> privilege), clients might display the resource owner in their
        /// user interface.
        /// </summary>
        public async Task SetOwnerAsync(IPrincipal value)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Retrieves a particular principal as being the "owner" of the item. Since the owner of a
        /// resource often has special access control capabilities (e.g., the owner frequently has permanent
        /// <see cref="Privilege.WriteAcl"/> privilege), clients might display the resource owner in their user
        /// interface.
        /// </summary>
        /// <remarks>Can be null.</remarks>
        public async Task<IPrincipal> GetOwnerAsync()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Retrieves a particular principal as being the "group" of the item. This property is commonly
        /// found on repositories that implement the Unix privileges model.
        /// </summary>
        public async Task SetGroupAsync(IPrincipal value)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Retrieves a particular principal as being the "group" of the item. This property is commonly
        /// found on repositories that implement the Unix privileges model.
        /// </summary>
        /// <remarks>
        /// Can return null if group is not assigned.
        /// </remarks>
        public async Task<IPrincipal> GetGroupAsync()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Retrieves the privileges defined for the resource.
        /// </summary>
        public async Task<IEnumerable<SupportedPrivilege>> GetSupportedPrivilegeSetAsync()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Retrieves the exact set of privileges (as computed by
        /// the server) granted to the currently authenticated HTTP user. Aggregate privileges and their contained
        /// privileges are listed. A user-agent can use the value of this property to adjust its user interface to make
        /// actions inaccessible (e.g., by graying out a menu item or button) for which the current principal does
        /// not have permission. This property is also useful for determining what operations the current principal
        /// can perform, without having to actually execute an operation.
        /// </summary>
        public async Task<IEnumerable<Privilege>> GetCurrentUserPrivilegeSetAsync()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Specifies the list of access control entries (ACEs), which define what principals
        /// are to get what privileges for this resource.
        /// </summary>
        /// <param name="propertyNames">List of properties that might be retrieved from those <see cref="IPrincipal"/>
        /// returned.</param>
        /// <exception cref="NeedPrivilegesException">Not enough permissions.</exception>
        /// <exception cref="DavException">In case of other errors.</exception>
        /// <remarks>
        /// <paramref name="propertyNames"/> is for optimization purposes and you may ignore it.
        /// For "principal-pop-set" report engine will call this method, retrieve all principals from result
        /// and extract properties specified in <paramref name="propertyNames"/> from it. In this case prefetching
        /// these properties may improve performance.
        /// </remarks>
        public async Task<IEnumerable<ReadAce>> GetAclAsync(IList<PropertyName> propertyNames)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Sets list of access control entries (ACEs), which define what principals
        /// are to get what privileges for this resource.
        /// </summary>
        /// <remarks>
        /// Two common operations are to add or remove an ACE from an existing access
        /// control list. To accomplish this, a client uses the PROPFIND method to retrieve the value of the DAV:acl
        /// property, then parses the returned access control list to remove all inherited and protected ACEs (these
        /// ACEs are tagged with the DAV:inherited and DAV:protected XML elements). In the remaining set of
        /// non-inherited, non-protected ACEs, the client can add or remove one or more ACEs before submitting the
        /// final ACE set in the request body of the ACL method.
        /// It is possible that the ACEs visible to the current user in the DAV:acl property may only be a portion of
        /// the complete set of ACEs on that resource. If this is the case, an ACL request only modifies the set of
        /// ACEs visible to the current user, and does not affect any non-visible ACE.
        /// </remarks>
        public async Task SetAclAsync(IList<WriteAce> aces)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Defines the types of ACLs supported by this server, to avoid clients needlessly getting
        /// errors. When a client tries to set an ACL via the ACL method, the server may reject the attempt to set
        /// the ACL as specified.
        /// </summary>
        public async Task<AclRestriction> GetAclRestrictionsAsync()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Contains a set of items that also control the access to this item.
        /// To have a privilege on an item, not only must the ACL on that item (specified in the
        /// <see cref="ITHit.WebDAV.Server.Acl"/> property of that item) grant the privilege, but so must the ACL of
        /// each items identified in the <see cref="IAclHierarchyItem.GetInheritedAclSetAsync"/> property of that item.
        /// Effectively, the privileges granted by the current ACL are ANDed with the privileges granted by each
        /// inherited ACL.
        /// </summary>
        public async Task<IEnumerable<IHierarchyItem>> GetInheritedAclSetAsync()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Retrieves a set of root collections that contain the
        /// principals that are available on the server that implements this resource. It can be used to retrieve the
        /// <see cref="IHierarchyItem.Name"/> properties
        /// of all principals on that server, thereby yielding human-readable
        /// names for each principal that could be displayed in a user interface.
        /// </summary>
        public async Task<IEnumerable<IPrincipalFolder>> GetPrincipalCollectionSetAsync()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Returns predefined principal.
        /// </summary>
        /// <param name="wellKnownPrincipal">One of <see cref="WellKnownPrincipal"/> values.</param>
        /// <returns>Well known principal or <c>null</c> if it is not supported.</returns>
        public async Task<IPrincipal> ResolveWellKnownPrincipalAsync(WellKnownPrincipal wellKnownPrincipal)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Find all resources in the subtree which have either Group or Owner (defined by <paramref name="matchBy"/>
        /// parameter) which identifies current user or group current user belongs to.
        /// For example, this report can return all of the resources in a collection
        /// hierarchy that are owned by the current user.
        /// </summary>
        /// <param name="matchBy">Identifies whether to search by owner or group.</param>
        /// <param name="props">Properties requested for found items.</param>
        /// <returns>Matching resources.</returns>
        public async Task<IEnumerable<IAclHierarchyItem>> GetItemsByPropertyAsync(MatchBy matchBy, IList<PropertyName> props)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Retrieves currently logged in principal.
        /// </summary>
        /// <returns>Currently logged in principal.</returns>
        public async Task<IPrincipal> GetCurrentUserPrincipalAsync()
        {
            throw new NotImplementedException();
        }

        //!<# } #>

        //!<# if (Params.Class2) { #>
        /// <summary>
        /// Gets the array of all locks for this item.
        /// </summary>
        /// <remarks>
        /// <para>
        /// This property must return all locks for the item including deep locks on any of the parent folders.
        /// All fields of each <see cref="LockInfo"/> structure in the array must be set.
        /// </para>
        /// </remarks>
        public async Task<IEnumerable<LockInfo>> GetActiveLocksAsync()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Locks this item.
        /// </summary>
        /// <param name="level">Whether lock is share or exclusive</param>
        /// <param name="isDeep">Whether lock is deep.</param>
        /// <param name="requestedTimeOut">Lock timeout which was requested by client.
        /// Server may ignore this parameter and set any timeout.</param>
        /// <param name="owner">Owner of the lock as specified by client.</param> 
        /// <returns>
        /// Instance of <see cref="LockResult"/> with information about the lock.
        /// </returns>
        /// <remarks>
        /// WebDAV engine passes <see cref="LockInfo"/> structure to this method by reference.
        /// All fields of the structure are provided by WebDAV client except <see cref="LockInfo.Token"/> field.
        /// In your <c>Lock</c> implementation you must create lock token and set <see cref="LockInfo.Token"/> member.
        /// You must also associate generated token with the hierarchy item in the repository during this call.
        /// The token is sent to the WebDAV client.
        /// </remarks>
        public async Task<LockResult> LockAsync(LockLevel level, bool isDeep, TimeSpan? requestedTimeOut, string owner)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Updates lock timeout information on this item.
        /// </summary>
        /// <param name="token">Lock token.</param>
        /// <param name="requestedTimeOut">Lock timeout which was requested by client.
        /// Server may ignore this parameter and set any timeout.</param>
        /// <returns>
        /// Instance of <see cref="LockResult"/> with information about the lock.
        /// </returns>
        public async Task<RefreshLockResult> RefreshLockAsync(string token, TimeSpan? requestedTimeOut)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Removes lock with the specified token from this item.
        /// </summary>
        /// <param name="lockToken">Lock with this token should be removed from the item.</param>
        /// <remarks>
        /// <para>
        /// If this lock included more than one hierarchy item, the lock is removed from all items included in the lock.
        /// </para>
        /// </remarks>
        public async Task UnlockAsync(string lockToken)
        {
            throw new NotImplementedException();
        }
        //!<# } #>

        //!<# if (Params.DeltaV) { #>
        /// <summary>
        /// Sets a brief comment about a file that is suitable for presentation to a user.
        /// </summary>
        /// <param name="comment">Comment string.</param>
        /// <remarks>
        /// Comment can be used to indicate why that version was created.
        /// </remarks>
        public async Task SetCommentAsync(string comment)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Retrieves a brief comment about a file that is suitable for presentation to a user.
        /// </summary>
        /// <remarks>
        /// Comment can be used to indicate why that version was created.
        /// </remarks>
        /// <returns>Comment string.</returns>
        public async Task<string> GetCommentAsync()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Sets display name of the user that created this item.
        /// </summary>
        /// <remarks>
        /// Sets description of the creator of the file that is
        /// suitable for presentation to a user. Can be used to indicate who created that version.
        /// </remarks>
        /// <param name="creatorDisplayName">String representing author name.</param>
        public async Task SetCreatorDisplayNameAsync(string creatorDisplayName)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Retrieves display name of the user that created this item.
        /// </summary>
        /// <remarks>
        /// Retrieves description of the creator of the file that is
        /// suitable for presentation to a user. Can be used to indicate who created that version.
        /// </remarks>
        /// <returns>String representing author name.</returns>
        public async Task<string> GetCreatorDisplayNameAsync()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Current item version history. Null, if item is not under version control.
        /// </summary>
        /// <value>
        /// Item implementing <see cref="IHistory"/> interface or null if item is not under version control.
        /// </value>
        /// <remarks>If item is under version control it always has at last
        /// one version in its versions list. This property is used for precondition checking and shall not throw
        /// exceptions.
        /// </remarks>
        public IHistory VersionHistory
        {
            get { throw new NotImplementedException(); }
        }

        /// <summary>
        /// Creates new version. Copies all properties and content from this item.
        /// </summary>
        /// <returns>
        /// Url of the newly created version.
        /// </returns>
        /// <remarks>
        /// <para>
        /// In your implementation you must create a new version. The content and properties of the new version must
        /// be copied from this item. 
        /// </para>
        /// <para>
        /// After the call to this method method <see cref="IHistory.GetCurrentVersionAsync"/> must return the
        /// created version.
        /// </para>
        /// </remarks>
        public async Task<string> CheckInAsync()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Allow modifications to the content and properties of this version-controlled item.
        /// </summary>
        /// <remarks>
        /// <para>
        /// In your <see cref="IVersionableItem.CheckOutAsync"/> implementation you will mark item as checked-out and
        /// allow item modifications. 
        /// When item is in check-out state WebDAV client can issue commands updating item contents and properties.
        /// </para>
        /// </remarks>
        public async Task CheckOutAsync(bool isAutoCheckOut)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Cancels the checkout and restores the pre-checkout state of the version-controlled item.
        /// </summary>
        /// <remarks>
        /// In your <see cref="IVersionableItem.UnCheckOutAsync"/> implementation you will discard changes and restore
        /// pre-checkout state. Content and properties must be copied from current version to this item.
        /// The item must be marked as checked-in.
        /// </remarks>
        public async Task UnCheckOutAsync()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Updates content and properties of the item to those identified by <paramref name="version"/> parameter.
        /// </summary>
        /// <remarks>
        /// <para>
        /// In your <see cref="IVersionableItem.UpdateToVersionAsync"/> implementation you will create a new version and
        /// copy content and properties from <see cref="IVersion"/> passed as a parameter to new version.
        /// You will also replace content and properties of this item. 
        /// The new created version becomes current version. 
        /// </para>
        /// <para>
        /// The <see cref="IVersionableItem.UpdateToVersionAsync"/> method can only be called when item is in check-in state.
        /// </para>
        /// </remarks>
        public async Task UpdateToVersionAsync(IVersion version)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Sets property which determines how checked-in item responds to
        /// WebDAV client attempts to modify its content or properties.
        /// </summary>
        /// <value>One of <see cref="ITHit.WebDAV.Server.DeltaV.AutoVersion"/> enum values.</value>
        public async Task SetAutoVersionAsync(AutoVersion value)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Retrieves property which determines how checked-in item responds to WebDAV
        /// client attempts to modify its content or properties.
        /// </summary>
        /// <value>One of <see cref="ITHit.WebDAV.Server.DeltaV.AutoVersion"/> enum values.</value>
        public async Task<AutoVersion> GetAutoVersionAsync()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Gets a value indicating whether the item is in checked-in or checked-out state.
        /// </summary>
        /// <value>
        /// Boolean value indicating if item is in checked-out state.
        /// </value>
        /// <remarks>
        /// This property is used for precondition checking and shall not throw exceptions.
        /// </remarks>
        public bool IsCheckedOut
        {
            get { throw new NotImplementedException(); }
        }

        /// <summary>
        /// Puts or removes current item from version control.
        /// </summary>
        /// <remarks>
        /// <para>
        /// By default items in the repository are not under version control. When item is being put under
        /// version control engine calls 
        /// <see cref="IVersionableItem.PutUnderVersionControlAsync"/> method passing <b>true</b> as a parameter.
        /// In your <see cref="IVersionableItem.PutUnderVersionControlAsync"/> 
        /// implementation you must create a new version. The content and properties of the new version must be copied
        /// from this item. After the call to <see cref="IVersionableItem.PutUnderVersionControlAsync"/> 
        /// <see cref="IVersionableItem.VersionHistory"/> property must point to the object implementing 
        /// <see cref="IHistory"/> interface that will contain single version.
        /// The <see cref="IVersionableItem.IsCheckedOut"/> property must return <b>false</b>;
        /// </para>
        /// <para><b>If item is under version control it mast always have at last one version in its versions list.</b>
        /// </para>
        /// <para>
        /// If <see cref="DavEngineAsync.AutoPutUnderVersionControl"/> is <b>true</b> and item is not under version control
        /// prior to any item content or properties update <see cref="IVersionableItem.PutUnderVersionControlAsync"/>
        /// will be called.
        /// </para>
        /// <para>
        /// When item is being removed from version control engine calls 
        /// <see cref="IVersionableItem.PutUnderVersionControlAsync"/> method passing <b>false</b> 
        /// as a parameter. In your implementation you will usually delete all versions. 
        /// <see cref="IVersionableItem.VersionHistory"/> property must return 
        /// <b>null</b> after this call.
        /// </para>
        /// </remarks>
        public async Task PutUnderVersionControlAsync(bool enable)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Gets a value indicating whether the item was check-out automatically by engine without explicit request
        /// from client.
        /// </summary>
        /// <remarks>
        /// Before checking-out the engine sets this property.
        /// When item is being unlocked engine reads this property and calls 
        /// <see cref="IVersionableItem.CheckInAsync"/> if necessary. This property is required for auto-versioning.
        /// This property shall not throw exceptions.
        /// </remarks>
        public bool IsAutoCheckedOut
        {
            get { throw new NotImplementedException(); }
        }
        //!<# } #>

        //!<# if (Params.Quota) { #>
        /// <summary>
        /// Value in bytes representing the amount of additional disk space beyond the current
        /// allocation that can be allocated to the folder(or other item) before further
        /// allocations will be refused.  It is understood that this space may be
        /// consumed by allocations to other files/folders.
        /// </summary>
        /// <returns>Bytes that can be additionally allocated in folder/file.</returns>
        public async Task<long> GetAvailableBytesAsync()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        ///  Value in bytes representing the amount of space used by this folder/file
        ///  and possibly a number of other similar folders/files, where the set of "similar" meets at least
        ///  the criterion that allocating space to any folder/file in the set will
        ///  count against the <see cref="IQuota.GetAvailableBytesAsync"/>.  It MUST include the
        ///  total count including usage derived from sub-items if
        ///  appropriate.  It SHOULD include metadata storage size if metadata
        ///  storage is counted against the <see cref="IQuota.GetAvailableBytesAsync"/>
        /// </summary>
        /// <returns>Bytes occupied by folder/file.</returns>
        public async Task<long> GetUsedBytesAsync()
        {
            throw new NotImplementedException();
        }
        //!<# } #>

        //!<# if (Params.MicrosoftExtensions) { #>
        /// <summary>
        /// Retrieves file attributes.
        /// </summary>
        /// <returns>File attributes.</returns>
        public async Task<FileAttributes> GetFileAttributesAsync()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Updates file attributes.
        /// </summary>
        /// <param name="value">File attributes.</param>
        public async Task SetFileAttributesAsync(FileAttributes value)
        {
            throw new NotImplementedException();
        }
        //!<# } #>
    }
}
