//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (Params.Storage != StorageType.Blank)  return string.Empty; #>
using System;
using System.Collections.Generic;
//!<# if (Params.MicrosoftExtensions) { #>
using System.IO;
//!<# } #>
using System.Threading.Tasks;

using ITHit.WebDAV.Server;
//!<# if (Params.ACL) { #>
using ITHit.WebDAV.Server.Acl;
//!<# } #>
using ITHit.WebDAV.Server.Class1;
//!<# if (Params.Class2) { #>
using ITHit.WebDAV.Server.Class2;
//!<# } #>
//!<# if (Params.MicrosoftExtensions) { #>
using ITHit.WebDAV.Server.DeltaV;
using ITHit.WebDAV.Server.Extensibility;
using ITHit.WebDAV.Server.MicrosoftExtensions;
//!<# } #>
//!<# if (Params.Quota) { #>
using ITHit.WebDAV.Server.Quota;
//!<# } #>
using ITHit.WebDAV.Server.Paging;
using IPrincipal = ITHit.WebDAV.Server.Acl.IPrincipal;

namespace ITHit.WebDav.Server.SampleModel.__DavImpl__
{
    /// <summary>
    /// Represents a folder in a WebDAV repository.
    /// </summary>
    public class DavFolder : IFolder/*!<# if (Params.Class2) { #>!*/, ILock/*!<# } #>!*//*!<# if (Params.ACL) { #>!*/, IAclHierarchyItem/*!<# } #>!*//*!<# if (Params.Quota) { #>!*/, IQuota/*!<# } #>!*//*!<# if (Params.MicrosoftExtensions) { #>!*/, IMsItem/*!<# } #>!*/
    {
        /// <summary>
        /// Initializes a new instance of the DavFolder class.
        /// </summary>
        /// <param name="context">Instance of <see cref="DavContext"/> class.</param>
        public DavFolder(DavContext context)
        {
        }

        /// <summary>
        /// Gets direct children of this folder.
        /// </summary>
        /// <value>
        /// Array of <see cref="IHierarchyItem"/> objects. Each item is a file or folder item.
        /// </value>
        public async Task<PageResults> GetChildrenAsync(IList<PropertyName> propNames, long? offset, long? nResults, IList<OrderProperty> orderProps)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Creates new WebDAV file with the specified name in this folder.
        /// </summary>
        /// <param name="name">Name of the file to create.</param>
        /// <param name="content">Stream to read the content of the file from.</param>
        /// <param name="contentType">Indicates the media type of the file.</param>
        /// <param name="totalFileSize">Size of file as it will be after all parts are uploaded. -1 if unknown (in case of chunked upload).</param>
        /// <remarks>You must create a file in your repository during this call. After calling this method Engine calls
        /// <see cref="IContent.WriteAsync"/>.</remarks>
        public async Task<IFile> CreateFileAsync(string name, Stream content, string contentType, long totalFileSize)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Creates new WebDAV folder with the specified name in this folder.
        /// </summary>
        /// <param name="name">Name of the folder to create.</param>
        public async Task<IFolder> CreateFolderAsync(string name)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Deletes this item.
        /// </summary>
        /// <param name="multistatus">If some items fail to delete but operation in whole shall be continued, add
        /// information about the error into <paramref name="multistatus"/> using
        /// <see cref="MultistatusException.AddInnerException(string,ITHit.WebDAV.Server.DavException)"/>.
        /// </param>
        public async Task DeleteAsync(MultistatusException multistatus)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Gets values of all properties or selected properties for this item.
        /// </summary>
        /// <param name="props">
        /// Array of property names which values are requested. 
        /// If a property does not exist for this hierarchy item then the property value shall not be returned.
        /// </param>
        /// <param name="allprop">
        /// If it is <c>true</c> it means that besides properties listed in <paramref name="props"/> you need to 
        /// return all properties you think may be useful to client.
        /// </param>
        /// <returns>
        /// Enumerable with property values.
        /// </returns>
        public async Task<IEnumerable<PropertyValue>> GetPropertiesAsync(IList<PropertyName> props, bool allprop)
        {
            throw new NotImplementedException();
        }

        public async Task<IEnumerable<PropertyName>> GetPropertyNamesAsync()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Modifies and removes properties for this item.
        /// </summary>
        /// <param name="setProps">Array of properties to be set.</param>
        /// <param name="delProps">
        /// Property names to be removed. Properties that don't exist shall be skipped.
        /// </param>
        /// <param name="multistatus">The standard requires this operation to be transactional.
        /// If some properties fail to update but there is no possibility to rollback the transaction
        /// in <see cref="ContextAsync{IHierarchyItem}.BeforeResponseAsync"/>, add
        /// information about the error into <paramref name="multistatus"/> 
        /// using <see cref="MultistatusException.AddInnerException(string,ITHit.WebDAV.Server.PropertyName,ITHit.WebDAV.Server.DavException)"/>.
        /// In this case engine will report correct statuses for all properties at least
        /// (although this is against standard).
        /// </param>
        /// <remarks>
        /// <para>
        /// In your <c>UpdateProperties</c> implementation you will create,
        /// modify and delete item properties.
        /// Engine will update properties (call these methods) one by one unless exception is thrown.
        /// If an exception is thrown during a property update engine will report all remaining properties
        /// as failed with status <see cref="DavStatus.FAILED_DEPENDENCY"/>
        /// </para>
        /// <para>
        /// The standard requires that request which updates properties is atomic (PROPPATCH).
        /// If your storage supports transactions then atomicity requirement can be implemented
        /// by committing or rollbacking the transaction in <see cref="ContextAsync{IHierarchyItem}.BeforeResponseAsync"/>.
        /// </para>
        /// </remarks>
        public async Task UpdatePropertiesAsync(
            IList<PropertyValue> setProps,
            IList<PropertyName> delProps,
            MultistatusException multistatus)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Unique item path in the repository relative to storage root.
        /// </summary>
        /// <remarks>
        /// <para>
        /// The URL returned by this property is relative to storage root.
        /// If your server root is located at http://example.webdavsystem.com:8080/myserver/ and the item URL is
        /// http://example.webdavsystem.com:8080/myserver/myfolder/myitem.doc this property implementation must
        /// return myfolder/myitem.doc. To calculate the entire item URL the engine will
        /// call <see cref="DavRequest.ApplicationPath"/> property and attach it to url returned by
        /// <see cref="IHierarchyItem.Path"/> property.
        /// </para>
        /// <para>
        /// Every part of the path (between '/' characters) shall be encoded,
        /// for example using <see cref="EncodeUtil"/>.
        /// </para>
        /// <para>Examples:
        /// <list type="bullet">
        /// <item><description>File: myfolder/my%20doc.docx</description></item>
        /// <item><description>Folder: myfolder/folder/</description></item>
        /// <item><description>History item: myfolder/mydoc.docx?history</description></item>
        /// <item><description>Version: myfolder/mydoc.docx?version=5</description></item>
        /// </list>
        /// </para>
        /// </remarks>
        /// <value><c>String</c> representing relative item path in the repository.</value>
        public string Path
        {
            get { throw new NotImplementedException(); }
        }

        /// <summary>
        /// Moves this item to the destination folder under a new name.
        /// </summary>
        /// <param name="destFolder">Destination folder.</param>
        /// <param name="destName">Name of the destination item.</param>
        /// <param name="multistatus">If some items fail to copy but operation in whole shall be continued, add
        /// information about the error into <paramref name="multistatus"/> using 
        /// <see cref="MultistatusException.AddInnerException(string,ITHit.WebDAV.Server.DavException)"/>.
        /// </param>
        /// <remarks>
        /// <papa>
        /// If the item is locked the server must not move any locks with the item. However, items must be added to an
        /// existing lock at the destination.
        /// </papa>
        /// </remarks>
        public async Task MoveToAsync(IItemCollection destFolder, string destName, MultistatusException multistatus)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Gets the name of the item in repository.
        /// </summary>
        public string Name
        {
            get { throw new NotImplementedException(); }
        }

        /// <summary>
        /// Gets the creation date of the item in repository expressed as the coordinated universal time (UTC).
        /// </summary>
        public DateTime Created
        {
            get { throw new NotImplementedException(); }
        }

        /// <summary>
        /// Gets the last modification date of the item in repository expressed as the coordinated universal time (UTC).
        /// </summary>
        public DateTime Modified
        {
            get { throw new NotImplementedException(); }
        }

        /// <summary>
        /// Creates a copy of this item with a new name in the destination folder.
        /// </summary>
        /// <param name="destFolder">Destination folder.</param>
        /// <param name="destName">Name of the destination item.</param>
        /// <param name="deep">Indicates whether to copy entire subtree.</param>
        /// <param name="multistatus">If some items fail to copy but operation in whole shall be continued, add
        /// information about the error into <paramref name="multistatus"/> using 
        /// <see cref="MultistatusException.AddInnerException(string,ITHit.WebDAV.Server.DavException)"/>.
        /// </param>
        /// <remarks>
        /// <para>
        /// If error occurred while copying file located in a subtree, the server 
        /// should try to continue copy operation and copy all other items. In this case 
        /// you must add that error <paramref name="multistatus"/> container.
        /// </para>
        /// <para>
        /// A CopyTo method invocation must not copy any locks active on the source item.
        /// However, if this method copies the item into a folder that has a deep lock,
        /// then the destination item must be added to the lock.
        /// </para>
        /// </remarks>
        public async Task CopyToAsync(IItemCollection destFolder, string destName, bool deep, MultistatusException multistatus)
        {
            throw new NotImplementedException();
        }

        //!<# if (Params.ACL) { #>
        /// <summary>
        /// Retrieves a particular principal as being the "owner" of the item. Since the owner of a
        /// resource often has special access control capabilities (e.g., the owner frequently has permanent
        /// <see cref="Privilege.WriteAcl"/> privilege), clients might display the resource owner in their
        /// user interface.
        /// </summary>
        public async Task SetOwnerAsync(IPrincipal value)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Retrieves a particular principal as being the "owner" of the item. Since the owner of a
        /// resource often has special access control capabilities (e.g., the owner frequently has permanent
        /// <see cref="Privilege.WriteAcl"/> privilege), clients might display the resource owner in their user
        /// interface.
        /// </summary>
        public async Task<IPrincipal> GetOwnerAsync()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Retrieves a particular principal as being the "group" of the item. This property is commonly
        /// found on repositories that implement the Unix privileges model.
        /// </summary>
        public async Task SetGroupAsync(IPrincipal value)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Retrieves a particular principal as being the "group" of the item. This property is commonly
        /// found on repositories that implement the Unix privileges model.
        /// </summary>
        /// <remarks>
        /// Can return null if group is not assigned.
        /// </remarks>
        public async Task<IPrincipal> GetGroupAsync()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Retrieves the privileges defined for the resource.
        /// </summary>
        public async Task<IEnumerable<SupportedPrivilege>> GetSupportedPrivilegeSetAsync()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Retrieves the exact set of privileges (as computed by
        /// the server) granted to the currently authenticated HTTP user. Aggregate privileges and their contained
        /// privileges are listed. A user-agent can use the value of this property to adjust its user interface to
        /// make actions inaccessible (e.g., by graying out a menu item or button) for which the current principal
        /// does not have permission. This property is also useful for determining what operations the current
        /// principal can perform, without having to actually execute an operation.
        /// </summary>
        public async Task<IEnumerable<Privilege>> GetCurrentUserPrivilegeSetAsync()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Specifies the list of access control entries (ACEs), which define what principals
        /// are to get what privileges for this resource.
        /// </summary>
        /// <param name="propertyNames">List of properties that might be retrieved from those <see cref="IPrincipal"/>
        /// returned.</param>
        /// <remarks>
        /// <paramref name="propertyNames"/>Is for optimization purposes and you may ignore it.
        /// For "principal-pop-set" report engine will call this method, retrieve all principals from result
        /// and extract properties specified in <paramref name="propertyNames"/> from it. In this case prefetching
        /// these properties may improve performance.
        /// </remarks>
        public async Task<IEnumerable<ReadAce>> GetAclAsync(IList<PropertyName> propertyNames)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Sets list of access control entries (ACEs), which define what principals
        /// are to get what privileges for this resource.
        /// </summary>
        /// <remarks>
        /// Two common operations are to add or remove an ACE from an existing access
        /// control list. To accomplish this, a client uses the PROPFIND method to retrieve the value of the DAV:acl
        /// property, then parses the returned access control list to remove all inherited and protected ACEs (these
        /// ACEs are tagged with the DAV:inherited and DAV:protected XML elements). In the remaining set of
        /// non-inherited, non-protected ACEs, the client can add or remove one or more ACEs before submitting the
        /// final ACE set in the request body of the ACL method.
        /// It is possible that the ACEs visible to the current user in the DAV:acl property may only be a portion of
        /// the complete set of ACEs on that resource. If this is the case, an ACL request only modifies the set of
        /// ACEs visible to the current user, and does not affect any non-visible ACE.
        /// </remarks>
        public async Task SetAclAsync(IList<WriteAce> aces)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Defines the types of ACLs supported by this server, to avoid clients needlessly getting
        /// errors. When a client tries to set an ACL via the ACL method, the server may reject the attempt to set
        /// the ACL as specified.
        /// </summary>
        public async Task<AclRestriction> GetAclRestrictionsAsync()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Contains a set of items that also control the access to this item.
        /// To have a privilege on an item, not only must the ACL on that item (specified in the
        /// <see cref="ITHit.WebDAV.Server.Acl"/>
        /// property of that item) grant the privilege, but so must the ACL of each items identified in the
        /// <see cref="IAclHierarchyItem.GetInheritedAclSetAsync"/> property of that item. Effectively,
        /// the privileges granted by the current
        /// ACL are ANDed with the privileges granted by each inherited ACL.
        /// </summary>
        public async Task<IEnumerable<IHierarchyItem>> GetInheritedAclSetAsync()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Retrieves a set of root collections that contain the
        /// principals that are available on the server that implements this resource. It can be used to retrieve the 
        /// <see cref="IHierarchyItem.Name"/> properties
        /// of all principals on that server, thereby yielding human-readable
        /// names for each principal that could be displayed in a user interface.
        /// </summary>
        public async Task<IEnumerable<IPrincipalFolder>> GetPrincipalCollectionSetAsync()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Returns predefined principal.
        /// </summary>
        /// <param name="wellKnownPrincipal">One of <see cref="WellKnownPrincipal"/> values.</param>
        /// <returns>Well known principal or <c>null</c> if it is not supported.</returns>
        public async Task<IPrincipal> ResolveWellKnownPrincipalAsync(WellKnownPrincipal wellKnownPrincipal)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Find all resources in the subtree which have either Group or Owner (defined by <paramref name="matchBy"/>
        /// parameter) which identifies current user or group current user belongs to.
        /// For example, this report can return all of the resources in a collection
        /// hierarchy that are owned by the current user.
        /// </summary>
        /// <param name="matchBy">Identifies whether to search by owner or group.</param>
        /// <param name="props">Properties requested for found items.</param>
        /// <returns>Matching resources.</returns>
        public async Task<IEnumerable<IAclHierarchyItem>> GetItemsByPropertyAsync(MatchBy matchBy, IList<PropertyName> props)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Retrieves currently logged in principal.
        /// </summary>
        /// <returns>Currently logged in principal.</returns>
        public async Task<IPrincipal> GetCurrentUserPrincipalAsync()
        {
            throw new NotImplementedException();
        }

        //!<# } #>
        
        //!<# if (Params.Class2) { #>
        /// <summary>
        /// Gets the array of all locks for this item.
        /// </summary>
        /// <remarks>
        /// <para>
        /// This property must return all locks for the item including deep locks on any of the parent folders.
        /// All fields of each <see cref="LockInfo"/> structure in the array must be set.
        /// </para>
        /// </remarks>
        public async Task<IEnumerable<LockInfo>> GetActiveLocksAsync()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Locks this item.
        /// </summary>
        /// <param name="level">Whether lock is share or exclusive</param>
        /// <param name="isDeep">Whether lock is deep.</param>
        /// <param name="requestedTimeOut">Lock timeout which was requested by client.
        /// Server may ignore this parameter and set any timeout.</param>
        /// <param name="owner">Owner of the lock as specified by client.</param> 
        /// <returns>
        /// Instance of <see cref="LockResult"/> with information about the lock.
        /// </returns>
        /// <remarks>
        /// WebDAV engine passes <see cref="LockInfo"/> structure to this method by reference.
        /// All fields of the structure are provided by WebDAV client except <see cref="LockInfo.Token"/> field.
        /// In your <c>Lock</c> implementation you must create lock token and set <see cref="LockInfo.Token"/> member.
        /// You must also associate generated token with the hierarchy item in the repository during this call.
        /// The token is sent to the WebDAV client.
        /// </remarks>
        public async Task<LockResult> LockAsync(LockLevel level, bool isDeep, TimeSpan? requestedTimeOut, string owner)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Updates lock timeout information on this item.
        /// </summary>
        /// <param name="token">Lock token.</param>
        /// <param name="requestedTimeOut">Lock timeout which was requested by client.
        /// Server may ignore this parameter and set any timeout.</param>
        /// <returns>
        /// Instance of <see cref="LockResult"/> with information about the lock.
        /// </returns>
        public async Task<RefreshLockResult> RefreshLockAsync(string token, TimeSpan? requestedTimeOut)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Removes lock with the specified token from this item.
        /// </summary>
        /// <param name="lockToken">Lock with this token should be removed from the item.</param>
        /// <remarks>
        /// <para>
        /// If this lock included more than one hierarchy item, the lock is removed from all items included in the lock.
        /// </para>
        /// </remarks>
        public async Task UnlockAsync(string lockToken)
        {
            throw new NotImplementedException();
        }
        //!<# } #>

        //!<# if (Params.Quota) { #>
        /// <summary>
        /// Value in bytes representing the amount of additional disk space beyond the current
        /// allocation that can be allocated to the folder(or other item) before further
        /// allocations will be refused.  It is understood that this space may be
        /// consumed by allocations to other files/folders.
        /// </summary>
        /// <returns>Bytes that can be additionally allocated in folder/file.</returns>
        public async Task<long> GetAvailableBytesAsync()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        ///  Value in bytes representing the amount of space used by this folder/file
        ///  and possibly a number of other similar folders/files, where the set of "similar" meets at least
        ///  the criterion that allocating space to any folder/file in the set will
        ///  count against the <see cref="IQuota.GetAvailableBytesAsync"/>.  It MUST include the
        ///  total count including usage derived from sub-items if
        ///  appropriate.  It SHOULD include metadata storage size if metadata
        ///  storage is counted against the <see cref="IQuota.GetAvailableBytesAsync"/>
        /// </summary>
        /// <returns>Bytes occupied by folder/file.</returns>
        public async Task<long> GetUsedBytesAsync()
        {
            throw new NotImplementedException();
        }
        //!<# } #>

        //!<# if (Params.MicrosoftExtensions) { #>
        /// <summary>
        /// Retrieves file attributes.
        /// </summary>
        /// <returns>File attributes.</returns>
        public async Task<FileAttributes> GetFileAttributesAsync()
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// Updates file attributes.
        /// </summary>
        /// <param name="value">File attributes.</param>
        public async Task SetFileAttributesAsync(FileAttributes value)
        {
            throw new NotImplementedException();
        }
        //!<# } #>
    }
}
