﻿//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (Params.Hosting != HostingEnvironment.NetCore) return string.Empty; #>
using Microsoft.Extensions.Configuration;
using Microsoft.AspNetCore.Hosting;
using System;
using System.IO;
using System.Threading.Tasks;

//!<# if (Params.DeltaV) { #>
using ITHit.WebDAV.Server.DeltaV;
//!<# } #>
//!<# if (Params.Storage == StorageType.Ntfs) { #>
using ITHit.WebDav.Server.SampleModel.ExtendedAttributes;
//!<# } #>
namespace ITHit.WebDav.Server.SampleModel.__NetCore__.Configuration
{
    /// <summary>
    /// Represents WebDAV Context configuration.
    /// </summary>
    public class DavContextConfig
    {
        //!<# if (Params.Storage == StorageType.Ntfs) { #>
        /// <summary>
        /// Files and folders in this folder become available via WebDAV.
        /// </summary>
        public string RepositoryPath { get; set; } = string.Empty;

        /// <summary>
        /// File system search provider. 
        /// </summary>
        public string WindowsSearchProvider { get; set; } = string.Empty;
        /// <summary>
        /// This folder used for extended attributes storage.
        /// </summary>
        public string AttrStoragePath { get; set; } = string.Empty;
        //!<# } #>
        //!<# if (Params.Storage == StorageType.Db) { #>
        /// <summary>
        /// Database conntion string.
        /// </summary>
        public string ConnectionString { get; set; } = string.Empty;
        //!<# } #>
        //!<# if (Params.DeltaV) { #>
        public AutoVersion AutoVersionMode { get; set; }
        //!<# } #>
       
    }

    /// <summary>
    /// Binds, validates and normalizes WebDAV Context configuration.
    /// </summary>
    public static class DavContextConfigValidator
    {
        /// <summary>
        /// Binds, validates and normalizes WebDAV Context configuration.
        /// </summary>
        /// <param name="configurationSection">Instance of <see cref="IConfigurationSection"/>.</param>
        /// <param name="config">WebDAV Context configuration.</param>
        //!<# if (Params.NetCoreVersionNum >= 3) { #>
        /// <param name="env">Instance of <see cref="IWebHostEnvironment"/>.</param>
        public static async Task ReadConfigurationAsync(this IConfigurationSection configurationSection, DavContextConfig config, IWebHostEnvironment env)
        //!<# } else { #>
        /// <param name="env">Instance of <see cref="IHostingEnvironment"/>.</param>
        public static async Task ReadConfigurationAsync(this IConfigurationSection configurationSection, DavContextConfig config, IHostingEnvironment env)
        //!<# } #>
        {
            if (configurationSection == null)
            {
                throw new ArgumentNullException("configurationSection");
            }

            configurationSection.Bind(config);

            //!<# if (Params.Storage == StorageType.Ntfs) { #>
            if (string.IsNullOrEmpty(config.RepositoryPath))
            {
                throw new ArgumentNullException("DavContext.RepositoryPath");
            }

            if (!Path.IsPathRooted(config.RepositoryPath))
            {
                config.RepositoryPath = Path.GetFullPath(Path.Combine(env.ContentRootPath, config.RepositoryPath));
            }

            //!<# if (!(Params.CalDav || Params.CardDav)) { #>
            if (!Directory.Exists(config.RepositoryPath))
            {
                throw new DirectoryNotFoundException(string.Format("DavContext.RepositoryPath specified in appsettings.webdav.json is invalid: '{0}'.", config.RepositoryPath));
            }
            //!<# } #>

            if (!(string.IsNullOrEmpty(config.AttrStoragePath) || Path.IsPathRooted(config.AttrStoragePath)))
            {
                config.AttrStoragePath = Path.GetFullPath(Path.Combine(env.ContentRootPath, config.AttrStoragePath));
            }

            if (!string.IsNullOrEmpty(config.AttrStoragePath))
            {
                FileSystemInfoExtension.UseFileSystemAttribute(new FileSystemExtendedAttribute(config.AttrStoragePath, config.RepositoryPath));
            }
            //!<# } #>
            //!<# if (Params.Storage == StorageType.Db) { #>
            if (string.IsNullOrEmpty(config.ConnectionString))
            {
                throw new ArgumentNullException("DavContext.ConnectionString");
            }
            //!<# } #>
        }
    }
}
