﻿//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (Params.Hosting != HostingEnvironment.NetCore) return string.Empty; #>
//!<# if (!(Params.CalDav || Params.CardDav)) return string.Empty; #>
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Http;
using System.Threading.Tasks;

using ITHit.Server;
using ITHit.WebDAV.Server;
//!<# if (Params.CalDav) { #>
using ITHit.WebDav.Server.SampleModel.__CalDavSql__.CalDav;
//!<# } #>
//!<# if (Params.CardDav) { #>
using ITHit.WebDav.Server.SampleModel.__CalDavSql__.CardDav;
//!<# } #>

namespace ITHit.WebDav.Server.SampleModel.__NetCore__
{
    public class ProvisioningMiddleware
    {
        /// <summary>
        /// Next middleware instance.
        /// </summary>
        private readonly RequestDelegate next;

        /// <summary>
        /// Initializes new instance of this class.
        /// </summary>
        /// <param name="next">Next middleware instance.</param>
        public ProvisioningMiddleware(RequestDelegate next)
        {
            this.next = next;
        }

        /// <summary>
        /// Processes Provisioning request.
        /// </summary>
        public async Task Invoke(HttpContext context, ContextCoreAsync<IHierarchyItem> davContext)
        {
            if (context.User != null && context.User.Identity != null && context.User.Identity.IsAuthenticated)
            {
                //!<# if (Params.CalDav) { #>
                await Provisioning.CreateCalendarFoldersAsync(davContext as DavContext);
                //!<# } #>
                //!<# if (Params.CardDav) { #>
                await Provisioning.CreateAddressbookFoldersAsync(davContext as DavContext);
                //!<# } #>
            }

            await next.Invoke(context);
        }
    }

    /// <summary>
    /// Extension methods to add Provisioning capabilities to an HTTP application pipeline.
    /// </summary>
    public static class ProvisioningeMiddlewareExtensions
    {
        /// <summary>
        /// Adds a capabilities middleware type to the application's request pipeline.
        /// </summary>
        /// <param name="builder">The <see cref="IApplicationBuilder"/> instance.</param>
        public static IApplicationBuilder UseProvisioninge(this IApplicationBuilder builder)
        {
            return builder.UseMiddleware<ProvisioningMiddleware>();
        }
    }
}
