﻿//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (Params.AuthenticationType == AuthenticationType.Ntlm) { #>
using Microsoft.AspNetCore.Authentication.Negotiate;
//!<# } #>
//!<# if (Params.Hosting != HostingEnvironment.NetCore || Params.IsUpdateExistingSolution) return string.Empty; #>
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
//!<# if (Params.NetCoreVersionNum >= 3) { #>
using Microsoft.AspNetCore.Http;
//!<# } #>
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.AspNetCore.Server.Kestrel.Core;
//!<# if (Params.NetCoreVersionNum >= 3) { #>
using Microsoft.Extensions.Hosting;
//!<# } else { #>
using Microsoft.Extensions.Logging;
//!<# } #>
//!<# if (Params.AuthenticationType.HasFlag(AuthenticationType.Msofba)) { #>
using __NetCore__.WebDAVServerImpl.MSOFBAuthentication;
//!<# } #>

namespace ITHit.WebDav.Server.SampleModel.__NetCore__
{
    public class Startup
    {
        //!<# if (Params.NetCoreVersionNum >= 3) { #>
        public Startup(IWebHostEnvironment env)
        //!<# } else { #>
        public Startup(IHostingEnvironment env)
        //!<# } #>
        {
            var builder = new ConfigurationBuilder()
                .SetBasePath(env.ContentRootPath)
                .AddJsonFile("appsettings.json", optional: true, reloadOnChange: true)
                .AddJsonFile("appsettings.webdav.json", optional: true, reloadOnChange: true)
                .AddJsonFile($"appsettings.{env.EnvironmentName}.json", optional: true);

            HostingEnvironment = env;
            builder.AddEnvironmentVariables();
            Configuration = builder.Build();
        }

        public IConfigurationRoot Configuration { get; }
        //!<# if (Params.NetCoreVersionNum >= 3) { #>
        public IWebHostEnvironment HostingEnvironment { get; }
        //!<# } else { #>
        public IHostingEnvironment HostingEnvironment { get; }
        //!<# } #>

        // This method gets called by the runtime. Use this method to add services to the container.
        // For more information on how to configure your application, visit http://go.microsoft.com/fwlink/?LinkID=398940
        public void ConfigureServices(IServiceCollection services)
        {
            //!<# if(Params.AuthenticationType == AuthenticationType.Ntlm) { #>

            // Enable Windows authentication.
            services.AddAuthentication(NegotiateDefaults.AuthenticationScheme).AddNegotiate();
            services.AddAuthorization(options =>
            {
                options.FallbackPolicy = options.DefaultPolicy;
            });
            //!<# } #>
            services.AddWebDav(Configuration, HostingEnvironment);
            //!<# if(Params.GSuite) { #>

            //Enables documents editing and preview in Google G Suite online editor.
            services.AddGSuite(Configuration);
            //!<# } #>
            //!<# if(Params.WebSockets) { #>

            //Enables web sockets. Web sockets are used to update the documents list in case of any changes on the server.
            services.AddSingleton<WebSocketsService>();
            //!<# } #>      
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        //!<# if (Params.NetCoreVersionNum >= 3) { #>
        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        //!<# } else { #>
        public void Configure(IApplicationBuilder app, IHostingEnvironment env)
        //!<# } #>
        {
            app.UseStaticFiles(new StaticFileOptions { ServeUnknownFileTypes = true });

            if (env.IsDevelopment())
                app.UseDeveloperExceptionPage();

            //!<# if(Params.AuthenticationType.HasFlag(AuthenticationType.Basic) && Params.BuildType != BuildType.Server) { #>
            // Basic auth requires SSL connection. To enable non - SSL connection for testing purposes read the following articles:
            // - In case of Windows & MS Office: http://support.microsoft.com/kb/2123563
            // - In case of Mac OS X & MS Office: https://support.microsoft.com/en-us/kb/2498069
            app.UseBasicAuth();
            //!<# } #>
            //!<# if(Params.AuthenticationType.HasFlag(AuthenticationType.Digest) && Params.BuildType != BuildType.Server) { #>
            app.UseDigestAuth();
            //!<# } #>
            //!<# if(Params.BuildType == BuildType.Server) { #>

            // Basic auth requires SSL connection. To enable non - SSL connection for testing purposes read the following articles:
            // - In case of Windows & MS Office: http://support.microsoft.com/kb/2123563
            // - In case of Mac OS X & MS Office: https://support.microsoft.com/en-us/kb/2498069
            //app.UseBasicAuth();
            //app.UseDigestAuth();
            //!<# } #>

            //!<# if(Params.CalDav || Params.CardDav) { #>
            app.UseProvisioninge();
            //!<# } #>
            //!<# if(Params.AuthenticationType == AuthenticationType.Ntlm) { #>

            // Enable Windows authentication.
            app.UseAuthentication();
            app.UseAuthorization();
            //!<# } #>
            //!<# if(Params.WebSockets) { #>

            //Enables web sockets. Web sockets are used to update the documents list in case of any changes on the server.
            app.UseWebSockets();
            app.UseWebSocketsMiddleware();
            //!<# } #>
            //!<# if(Params.GSuite) { #>

            //Enables documents editing and preview in Google G Suite online editor.
            app.UseGSuite();
            //!<# } #>
            app.UseWebDav(HostingEnvironment);
        }
    }
}
