//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (!Params.ACL)  return string.Empty; #>
//!<# if (Params.Storage != StorageType.Ntfs)  return string.Empty; #>
//!<# if ( !Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm) ) return string.Empty; #>
using System;
using System.Collections.Generic;
using System.Threading.Tasks;

using ITHit.WebDAV.Server;
using ITHit.WebDAV.Server.Paging;

namespace ITHit.WebDav.Server.SampleModel.__NtfsStorage__.Acl
{
    /// <summary>
    /// Logical folder right under webdav root with name 'acl' which
    /// contains folders 'users' and 'groups'.
    /// </summary>
    public class AclFolder : LogicalFolder
    {
        /// <summary>
        /// Path to current logical folder which contains users and groups.
        /// </summary>
        public static string PREFIX = "acl";

        /// <summary>
        /// Gets path to current logical folder which contains users and groups.
        /// </summary>
        private static string PATH
        {
            get { return PREFIX + "/"; }
        }

        //!<# if (false) {#>
        //TODO: check read privilege in all property gets?
        //!<# } #>
        /// <summary>
        /// Initializes a new instance of the AclFolder class.
        /// </summary>
        /// <param name="context">Instace of <see cref="DavContext"/>.</param>
        public AclFolder(DavContext context) : base(context, "acl", PATH)
        {
        }

        /// <summary>
        /// Retrieves children of /acl folder.
        /// We have here 'user' and 'group' folder for holding users and groups respectively.
        /// </summary>
        /// <param name="propNames">Property names to be fetched lated.</param>
        /// <returns>Children of this folder.</returns>
        public override async Task<PageResults> GetChildrenAsync(IList<PropertyName> propNames, long? offset, long? nResults, IList<OrderProperty> orderProps)
        {
            IList<IHierarchyItem> children = new List<IHierarchyItem>();
            children.Add(new UserFolder(Context));
            children.Add(new GroupFolder(Context));
            return new PageResults(children, null);
        }
    }
}
