//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (!Params.ACL)  return string.Empty; #>
//!<# if (Params.Storage != StorageType.Ntfs)  return string.Empty; #>
//!<# return string.Empty; #>
using System;
using System.Runtime.InteropServices;
using System.Security.AccessControl;
using ITHit.WebDav.Server.SampleModel.__NtfsStorage__.NtfsStreamsEngine;

namespace ITHit.WebDav.Server.SampleModel.__NtfsStorage__.Acl
{
    internal static class NtfsEffectiveRights
    {
        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Unicode)]
        public struct Trustee
        {
            public IntPtr pMultipleTrustee;
            public int MultipleTrusteeOperation;
            public int TrusteeForm;
            public int TrusteeType;
            [MarshalAs(UnmanagedType.LPTStr)]
            public string name;
        }

        [DllImport("advapi32.dll", CharSet = CharSet.Unicode)]
        internal static extern int GetEffectiveRightsFromAclW(byte[] pAcl, ref Trustee trustee, out uint pAccessMask);

        public static FileSystemRights GetEffectiveRights(string path, string user)
        {
            FileSecurity fss = new FileSecurity(path, AccessControlSections.Access);
            byte[] rawsd = fss.GetSecurityDescriptorBinaryForm();
            CommonSecurityDescriptor csd = new CommonSecurityDescriptor(true, false, rawsd, 0);
            int aclLen = csd.DiscretionaryAcl.BinaryLength;
            byte[] rawAcl = new byte[aclLen];
            csd.DiscretionaryAcl.GetBinaryForm(rawAcl, 0);
            uint mask;
            Trustee trustee = new Trustee();
            trustee.TrusteeForm = 1;
            trustee.TrusteeType = 1;
            trustee.name = user;
            int result = GetEffectiveRightsFromAclW(rawAcl, ref trustee, out mask);
            if (result != 0)
            {
                SafeNativeMethods.ThrowLastWin32Error();
            }

            return (FileSystemRights)mask;
        }
    }
}
