//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (!Params.CalDav)  return string.Empty; #>
//!<# if (Params.Storage != StorageType.Ntfs)  return string.Empty; #>
//!<# if (true)  return string.Empty; #>
using System;
using System.IO;
using System.Text.RegularExpressions;
using System.Collections.Generic;
using System.Threading.Tasks;

using ITHit.WebDAV.Server;
using ITHit.WebDAV.Server.CalDav;

namespace ITHit.WebDav.Server.SampleModel.__NtfsStorage__.CalDav
{
    /// <summary>
    /// Represents outbox folder.
    /// </summary>
    /// <remarks>
    /// Folders of this type correspond to the following path: /calendars/[user_name]/outbox/.
    /// </remarks>
    public class ScheduleOutboxFolder : DavFolder, IScheduleOutboxFolder
    {
        public static readonly string ScheduleOutboxFolderName = "outbox";

        /// <summary>
        /// Returns outbox folder that corresponds to path.
        /// </summary>
        /// <param name="context">Instance of <see cref="DavContext"/></param>
        /// <param name="path">Encoded path relative to WebDAV root.</param>
        /// <returns>ScheduleOutboxFolder instance or null if not found.</returns>
        public static ScheduleOutboxFolder GetScheduleOutboxFolder(DavContext context, string path)
        {
            string pattern = string.Format("^/?{0}/(?<user_name>[^/]+)/{1}/?",
                                           CalendarsRootFolder.CalendarsRootFolderName, ScheduleOutboxFolderName);
            if (!Regex.IsMatch(path, pattern))
                return null;

            DirectoryInfo folder = new DirectoryInfo(context.MapPath(path));
            if (!folder.Exists)
                return null;

            return new ScheduleOutboxFolder(folder, context, path);
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ScheduleOutboxFolder"/> class.
        /// </summary>
        /// <param name="directoryInfo">Instance of <see cref="DirectoryInfo"/> class with information about the folder in file system.</param>
        /// <param name="context">Instance of <see cref="DavContext"/>.</param>
        /// <param name="path">Relative to WebDAV root folder path.</param>
        private ScheduleOutboxFolder(DirectoryInfo directoryInfo, DavContext context, string path)
            : base(directoryInfo, context, path)
        {
        }

        /// <summary>
        /// Returns a list of calendar files requested by the client.
        /// </summary>
        /// <remarks>
        /// <para>
        /// This method is called by the Engine during <b>calendar-multiget</b> call.
        /// </para>
        /// <para>
        /// For each item from the <b>pathList</b> parameter return an item that corresponds to path or null if the item is not found.
        /// </para>
        /// </remarks>
        /// <param name="pathList">Calendar files path list.</param>
        /// <param name="propNames">
        /// Properties requested by the client. You can use this as a hint about what properties will be called by 
        /// the Engine for each item that are returned from this method.
        /// </param>
        /// <returns>List of calendar files. Returns null for any item that is not found.</returns>
        public async Task<IEnumerable<ICalendarFile>> MultiGetAsync(IEnumerable<string> pathList, IEnumerable<PropertyName> propNames)
        {
            throw new DavException("Not implemented.", DavStatus.NOT_IMPLEMENTED);
        }
    }
}
