//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (Params.Storage != StorageType.Ntfs)  return string.Empty; #>
using System;
//!<# if (Params.ACL && Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm)) { #>
using System.DirectoryServices.AccountManagement;
//!<# } #>
using System.IO;
using System.Linq;
using System.Net;
using System.Runtime.InteropServices;
using System.Security.Principal;

//!<# if (Params.Hosting == HostingEnvironment.IIS) { #>
using System.Web;
//!<# } #>
//!<# if ( (Params.Hosting == HostingEnvironment.IIS) || (Params.Hosting == HostingEnvironment.Listener) ) { #>
using System.Configuration;
//!<# } #>
using System.Threading.Tasks;
//!<# if (Params.Hosting == HostingEnvironment.NetCore) { #>
using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.Options;
//!<# } #>

using ITHit.Server;
using ITHit.WebDAV.Server;
using ITHit.WebDAV.Server.Acl;
using ITHit.WebDAV.Server.Quota;
//!<# if (Params.SynchronizationCollection) { #>
using ITHit.WebDAV.Server.Synchronization;
using ITHit.FileSystem.Core;
//!<# } #>
using ILogger = ITHit.Server.ILogger;
//!<# if (Params.ACL) { #>
using ITHit.WebDav.Server.SampleModel.__NtfsStorage__.Acl;
//!<# } #>
//!<# if (Params.CalDav) { #>
using ITHit.WebDav.Server.SampleModel.__NtfsStorage__.CalDav;
//!<# } #>
//!<# if (Params.CardDav) { #>
using ITHit.WebDav.Server.SampleModel.__NtfsStorage__.CardDav;
//!<# } #>
//!<# if (Params.Hosting == HostingEnvironment.NetCore) { #>
using ITHit.WebDav.Server.SampleModel.__NetCore__.Configuration;
//!<# } #>
//!<# if (Params.Storage == StorageType.Ntfs) { #>
using ITHit.WebDav.Server.SampleModel.__NtfsStorage__.ExtendedAttributes;
//!<# } #>

namespace ITHit.WebDav.Server.SampleModel.__NtfsStorage__
{
    /// <summary>
    /// Implementation of <see cref="ContextAsync{IHierarchyItem}"/>.
    /// Resolves hierarchy items by paths.
    /// </summary>
    public class DavContext :
        //!<# if (Params.Hosting == HostingEnvironment.IIS) { #>
        ContextWebAsync<IHierarchyItem>
        //!<# } #>
        //!<# if (Params.Hosting == HostingEnvironment.Listener) { #>
        ContextHttpListenerAsync<IHierarchyItem>
        //!<# } #>
        //!<# if (Params.Hosting == HostingEnvironment.NetCore) { #>
        ContextCoreAsync<IHierarchyItem>
        //!<# } #>
        /*!<# if (Params.ACL && Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm)) { #>!*/, IDisposable/*!<# } #>!*/
    {
        //!<# if (Params.CalDav || Params.CardDav) { #>

        /// <summary>
        /// Disk full windows error code.
        /// </summary>
        private const int ERROR_DISK_FULL = 0x70;
        //!<# } #>
        //!<# if (Params.ACL && Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm)) { #>

        /// <summary>
        /// A <see cref="PrincipalContext"/> to be used for windows users operations.
        /// </summary>
        private PrincipalContext principalContext;
        //!<# } #>

        //!<# if (Params.Hosting == HostingEnvironment.NetCore) { #>

        /// <summary>
        /// Context configuration
        /// </summary>
        public DavContextConfig Config { get; private set; }
        /// <summary>
        /// Path to the folder which become available via WebDAV.
        /// </summary>
        public string RepositoryPath { get => Config.RepositoryPath; }
        //!<# } else {#>

        /// <summary>
        /// Path to the folder which become available via WebDAV.
        /// </summary>
        public string RepositoryPath { get; private set; }
        //!<# } #>

        /// <summary>
        /// Gets WebDAV Logger instance.
        /// </summary>
        public ILogger Logger { get; private set; }

        //!<# if(Params.WebSockets) { #>

        /// <summary>
        /// Singleton instance of <see cref="WebSocketsService"/>.
        /// </summary>
        //!<# if(Params.Hosting == HostingEnvironment.NetCore) { #>
        public WebSocketsService socketService { get; private set; }
        //!<# } else { #>
        public WebSocketsService socketService { get { return WebSocketsService.Service; } private set { } }
        //!<# } #>
        //!<# } #>

        //!<# if ( Params.ACL || !Params.AuthenticationType.HasFlag(AuthenticationType.None)) { #>

        /// <summary>
        /// Gets user name.
        /// </summary>
        /// <remarks>In case of windows authentication returns user name without domain part.</remarks>
        public string UserName
        {
            get
            {
                int i = Identity.Name.IndexOf("\\");
                return i > 0 ? Identity.Name.Substring(i + 1, Identity.Name.Length - i - 1) : Identity.Name;
            }
        }

        /// <summary>
        /// Gets currently authenticated user.
        /// </summary>
        //!<# if (Params.ACL && Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm)) { #>
        public WindowsIdentity WindowsIdentity
        {
            get
            {
                WindowsIdentity winIdentity = Identity as WindowsIdentity;
                if (winIdentity != null && !winIdentity.IsAnonymous)
                {
                    return winIdentity;
                }

                if (AnonymousUser != null)
                {
                    return AnonymousUser;
                }

                throw new Exception("Anonymous user is not configured.");
            }
        }
        //!<# } #>

        /// <summary>
        /// Currently logged in identity.
        /// </summary>
        public IIdentity Identity { get; private set; } 
        //!<# } #>
        //!<# if (Params.ACL && Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm)) { #>

        /// <summary>
        /// Gets domain of currently logged in user.
        /// </summary>
        public string Domain
        {
            get
            {
                int i = WindowsIdentity.Name.IndexOf("\\");
                return i > 0 ? WindowsIdentity.Name.Substring(0, i) : Environment.MachineName;
            }
        }

        /// <summary>
        /// Gets or sets user configured as anonymous.
        /// </summary>
        public WindowsIdentity AnonymousUser { get; set; }

        /// <summary>
        /// Retrieves or creates <see cref="PrincipalContext"/> to be used for user related operations.
        /// </summary>
        /// <returns>Instance of <see cref="PrincipalContext"/>.</returns>
        public PrincipalContext GetPrincipalContext()
        {
            if (principalContext == null)
            {
                if (string.IsNullOrEmpty(Domain) ||
                    Environment.MachineName.Equals(Domain, StringComparison.InvariantCultureIgnoreCase))
                {
                    principalContext = new PrincipalContext(ContextType.Machine, Domain);
                }
                else
                {
                   principalContext = new PrincipalContext(ContextType.Domain);
                }
            }

            return principalContext;
        }

        /// <summary>
        /// Performs operation which creates, deletes or modifies user or group.
        /// Performs impersonification and exception handling.
        /// </summary>
        /// <param name="action">Action which performs action with a user or group.</param>
        public void PrincipalOperation(Action action)
        {
            using (impersonate())
            {
                try
                {
                    action();
                }
                catch (PrincipalOperationException ex)
                {
                    Logger.LogError("Principal operation failed", ex);
                    throw new DavException("Principal operation failed", ex);
                }
            }
        }

        /// <summary>
        /// Performs operation which queries, creates, deletes or modifies user or group.
        /// Performs impersonification and exception handling.
        /// </summary>
        /// <param name="func">Function to perform.</param>
        /// <typeparam name="T">Type of operation result.</typeparam>
        /// <returns>The value which <paramref name="func"/> returned.</returns>
        public T PrincipalOperation<T>(Func<T> func)
        {
            using (impersonate())
            {
                try
                {
                    return func();
                }
                catch (PrincipalOperationException ex)
                {
                    Logger.LogError("Principal operation failed", ex);
                    throw new DavException("Principal operation failed", ex);
                }
            }
        }
        //!<# } #>
        //!<# if (Params.Hosting == HostingEnvironment.IIS) { #>

        /// <summary>
        /// Initializes a new instance of the DavContext class.
        /// </summary>
        /// <param name="httpContext"><see cref="HttpContext"/> instance.</param>
        public DavContext(HttpContext httpContext) : base(httpContext)
        {
            Logger = ITHit.WebDav.Server.SampleModel.Logger.Instance;
            RepositoryPath = ConfigurationManager.AppSettings["RepositoryPath"] ?? string.Empty;
            bool isRoot = new DirectoryInfo(RepositoryPath).Parent == null;
            string configRepositoryPath = isRoot ? RepositoryPath : RepositoryPath.TrimEnd(Path.DirectorySeparatorChar);
            RepositoryPath = configRepositoryPath.StartsWith("~") ?
                HttpContext.Current.Server.MapPath(configRepositoryPath) : configRepositoryPath;

            string attrStoragePath = (ConfigurationManager.AppSettings["AttrStoragePath"] ?? string.Empty).TrimEnd(Path.DirectorySeparatorChar);
            attrStoragePath = attrStoragePath.StartsWith("~") ?
                HttpContext.Current.Server.MapPath(attrStoragePath) : attrStoragePath;

            if (!FileSystemInfoExtension.IsUsingFileSystemAttribute)
            {
                if (!string.IsNullOrEmpty(attrStoragePath))
                {
                    FileSystemInfoExtension.UseFileSystemAttribute(new FileSystemExtendedAttribute(attrStoragePath, this.RepositoryPath));
                }            
            }


            if (!Directory.Exists(RepositoryPath))
            {
                ITHit.WebDav.Server.SampleModel.Logger.Instance.LogError("Repository path specified in Web.config is invalid.", null);
            }
            //!<# if ( Params.ACL || !Params.AuthenticationType.HasFlag(AuthenticationType.None)) { #>

            if (httpContext.User != null)
                Identity = httpContext.User.Identity;
            //!<# } #>
        }
        //!<# } #>      
        //!<# if (Params.Hosting == HostingEnvironment.Listener) { #>

        /// <summary>
        /// Initializes a new instance of the DavContext class.
        /// </summary>
        /// <param name="listenerContext"><see cref="HttpListenerContext"/> instance.</param>
        /// <param name="prefixes">Http listener prefixes.</param>
        /// <param name="repositoryPath">Local path to repository.</param>
        /// <param name="logger"><see cref="ILogger"/> instance.</param>
        public DavContext(
            HttpListenerContext listenerContext,
            HttpListenerPrefixCollection prefixes,
            System.Security.Principal.IPrincipal principal,
            string repositoryPath,
            ILogger logger)
            : base(listenerContext, prefixes)
        {
            this.Logger = logger;
            this.RepositoryPath = repositoryPath;
            if (!Directory.Exists(repositoryPath))
            {
                Logger.LogError("Repository path specified in Web.config is invalid.", null);
            }
            //!<# if ( Params.ACL || !Params.AuthenticationType.HasFlag(AuthenticationType.None)) { #>

            if (principal != null)
                Identity = principal.Identity;
            //!<# } #>
        }
        //!<# } #>
        //!<# if (Params.Hosting == HostingEnvironment.NetCore) { #>

        //!<# if (Params.CalDav || Params.CardDav) { #>
        /// <summary>
        /// Represents array of users from storage.
        /// </summary>
        internal DavUser[] Users { get; private set; }
        //!<# } #>

        /// <summary>
        /// Initializes a new instance of the class.
        /// </summary>
        /// <param name="httpContextAccessor">Http context.</param>
        /// <param name="config">WebDAV Context configuration.</param>
        /// <param name="logger">WebDAV Logger instance.</param>
        //!<# if (Params.CalDav || Params.CardDav) { #>
        /// <param name="configUsers">WebDAV Users configuration.</param>
        //!<# } #>
        //!<# if(Params.WebSockets) { #>
        /// <param name="socketService">Singleton instance of <see cref="WebSocketsService"/>.</param>
        //!<# } #>
        public DavContext(IHttpContextAccessor httpContextAccessor, IOptions<DavContextConfig> config, ILogger logger
            //!<# if (Params.CalDav || Params.CardDav) { #>
            , IOptions<DavUsersConfig> configUsers
            //!<# } #>
            //!<# if(Params.WebSockets) { #>
            , WebSocketsService socketService
            //!<# } #>
            )
            : base(httpContextAccessor.HttpContext)
        {
            //!<# if ( Params.ACL || !Params.AuthenticationType.HasFlag(AuthenticationType.None) ) { #>
            HttpContext httpContext = httpContextAccessor.HttpContext;
            if (httpContext.User != null)
                Identity = httpContext.User.Identity;
            //!<# } #>
            //!<# if (Params.CalDav || Params.CardDav) { #>
            Users = configUsers.Value.Users;
            //!<# } #>
            Config = config.Value;
            Logger = logger;
            //!<# if(Params.WebSockets) { #>
            this.socketService = socketService;
            //!<# } #>
        }
        //!<# } #>

        //$<ContextAsync{IHierarchyItem}.GetHierarchyItemAsync
        /// <summary>
        /// Creates <see cref="IHierarchyItem"/> instance by path.
        /// </summary>
        /// <param name="path">Item relative path including query string.</param>
        /// <returns>Instance of corresponding <see cref="IHierarchyItem"/> or null if item is not found.</returns>
        public override async Task<IHierarchyItem> GetHierarchyItemAsync(string path)
        {
            path = path.Trim(new[] { ' ', '/' });

            //remove query string.
            int ind = path.IndexOf('?');
            if (ind > -1)
            {
                path = path.Remove(ind);
            }

            IHierarchyItem item = null;
            //!<# if (Params.ACL) { #>

            // Return items from [DAVLocation]/acl/ folder and subfolders.
            item = await AclFactory.GetAclItemAsync(this, path);
            if (item != null)
                return item;
            //!<# } #>
            //!<# if (Params.CalDav) { #>

            // Return items from [DAVLocation]/calendars/ folder and subfolders.
            item = CalDavFactory.GetCalDavItem(this, path);
            if (item != null)
                return item;
            //!<# } #>
            //!<# if (Params.CardDav) { #>

            // Return items from [DAVLocation]/addressbooks/ folder and subfolders.
            item = CardDavFactory.GetCardDavItem(this, path);
            if (item != null)
                return item;
            //!<# } #>
            //!<# if (Params.CalDav || Params.CardDav) { #>

            // Return folder that corresponds to [DAVLocation] path. If no DavLocation is defined in config file this is a website root.
            item = DavLocationFolder.GetDavLocationFolder(this, path);
            if (item != null)
                return item;
            //!<# } #>

            item = await DavFolder.GetFolderAsync(this, path);
            if (item != null/*!<# if (Params.SynchronizationCollection) { #>!*/ && (item as DavHierarchyItem).ChangeType != Change.Deleted/*!<# } #>!*/)
                return item;

            item = await DavFile.GetFileAsync(this, path);
            if (item != null/*!<# if (Params.SynchronizationCollection) { #>!*/ && (item as DavHierarchyItem).ChangeType != Change.Deleted/*!<# } #>!*/)
                return item;

            Logger.LogDebug("Could not find item that corresponds to path: " + path);

            return null; // no hierarchy item that corresponds to path parameter was found in the repository
        }
        //$>

        //!<# if (Params.SynchronizationCollection) { #>
        /// <summary>
        /// Returns the physical file path that corresponds to the specified virtual path on the Web server.
        /// </summary>
        /// <param name="relativePath">Path relative to WebDAV root folder, can be updated when file/folder id is used in url scheme.</param>
        /// <returns>Corresponding path in file system.</returns>
        internal string MapPath(ref string relativePath)
        {
            //Convert to local file system path by decoding every part, reversing slashes and appending
            //to repository root.
            string[] encodedParts = relativePath.Split(new[] { "/" }, StringSplitOptions.RemoveEmptyEntries);
            string[] decodedParts = encodedParts.Select<string, string>(EncodeUtil.DecodeUrlPart).ToArray();

            // check scheme with resource id
            //!<# if (string.IsNullOrEmpty(Params.DavUrlPrefix)) { #>
            if (relativePath.StartsWith("ID/"))
            //!<# } else { #>
            if (relativePath.StartsWith("<#=Params.DavUrlPrefix.Trim('/')#>/ID/"))
            //!<# } #>         
            {
                string fileSystemItemPath = FileSystemItem.GetPathByItemId(decodedParts[decodedParts.Length - 1]);
                relativePath = fileSystemItemPath.Substring(RepositoryPath.Length).Replace(Path.DirectorySeparatorChar.ToString(), "/");

                return fileSystemItemPath;
            }
            else
            {
                return Path.Combine(RepositoryPath, string.Join(Path.DirectorySeparatorChar.ToString(), decodedParts));
            }
        }
        //!<# }else{ #>
        /// <summary>
        /// Returns the physical file path that corresponds to the specified virtual path on the Web server.
        /// </summary>
        /// <param name="relativePath">Path relative to WebDAV root folder.</param>
        /// <returns>Corresponding path in file system.</returns>
        internal string MapPath(string relativePath)
        {
            //Convert to local file system path by decoding every part, reversing slashes and appending
            //to repository root.
            string[] encodedParts = relativePath.Split(new[] { "/" }, StringSplitOptions.RemoveEmptyEntries);
            string[] decodedParts = encodedParts.Select<string, string>(EncodeUtil.DecodeUrlPart).ToArray();
            return Path.Combine(RepositoryPath, string.Join(Path.DirectorySeparatorChar.ToString(), decodedParts));
        }
        //!<# } #>
        //!<# if (Params.CalDav || Params.CardDav) { #>

        /// <summary>
        /// Performs file system operation with translating exceptions to those expected by WebDAV engine.
        /// </summary>
        /// <param name="item">Item on which operation is performed.</param>
        /// <param name="action">The action to be performed.</param>
        /// <param name="privilege">Privilege which is needed to perform the operation. If <see cref="UnauthorizedAccessException"/> is thrown
        /// this method will convert it to <see cref="NeedPrivilegesException"/> exception and specify this privilege in it.</param>
        public void FileOperation(IHierarchyItem item, Action action, Privilege privilege)
        {
            try
            {
                //!<# if (Params.ACL && Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm)) { #>
                using (impersonate())
                {
                    action();
                }
                //!<# }else{ #>
                action();
                //!<# } #>
            }
            catch (UnauthorizedAccessException)
            {
                NeedPrivilegesException ex = new NeedPrivilegesException("Not enough privileges");
                ex.AddRequiredPrivilege(item.Path, privilege);
                throw ex;
            }
            catch (IOException ex)
            {
                int hr = Marshal.GetHRForException(ex);
                if (hr == ERROR_DISK_FULL)
                {
                    throw new InsufficientStorageException();
                }

                throw new DavException(ex.Message, DavStatus.CONFLICT);
            }
        }
        //!<# if (Params.TargetFramework45OrLater) { #>

        /// <summary>
        /// Performs file system operation with translating exceptions to those expected by WebDAV engine.
        /// </summary>
        /// <param name="item">Item on which operation is performed.</param>
        /// <param name="action">The action to be performed.</param>
        /// <param name="privilege">Privilege which is needed to perform the operation. If <see cref="UnauthorizedAccessException"/> is thrown
        /// this method will convert it to <see cref="NeedPrivilegesException"/> exception and specify this privilege in it.</param>
        public async Task FileOperationAsync(IHierarchyItem item, Func<Task> actionAsync, Privilege privilege)
        {
            try
            {
                //!<# if (Params.ACL && Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm)) { #>
                using (impersonate())
                {
                   await actionAsync();
                }
                //!<# }else{ #>
                await actionAsync();
                //!<# } #>
            }
            catch (UnauthorizedAccessException)
            {
                NeedPrivilegesException ex = new NeedPrivilegesException("Not enough privileges");
                ex.AddRequiredPrivilege(item.Path, privilege);
                throw ex;
            }
            catch (IOException ex)
            {
                int hr = Marshal.GetHRForException(ex);
                if (hr == ERROR_DISK_FULL)
                {
                    throw new InsufficientStorageException();
                }

                throw new DavException(ex.Message, DavStatus.CONFLICT);
            }
        }

        /// <summary>
        /// Performs file system operation with translating exceptions to those expected by WebDAV engine.
        /// </summary>
        /// <param name="item">Item on which operation is performed.</param>
        /// <param name="func">The action to be performed.</param>
        /// <param name="privilege">Privilege which is needed to perform the operation.
        /// If <see cref="UnauthorizedAccessException"/> is thrown  this method will convert it to
        /// <see cref="NeedPrivilegesException"/> exception and specify this privilege in it.</param>
        /// <typeparam name="T">Type of operation's result.</typeparam>
        /// <returns>Result returned by <paramref name="func"/>.</returns>
        public async Task<T> FileOperationAsync<T>(IHierarchyItem item, Func<Task<T>> actionAsync, Privilege privilege)
        {
            try
            {
                //!<# if (Params.ACL && Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm)) { #>
                using (impersonate())
                {
                    return await actionAsync();
                }
                //!<# }else{ #>
                return await actionAsync();
                //!<# } #>
            }
            catch (UnauthorizedAccessException)
            {
                NeedPrivilegesException ex = new NeedPrivilegesException("Not enough privileges");
                ex.AddRequiredPrivilege(item.Path, privilege);
                throw ex;
            }
            catch (IOException ex)
            {
                int hr = Marshal.GetHRForException(ex);
                if (hr == ERROR_DISK_FULL)
                {
                    throw new InsufficientStorageException();
                }

                throw new DavException(ex.Message, DavStatus.CONFLICT);
            }
        }

        /// <summary>
        /// Performs file system operation with translating exceptions to those expected by WebDAV engine, except
        /// <see cref="UnauthorizedAccessException"/> which must be caught and translated manually.
        /// </summary>        
        /// <param name="action">The action to be performed.</param>
        public async Task FileOperationAsync(Func<Task> actionAsync)
        {
            try
            {
                //!<# if (Params.ACL && Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm)) { #>
                using (impersonate())
                {
                    await actionAsync();
                }
                //!<# }else{ #>
                await actionAsync();
                //!<# } #>
            }
            catch (IOException ex)
            {
                int hr = Marshal.GetHRForException(ex);
                if (hr == ERROR_DISK_FULL)
                {
                    throw new InsufficientStorageException();
                }

                throw new DavException(ex.Message, DavStatus.CONFLICT);
            }
        }
        //!<# } #>

        /// <summary>
        /// Performs file system operation with translating exceptions to those expected by WebDAV engine, except
        /// <see cref="UnauthorizedAccessException"/> which must be caught and translated manually.
        /// </summary>        
        /// <param name="action">The action to be performed.</param>
        public void FileOperation(Action action)
        {
            try
            {
                //!<# if (Params.ACL && Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm)) { #>
                using (impersonate())
                {
                    action();
                }
                //!<# }else{ #>
                action();
                //!<# } #>
            }
            catch (IOException ex)
            {
                int hr = Marshal.GetHRForException(ex);
                if (hr == ERROR_DISK_FULL)
                {
                    throw new InsufficientStorageException();
                }

                throw new DavException(ex.Message, DavStatus.CONFLICT);
            }
        }

        /// <summary>
        /// Performs file system operation with translating exceptions to those expected by WebDAV engine.
        /// </summary>
        /// <param name="item">Item on which operation is performed.</param>
        /// <param name="func">The action to be performed.</param>
        /// <param name="privilege">Privilege which is needed to perform the operation.
        /// If <see cref="UnauthorizedAccessException"/> is thrown  this method will convert it to
        /// <see cref="NeedPrivilegesException"/> exception and specify this privilege in it.</param>
        /// <typeparam name="T">Type of operation's result.</typeparam>
        /// <returns>Result returned by <paramref name="func"/>.</returns>
        public T FileOperation<T>(IHierarchyItem item, Func<T> func, Privilege privilege)
        {
            try
            {
                //!<# if (Params.ACL && Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm)) { #>
                using (impersonate())
                {
                    return func();
                }
                //!<# }else{ #>
                return func();
                //!<# } #>
            }
            catch (UnauthorizedAccessException)
            {
                NeedPrivilegesException ex = new NeedPrivilegesException("Not enough privileges");
                ex.AddRequiredPrivilege(item.Path, privilege);
                throw ex;
            }
            catch (IOException ex)
            {
                int hr = Marshal.GetHRForException(ex);
                if (hr == ERROR_DISK_FULL)
                {
                    throw new InsufficientStorageException();
                }

                throw new DavException(ex.Message, DavStatus.CONFLICT);
            }
        }
        //!<# } #>
        //!<# if (Params.ACL && Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm)) { #>

        /// <summary>
        /// Dispose everything we have.
        /// </summary>
        public void Dispose()
        {
            if (principalContext != null)
            {
                principalContext.Dispose();
            }
        }

        /// <summary>
        /// Impersonates current user.
        /// </summary>
        /// <returns>Impersonation context, which must be disposed to 'unimpersonate'</returns>
        private WindowsImpersonationContext impersonate()
        {
            return LogonUtil.DuplicateToken(WindowsIdentity).Impersonate();
        }
        //!<# } #>
    }
}
