//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (Params.Storage != StorageType.Ntfs)  return string.Empty; #>
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
//!<# if (Params.ACL) { #>
using System.Security.AccessControl;
using System.Security.Principal;
//!<# } #>
//!<# if (Params.IsOwinAuthMode) { #>
using Microsoft.AspNet.Identity;
//!<# } #>
using System.Threading.Tasks;

using ITHit.WebDAV.Server;
using ITHit.WebDAV.Server.Acl;
using IPrincipal = ITHit.WebDAV.Server.Acl.IPrincipal;
using ITHit.WebDAV.Server.Extensibility;
//!<# if (Params.Class2) { #>
using ITHit.WebDAV.Server.Class2;
//!<# } #>
//!<# if (Params.MicrosoftExtensions) { #>
using ITHit.WebDAV.Server.MicrosoftExtensions;
//!<# } #>
//!<# if (Params.SynchronizationCollection) { #>
using ITHit.WebDAV.Server.Synchronization;
//!<# } #>

//!<# if (Params.ACL) { #>

using ITHit.WebDav.Server.SampleModel.__NtfsStorage__.Acl;
//!<# } #>
//!<# if (Params.ACL && !Params.TargetFramework45OrLater) { #>
using IPrincipal = ITHit.WebDAV.Server.Acl.IPrincipal;
//!<# } #>
using ITHit.WebDav.Server.SampleModel.__NtfsStorage__.ExtendedAttributes;

namespace ITHit.WebDav.Server.SampleModel.__NtfsStorage__
{
    /// <summary>
    /// Base class for WebDAV items (folders, files, etc).
    /// </summary>
    public abstract class DavHierarchyItem :
        /*!<# if (Params.CalDav || Params.CardDav) { #>!*/ Discovery/*!<# } #>!*/
        /*!<# if (Params.CalDav || Params.CardDav) { #>!*/ ,/*!<# } #>!*/ IHierarchyItem
        /*!<# if (Params.ACL && !(Params.CalDav || Params.CardDav)) { #>!*/, IAclHierarchyItem/*!<# } #>!*/
        /*!<# if (Params.ACL &&  (Params.CalDav || Params.CardDav)) { #>!*/, ICurrentUserPrincipal/*!<# } #>!*/
        /*!<# if (Params.Class2) { #>!*/, ILock/*!<# } #>!*/
        /*!<# if (Params.MicrosoftExtensions) { #>!*/, IMsItem/*!<# } #>!*/
        /*!<# if (Params.SynchronizationCollection) { #>!*/, IBind, IChangedItem/*!<# } #>!*/
    {
        //!<# if (Params.DASL) { #>
        /// <summary>
        /// Property name to return text anound search phrase.
        /// </summary>
        internal const string snippetProperty = "snippet";
        //!<# } #>

        /// <summary>
        /// Name of properties attribute.
        /// </summary>
        internal const string propertiesAttributeName = "Properties";

        /// <summary>
        /// Name of locks attribute.
        /// </summary>
        internal const string locksAttributeName = "Locks";

        //$<IHierarchyItem.Name
        /// <summary>
        /// Gets name of the item.
        /// </summary>
        public string Name { get { return fileSystemInfo.Name; } }
        //$>

        //$<IHierarchyItem.Created
        /// <summary>
        /// Gets date when the item was created in UTC.
        /// </summary>
        public DateTime Created { get { return fileSystemInfo.CreationTimeUtc; } }
        //$>

        //$<IHierarchyItem.Modified
        /// <summary>
        /// Gets date when the item was last modified in UTC.
        /// </summary>
        public DateTime Modified { get { return fileSystemInfo.LastWriteTimeUtc; } }
        //$>

        //$<IHierarchyItem.Path
        /// <summary>
        /// Gets path of the item where each part between slashes is encoded.
        /// </summary>
        public string Path { get; private set; }
        //$>

        /// <summary>
        /// Gets full path for this file/folder in the file system.
        /// </summary>
        public string FullPath { get { return fileSystemInfo.FullName.TrimEnd(System.IO.Path.DirectorySeparatorChar); } }

        //!<# if (Params.SynchronizationCollection) { #>
        public Change ChangeType
        {
            get => fileSystemInfo.Attributes.HasFlag(FileAttributes.Hidden) &&
                (this is DavFolder || (this as DavFile).ContentLength == 0) ? Change.Deleted : Change.Changed;
        }

        /// <summary>
        /// Unique identifier of the resource.
        /// </summary>
        //!<# if (string.IsNullOrEmpty(Params.DavUrlPrefix)) { #>
        public string Id { get => $"{context.Request.UrlPrefix}/ID/{fileSystemInfo.GetId()}"; }
        //!<# } else { #>
        public string Id { get => $"{context.Request.UrlPrefix}/<#=Params.DavUrlPrefix.Trim('/')#>/ID/{fileSystemInfo.GetId()}"; }
        //!<# } #>

        /// <summary>
        /// Unique identifier of the resource parent. 
        /// </summary>
        //!<# if (string.IsNullOrEmpty(Params.DavUrlPrefix)) { #>
        public string ParentId { get => $"{context.Request.UrlPrefix}/ID/{Directory.GetParent(fileSystemInfo.FullName).GetId()}"; }
        //!<# } else { #>
        public string ParentId { get => $"{context.Request.UrlPrefix}/<#=Params.DavUrlPrefix.Trim('/')#>/ID/{Directory.GetParent(fileSystemInfo.FullName).GetId()}"; }
        //!<# } #>
        //!<# } #>

        /// <summary>
        /// Corresponding file or folder in the file system.
        /// </summary>
        internal FileSystemInfo fileSystemInfo;

        /// <summary>
        /// WebDAV Context.
        /// </summary>
        protected DavContext context;

        /// <summary>
        /// Initializes a new instance of this class.
        /// </summary>
        /// <param name="fileSystemInfo">Corresponding file or folder in the file system.</param>
        /// <param name="context">WebDAV Context.</param>
        /// <param name="path">Encoded path relative to WebDAV root folder.</param>
        protected DavHierarchyItem(FileSystemInfo fileSystemInfo, DavContext context, string path)
        /*!<# if (Params.CalDav || Params.CardDav) { #>!*/: base(context)/*!<# } #>!*/
        {
            this.fileSystemInfo = fileSystemInfo;
            this.context = context;
            this.Path = path;
        }

        /// <summary>
        /// Creates a copy of this item with a new name in the destination folder.
        /// </summary>
        /// <param name="destFolder">Destination folder.</param>
        /// <param name="destName">Name of the destination item.</param>
        /// <param name="deep">Indicates whether to copy entire subtree.</param>
        /// <param name="multistatus">If some items fail to copy but operation in whole shall be continued, add
        /// information about the error into <paramref name="multistatus"/> using 
        /// <see cref="MultistatusException.AddInnerException(string,ITHit.WebDAV.Server.DavException)"/>.
        /// </param>
        public abstract Task CopyToAsync(IItemCollection destFolder, string destName, bool deep, MultistatusException multistatus);

        /// <summary>
        /// Creates a copy of this item with a new name in the destination folder.
        /// </summary>
        /// <param name="destFolder">Destination folder.</param>
        /// <param name="destName">Name of the destination item.</param>
        /// <param name="deep">Indicates whether to copy entire subtree.</param>
        /// <param name="multistatus">If some items fail to copy but operation in whole shall be continued, add
        /// information about the error into <paramref name="multistatus"/> using 
        /// <see cref="MultistatusException.AddInnerException(string,ITHit.WebDAV.Server.DavException)"/>.
        /// </param>
        /// <param name="recursionDepth">Recursion depth.</param>
        public abstract Task CopyToInternalAsync(IItemCollection destFolder, string destName, bool deep, MultistatusException multistatus, int recursionDepth);

        /// <summary>
        /// Moves this item to the destination folder under a new name.
        /// </summary>
        /// <param name="destFolder">Destination folder.</param>
        /// <param name="destName">Name of the destination item.</param>
        /// <param name="multistatus">If some items fail to copy but operation in whole shall be continued, add
        /// information about the error into <paramref name="multistatus"/> using 
        /// <see cref="MultistatusException.AddInnerException(string,ITHit.WebDAV.Server.DavException)"/>.
        /// </param>
        public abstract Task MoveToAsync(IItemCollection destFolder, string destName, MultistatusException multistatus);

        /// <summary>
        /// Moves this item to the destination folder under a new name.
        /// </summary>
        /// <param name="destFolder">Destination folder.</param>
        /// <param name="destName">Name of the destination item.</param>
        /// <param name="multistatus">If some items fail to copy but operation in whole shall be continued, add
        /// information about the error into <paramref name="multistatus"/> using 
        /// <see cref="MultistatusException.AddInnerException(string,ITHit.WebDAV.Server.DavException)"/>.
        /// </param>
        /// <param name="recursionDepth">Recursion depth.</param>
        public abstract Task MoveToInternalAsync(IItemCollection destFolder, string destName, MultistatusException multistatus, int recursionDepth);

        /// <summary>
        /// Deletes this item.
        /// </summary>
        /// <param name="multistatus">If some items fail to delete but operation in whole shall be continued, add
        /// information about the error into <paramref name="multistatus"/> using
        /// <see cref="MultistatusException.AddInnerException(string,ITHit.WebDAV.Server.DavException)"/>.
        /// </param>
        public abstract Task DeleteAsync(MultistatusException multistatus);

        /// <summary>
        /// Deletes this item.
        /// </summary>
        /// <param name="multistatus">If some items fail to delete but operation in whole shall be continued, add
        /// information about the error into <paramref name="multistatus"/> using
        /// <see cref="MultistatusException.AddInnerException(string,ITHit.WebDAV.Server.DavException)"/>.
        /// </param>
        /// <param name="recursionDepth">Recursion depth.</param>
        public abstract Task DeleteInternalAsync(MultistatusException multistatus, int recursionDepth);

        //$<IHierarchyItem.GetPropertiesAsync
        /// <summary>
        /// Retrieves user defined property values.
        /// </summary>
        /// <param name="names">Names of dead properties which values to retrieve.</param>
        /// <param name="allprop">Whether all properties shall be retrieved.</param>
        /// <returns>Property values.</returns>
        public async Task<IEnumerable<PropertyValue>> GetPropertiesAsync(IList<PropertyName> props, bool allprop)
        {
            List<PropertyValue> propertyValues = await GetPropertyValuesAsync();
            //!<# if (Params.DASL) { #>

            PropertyName snippet = props.FirstOrDefault(s => s.Name == snippetProperty);
            if (snippet.Name == snippetProperty && this is DavFile)
            {
                propertyValues.Add(new PropertyValue(snippet, ((DavFile)this).Snippet));
            }
            //!<# } #>

            if (!allprop)
            {
                propertyValues = propertyValues.Where(p => props.Contains(p.QualifiedName)).ToList();
            }

            return propertyValues;
        }
        //$>

        //$<IHierarchyItem.GetPropertyNamesAsync
        /// <summary>
        /// Retrieves names of all user defined properties.
        /// </summary>
        /// <returns>Property names.</returns>
        public async Task<IEnumerable<PropertyName>> GetPropertyNamesAsync()
        {
            IList<PropertyValue> propertyValues = await GetPropertyValuesAsync();
            return propertyValues.Select(p => p.QualifiedName);
        }
        //$>

        //$<IHierarchyItem.GetPropertyValuesAsync
        /// <summary>
        /// Retrieves list of user defined propeties for this item.
        /// </summary>
        /// <returns>List of user defined properties.</returns>
        private async Task<List<PropertyValue>> GetPropertyValuesAsync()
        {
            List<PropertyValue> properties = new List<PropertyValue>();
            if (await fileSystemInfo.HasExtendedAttributeAsync(propertiesAttributeName))
            {
                properties = await fileSystemInfo.GetExtendedAttributeAsync<List<PropertyValue>>(propertiesAttributeName);
            }

            return properties;
        }
        //$>

        //$<IHierarchyItem.UpdatePropertiesAsync
        /// <summary>
        /// Saves property values to extended attribute.
        /// </summary>
        /// <param name="setProps">Properties to be set.</param>
        /// <param name="delProps">Properties to be deleted.</param>
        /// <param name="multistatus">Information about properties that failed to create, update or delate.</param>
        public async Task UpdatePropertiesAsync(IList<PropertyValue> setProps, IList<PropertyName> delProps, MultistatusException multistatus)
        {
            //!<# if (Params.Class2) { #>
            await RequireHasTokenAsync();

            //!<# } #>
            List<PropertyValue> propertyValues = await GetPropertyValuesAsync();
            foreach (PropertyValue propToSet in setProps)
            {
                // Microsoft Mini-redirector may update file creation date, modification date and access time passing properties:
                // <Win32CreationTime xmlns="urn:schemas-microsoft-com:">Thu, 28 Mar 2013 20:15:34 GMT</Win32CreationTime>
                // <Win32LastModifiedTime xmlns="urn:schemas-microsoft-com:">Thu, 28 Mar 2013 20:36:24 GMT</Win32LastModifiedTime>
                // <Win32LastAccessTime xmlns="urn:schemas-microsoft-com:">Thu, 28 Mar 2013 20:36:24 GMT</Win32LastAccessTime>
                // In this case update creation and modified date in your storage or do not save this properties at all, otherwise 
                // Windows Explorer will display creation and modification date from this props and it will differ from the values 
                // in the Created and Modified fields in your storage 
                if (propToSet.QualifiedName.Namespace == "urn:schemas-microsoft-com:")
                {
                    switch (propToSet.QualifiedName.Name)
                    {
                        case "Win32CreationTime":
                            fileSystemInfo.CreationTimeUtc = DateTime.Parse(propToSet.Value,
                                new System.Globalization.CultureInfo("en-US")).ToUniversalTime();
                            break;
                        case "Win32LastModifiedTime":
                            fileSystemInfo.LastWriteTimeUtc = DateTime.Parse(propToSet.Value,
                                new System.Globalization.CultureInfo("en-US")).ToUniversalTime();
                            break;
                        default:
                            context.Logger.LogDebug(string.Format("Unspecified case: DavHierarchyItem.UpdateProperties {0} from {1} namesapce",
                                propToSet.QualifiedName.Name, propToSet.QualifiedName.Namespace));
                            break;
                    }
                }
                else
                {
                    PropertyValue existingProp = propertyValues.FirstOrDefault(p => p.QualifiedName == propToSet.QualifiedName);

                    if (existingProp != null)
                    {
                        existingProp.Value = propToSet.Value;
                    }
                    else
                    {
                        propertyValues.Add(propToSet);
                    }
                }
            }

            propertyValues.RemoveAll(prop => delProps.Contains(prop.QualifiedName));

            await fileSystemInfo.SetExtendedAttributeAsync(propertiesAttributeName, propertyValues);

            //!<# if(Params.WebSockets) { #>
            await context.socketService.NotifyUpdatedAsync(Path, GetWebSocketID());
            //!<# } #>
        }
        //$>
        //!<# if (Params.MicrosoftExtensions) { #>

        //$<IMsItem.GetFileAttributesAsync
        /// <summary>
        /// Returns Windows file attributes (readonly, hidden etc.) for this file/folder.
        /// </summary>
        /// <returns>Windows file attributes.</returns>
        public async Task<FileAttributes> GetFileAttributesAsync()
        {
            if (Name.StartsWith("."))
            {
                return fileSystemInfo.Attributes | FileAttributes.Hidden;
            }
            return fileSystemInfo.Attributes;
        }
        //$>

        //$<IMsItem.SetFileAttributesAsync
        /// <summary>
        /// Sets Windows file attributes (readonly, hidden etc.) on this item.
        /// </summary>
        /// <param name="value">File attributes.</param>
        public async Task SetFileAttributesAsync(FileAttributes value)
        {
            File.SetAttributes(fileSystemInfo.FullName, value);
        }
        //$>
        //!<# } #>
        //!<# if (Params.Class2) { #>

        //$<ILock.GetActiveLocksAsync
        /// <summary>
        /// Retrieves non expired locks for this item.
        /// </summary>
        /// <returns>Enumerable with information about locks.</returns>
        public async Task<IEnumerable<LockInfo>> GetActiveLocksAsync()
        {
            List<DateLockInfo> locks = await GetLocksAsync();
            if (locks == null)
            {
                return new List<LockInfo>();
            }

            IEnumerable<LockInfo> lockInfoList = locks.Select(l => new LockInfo
            {
                IsDeep = l.IsDeep,
                Level = l.Level,
                Owner = l.ClientOwner,
                LockRoot = l.LockRoot,
                TimeOut = l.Expiration == DateTime.MaxValue ?
                            TimeSpan.MaxValue :
                            l.Expiration - DateTime.UtcNow,
                Token = l.LockToken
            }).ToList();

            return lockInfoList;
        }
        //$>

        //$<ILock.LockAsync
        /// <summary>
        /// Locks this item.
        /// </summary>
        /// <param name="level">Whether lock is share or exclusive.</param>
        /// <param name="isDeep">Whether lock is deep.</param>
        /// <param name="requestedTimeOut">Lock timeout which was requested by client.
        /// Server may ignore this parameter and set any timeout.</param>
        /// <param name="owner">Owner of the lock as specified by client.</param> 
        /// <returns>
        /// Instance of <see cref="LockResult"/> with information about the lock.
        /// </returns>
        public async Task<LockResult> LockAsync(LockLevel level, bool isDeep, TimeSpan? requestedTimeOut, string owner)
        {
            await RequireUnlockedAsync(level == LockLevel.Shared);
            string token = Guid.NewGuid().ToString();

            // If timeout is absent or infinit timeout requested,
            // grant 5 minute lock.
            TimeSpan timeOut = TimeSpan.FromMinutes(5);

            if (requestedTimeOut.HasValue && requestedTimeOut < TimeSpan.MaxValue)
            {
                timeOut = requestedTimeOut.Value;
            }

            DateLockInfo lockInfo = new DateLockInfo
            {
                Expiration = DateTime.UtcNow + timeOut,
                IsDeep = false,
                Level = level,
                LockRoot = Path,
                LockToken = token,
                ClientOwner = owner,
                TimeOut = timeOut
            };

            await SaveLockAsync(lockInfo);

            //!<# if(Params.WebSockets) { #>
            await context.socketService.NotifyLockedAsync(Path, GetWebSocketID());
            //!<# } #>

            return new LockResult(lockInfo.LockToken, lockInfo.TimeOut);
        }
        //$>

        //$<ILock.RefreshLockAsync
        /// <summary>
        /// Updates lock timeout information on this item.
        /// </summary>
        /// <param name="token">Lock token.</param>
        /// <param name="requestedTimeOut">Lock timeout which was requested by client.
        /// Server may ignore this parameter and set any timeout.</param>
        /// <returns>
        /// Instance of <see cref="LockResult"/> with information about the lock.
        /// </returns>
        public async Task<RefreshLockResult> RefreshLockAsync(string token, TimeSpan? requestedTimeOut)
        {
            if (string.IsNullOrEmpty(token))
            {
                throw new DavException("Lock can not be found.", DavStatus.BAD_REQUEST);
            }

            List<DateLockInfo> locks = await GetLocksAsync(getAllWithExpired: true);
            DateLockInfo lockInfo = locks.SingleOrDefault(x => x.LockToken == token);

            if (lockInfo == null || lockInfo.Expiration <= DateTime.UtcNow)
            {
                throw new DavException("Lock can not be found.", DavStatus.CONFLICT);
            }
            else
            {
                lockInfo.TimeOut = TimeSpan.FromMinutes(5);

                if (requestedTimeOut.HasValue && requestedTimeOut < TimeSpan.MaxValue)
                {
                    lockInfo.TimeOut = requestedTimeOut.Value;
                }


                lockInfo.Expiration = DateTime.UtcNow + lockInfo.TimeOut;

                await SaveLockAsync(lockInfo);
            }

            //!<# if(Params.WebSockets) { #>
            await context.socketService.NotifyLockedAsync(Path, GetWebSocketID());
            //!<# } #>

            return new RefreshLockResult(lockInfo.Level, lockInfo.IsDeep, lockInfo.TimeOut, lockInfo.ClientOwner);
        }
        //$>

        //$<ILock.UnlockAsync
        /// <summary>
        /// Removes lock with the specified token from this item.
        /// </summary>
        /// <param name="lockToken">Lock with this token should be removed from the item.</param>
        public async Task UnlockAsync(string lockToken)
        {
            if (string.IsNullOrEmpty(lockToken))
            {
                throw new DavException("Lock can not be found.", DavStatus.BAD_REQUEST);
            }

            List<DateLockInfo> locks = await GetLocksAsync(getAllWithExpired: true);
            DateLockInfo lockInfo = locks.SingleOrDefault(x => x.LockToken == lockToken);

            await RemoveExpiredLocksAsync(lockToken);

            if (lockInfo == null || lockInfo.Expiration <= DateTime.UtcNow)
            {
                throw new DavException("The lock could not be found.", DavStatus.CONFLICT);
            }

            //!<# if(Params.WebSockets) { #>
            await context.socketService.NotifyUnLockedAsync(Path, GetWebSocketID());
            //!<# } #>
        }
        //$>

        /// <summary>
        /// Check that if the item is locked then client has submitted correct lock token.
        /// </summary>
        public async Task RequireHasTokenAsync(bool skipShared = false)
        {
            List<DateLockInfo> locks = await GetLocksAsync();
            if (locks != null && locks.Any())
            {
                IList<string> clientLockTokens = context.Request.GetClientLockTokens();
                if (locks.All(l => !clientLockTokens.Contains(l.LockToken)))
                {
                    throw new LockedException();
                }
            }
        }

        /// <summary>
        /// Ensure that there are no active locks on the item.
        /// </summary>
        /// <param name="skipShared">Whether shared locks shall be checked.</param>
        public async Task RequireUnlockedAsync(bool skipShared)
        {
            List<DateLockInfo> locks = await GetLocksAsync();

            if (locks != null && locks.Any())
            {
                if ((skipShared && locks.Any(l => l.Level == LockLevel.Exclusive))
                    || (!skipShared && locks.Any()))
                {
                    throw new LockedException();
                }
            }
        }

        /// <summary>
        /// Retrieves non-expired locks acquired on this item.
        /// </summary>
        /// <param name="getAllWithExpired">Indicate needed return expired locks.</param>
        /// <returns>List of locks with their expiration dates.</returns>
        private async Task<List<DateLockInfo>> GetLocksAsync(bool getAllWithExpired = false)
        {

            List<DateLockInfo> locks = new List<DateLockInfo>();

            if (await fileSystemInfo.HasExtendedAttributeAsync(locksAttributeName))
            {
                locks = await fileSystemInfo.GetExtendedAttributeAsync<List<DateLockInfo>>(locksAttributeName);

                if (locks != null)
                {
                    locks.ForEach(l => l.LockRoot = Path);
                }
            }

            if (getAllWithExpired)
            {
                return locks;
            }
            else
            {
                return locks.Where(x => x.Expiration > DateTime.UtcNow).ToList();
            }
        }

        /// <summary>
        /// Saves lock acquired on this file/folder.
        /// </summary>
        private async Task SaveLockAsync(DateLockInfo lockInfo)
        {
            List<DateLockInfo> locks = await GetLocksAsync(getAllWithExpired: true);

            //remove all expired locks
            //await RemoveExpiretLocksAsync();
            //you can call this method but it will be second file operation
            locks.RemoveAll(x => x.Expiration <= DateTime.UtcNow);

            if (locks.Any(x => x.LockToken == lockInfo.LockToken))
            {
                //update value
                DateLockInfo existingLock = locks.Single(x => x.LockToken == lockInfo.LockToken);
                existingLock.TimeOut = lockInfo.TimeOut;

                existingLock.Level = lockInfo.Level;
                existingLock.IsDeep = lockInfo.IsDeep;
                existingLock.LockRoot = lockInfo.LockRoot;
                existingLock.Expiration = lockInfo.Expiration;
                existingLock.ClientOwner = lockInfo.ClientOwner;
            }
            else
            {
                //add new item
                locks.Add(lockInfo);
            }

            await fileSystemInfo.SetExtendedAttributeAsync(locksAttributeName, locks);
        }

        private async Task RemoveExpiredLocksAsync(string unlockedToken)
        {
            List<DateLockInfo> locks = await GetLocksAsync(getAllWithExpired: true);

            //remove expired and current lock
            locks.RemoveAll(x => x.Expiration <= DateTime.UtcNow);

            //remove from token
            if (!string.IsNullOrEmpty(unlockedToken))
            {
                locks.RemoveAll(x => x.LockToken == unlockedToken);
            }

            await fileSystemInfo.SetExtendedAttributeAsync(locksAttributeName, locks);
        }
        //!<# } #>
        //!<# if (Params.ACL) { #>

        //$<ICurrentUserPrincipal.GetCurrentUserPrincipalAsync
        /// <summary>
        /// Returns instance of <see cref="IPrincipal"/> which represents current user.
        /// </summary>
        /// <returns>Current user.</returns>
        /// <remarks>
        /// This method is usually called by the Engine when CalDAV/CardDAV client 
        /// is trying to discover current user URL.
        /// </remarks>
        public async Task<IPrincipal> GetCurrentUserPrincipalAsync()
        {
            // Typically there is no need to load all user properties here, only current 
            // user ID (or name) is required to form the user URL: [DAVLocation]/acl/users/[UserID]

            //!<# if ( Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm) ) { #>
            return AclFactory.GetPrincipalFromSid(context.WindowsIdentity.User.Value, context);
            //!<# } else if (Params.IsOwinAuthMode) { #>
            return new User(context, System.Web.HttpContext.Current.User.Identity.GetUserId());
            //!<# } else { #>
            return new User(context, context.UserName);
            //!<# } #>
        }
        //$>
        //!<# } #>
        //!<# if (Params.ACL && !(Params.CalDav || Params.CardDav)) { #>

        //$<IAclHierarchyItem.SetOwnerAsync
        /// <summary>
        /// Sets <paramref name="principal"/> as new owner for this item.
        /// </summary>
        /// <param name="principal">New owner.</param>
        public async Task SetOwnerAsync(IPrincipal principal)
        {
            //!<# if (Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm)) { #>
            context.FileOperation(
                this,
                () =>
                {
                    var accessControl = File.GetAccessControl(fileSystemInfo.FullName, AccessControlSections.Owner);
                    accessControl.SetOwner(((PrincipalBase)principal).Sid);
                    File.SetAccessControl(fileSystemInfo.FullName, accessControl);
                },
                Privilege.WriteProperties);
            //!<# } else { #>
            throw new DavException("Not implemented.", DavStatus.NOT_IMPLEMENTED);
            //!<# } #>
        }
        //$>

        //$<IAclHierarchyItem.GetOwnerAsync
        /// <summary>
        /// Retrieves user or group who is owner for this item.
        /// </summary>
        /// <returns>Owner for this item.</returns>
        public async Task<IPrincipal> GetOwnerAsync()
        {
            //!<# if ( Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm) ) { #>
            return context.FileOperation(
                this,
                () =>
                {
                    FileSecurity acl = File.GetAccessControl(fileSystemInfo.FullName);
                    return AclFactory.GetPrincipalFromSid(acl.GetOwner(typeof(SecurityIdentifier)).Value, context);
                },
                Privilege.Read);
            //!<# } else { #>
            throw new DavException("Not implemented.", DavStatus.NOT_IMPLEMENTED);
            //!<# } #>
        }
        //$>

        //$<IAclHierarchyItem.SetGroupAsync
        /// <summary>
        /// File groups are not supported.
        /// </summary>
        public async Task SetGroupAsync(IPrincipal principal)
        {
            throw new DavException("Group cannot be set", DavStatus.FORBIDDEN);
        }
        //$>

        //$<IAclHierarchyItem.GetGroupAsync
        /// <summary>
        /// File groups are not supported.
        /// </summary>
        public async Task<IPrincipal> GetGroupAsync()
        {
            return null;
        }
        //$>

        //$<IAclHierarchyItem.GetSupportedPrivilegeSetAsync
        /// <summary>
        /// Retrieves list of all privileges (permissions) which can be set for the item.
        /// </summary>
        /// <returns>Enumerable with supported permissions.</returns>
        public async Task<IEnumerable<SupportedPrivilege>> GetSupportedPrivilegeSetAsync()
        {
            RequireReadPrivilege();
            //!<# if ( Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm) ) { #>
            return NtfsPermissions.GetPermissionsTree();
            //!<# } else { #>
            throw new DavException("Not implemented.", DavStatus.NOT_IMPLEMENTED);
            //!<# } #>
        }
        //$>

        //$<IAclHierarchyItem.GetCurrentUserPrivilegeSetAsync
        /// <summary>
        /// Retrieves the exact set of permissions current user has for this file or folder.
        /// </summary>
        public async Task<IEnumerable<Privilege>> GetCurrentUserPrivilegeSetAsync()
        {
            //!<# if ( Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm) ) { #>
            var acl = File.GetAccessControl(
                fileSystemInfo.FullName,
                AccessControlSections.Access | AccessControlSections.Owner | AccessControlSections.Group);

            return context.FileOperation(
                this,
                () => NtfsPermissions.ExpandPrivileges(
                    NtfsPermissions.MapFileSystemRights(
                        EffectivePermissions.GetEffectivePermissions(context.WindowsIdentity, acl))),
                Privilege.ReadAcl);
            //!<# } else { #>
            throw new DavException("Not implemented.", DavStatus.NOT_IMPLEMENTED);
            //!<# } #>
        }
        //$>

        //$<IAclHierarchyItem.GetAclAsync
        /// <summary>
        /// Retrieves access control list for this file or folder.
        /// </summary>
        /// <param name="propertyNames">Properties which will be retrieved from users/groups specified in
        /// access control list returned.</param>
        /// <returns>Enumerable with access control entries.</returns>
        public async Task<IEnumerable<ReadAce>> GetAclAsync(IList<PropertyName> propertyNames)
        {
            //!<# if ( Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm) ) { #>
            AuthorizationRuleCollection rules = null;
            AuthorizationRuleCollection protectedRules = null;
            context.FileOperation(
                this,
                () =>
                {
                    FileSecurity acl = File.GetAccessControl(fileSystemInfo.FullName);
                    rules = acl.GetAccessRules(true, false, typeof(SecurityIdentifier));
                    protectedRules = acl.GetAccessRules(false, true, typeof(SecurityIdentifier));
                },
                Privilege.ReadAcl);

            IList<ReadAce> aceList = new List<ReadAce>();

            foreach (FileSystemAccessRule rule in rules)
            {
                var ace = new ReadAce();
                ace.Principal = AclFactory.GetPrincipalFromSid(rule.IdentityReference.Value, context);
                if (ace.Principal == null)
                {
                    continue;
                }

                if (rule.AccessControlType == AccessControlType.Allow)
                {
                    ace.GrantPrivileges.AddRange(NtfsPermissions.MapFileSystemRights(rule.FileSystemRights));
                }
                else
                {
                    ace.DenyPrivileges.AddRange(NtfsPermissions.MapFileSystemRights(rule.FileSystemRights));
                }

                ace.IsProtected = false;
                aceList.Add(ace);
            }

            foreach (FileSystemAccessRule rule in protectedRules)
            {
                var ace = new ReadAce();
                ace.Principal = AclFactory.GetPrincipalFromSid(rule.IdentityReference.Value, context);
                if (ace.Principal == null)
                {
                    context.Logger.LogDebug("Could not resolve principal with sid " + rule.IdentityReference.Value);
                    continue;
                }

                if (rule.AccessControlType == AccessControlType.Allow)
                {
                    ace.GrantPrivileges.AddRange(NtfsPermissions.MapFileSystemRights(rule.FileSystemRights));
                }
                else
                {
                    ace.DenyPrivileges.AddRange(NtfsPermissions.MapFileSystemRights(rule.FileSystemRights));
                }

                ace.IsProtected = true;
                ace.InheritedFrom = await GetParentAsync() ?? this;
                aceList.Add(ace);
            }

            return aceList;
            //!<# } else { #>
            throw new DavException("Not implemented.", DavStatus.NOT_IMPLEMENTED);
            //!<# } #>
        }
        //$>

        //$<IAclHierarchyItem.SetAclAsync
        /// <summary>
        /// Sets access control list for this file/folder.
        /// </summary>
        /// <param name="aces">List of access control entries.</param>
        public async Task SetAclAsync(IList<WriteAce> aces)
        {
            //!<# if ( Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm) ) { #>
            FileSecurity acl = File.GetAccessControl(fileSystemInfo.FullName);
            var rules = acl.GetAccessRules(true, false, typeof(SecurityIdentifier));
            foreach (FileSystemAccessRule rule in rules)
            {
                //remove only those records we can resolve (in our domain/computer)
                if (AclFactory.GetPrincipalFromSid(rule.IdentityReference.Value, context) != null)
                {
                    acl.RemoveAccessRule(rule);
                }
            }

            foreach (WriteAce ace in aces)
            {
                FileSystemRights grantRights = NtfsPermissions.MapPrivileges(ace.GrantPrivileges, context.Logger);
                var sid = ((PrincipalBase)ace.Principal).Sid;
                if (grantRights != 0)
                {
                    var grantRule = new FileSystemAccessRule(sid, grantRights, AccessControlType.Allow);
                    acl.AddAccessRule(grantRule);
                }

                FileSystemRights denyRights = NtfsPermissions.MapPrivileges(ace.DenyPrivileges, context.Logger);
                if (denyRights != 0)
                {
                    var denyRule = new FileSystemAccessRule(sid, denyRights, AccessControlType.Deny);
                    acl.AddAccessRule(denyRule);
                }
            }

            context.FileOperation(
                this,
                () => File.SetAccessControl(fileSystemInfo.FullName, acl),
                Privilege.WriteProperties);
            //!<# } else { #>
            throw new DavException("Not implemented.", DavStatus.NOT_IMPLEMENTED);
            //!<# } #>
        }
        //$>

        //$<IAclHierarchyItem.GetAclRestrictionsAsync
        /// <summary>
        /// Retrieves list of restrictions for access control entries.
        /// We don't support WebDAV inverted permissions.
        /// </summary>
        /// <returns>ACL restrictions.</returns>
        public async Task<AclRestriction> GetAclRestrictionsAsync()
        {
            return new AclRestriction { NoInvert = true };
        }
        //$>

        //$<IAclHierarchyItem.GetInheritedAclSetAsync
        /// <summary>
        /// Gets all folders, from which this file/folder has inherited access control entries.
        /// </summary>
        /// <returns>Enumerable with files/folders from which this file/folder has inherited
        /// access control entries.</returns>
        public async Task<IEnumerable<IHierarchyItem>> GetInheritedAclSetAsync()
        {
            var current = this;
            var parent = await current.GetParentAsync();
            List<IHierarchyItem> parents = new List<IHierarchyItem>();
            while (parent != null)
            {
                var accessControl = File.GetAccessControl(parent.fileSystemInfo.FullName);
                var acl = accessControl.GetAccessRules(true, false, typeof(SecurityIdentifier));
                if (acl.Count > 0)
                {
                    parents.Add(parent);
                }

                current = parent;
                parent = await current.GetParentAsync();
            }
            return parents;
        }
        //$>

        //$<IAclHierarchyItem.GetPrincipalCollectionSetAsync
        /// <summary>
        /// Gets collections which contain principals.
        /// </summary>
        //!<# if ( Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm) ) { #>
        /// <remarks>
        /// We return one folder which contains users and the other folder which contains groups.
        /// We put them into different folders because webdav doesn't have strong distinction between users
        /// and groups, but Windows does, so we put users and groups into separate virtual folders so it is obvious
        /// what user wants to do (create user or group for example).
        /// </remarks>
        //!<# } #>
        /// <returns>Folders which contain users/groups.</returns>
        public async Task<IEnumerable<IPrincipalFolder>> GetPrincipalCollectionSetAsync()
        {
            return new IPrincipalFolder[]
                       {
                           //!<# if ( Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm) ) { #>
                           new UserFolder(context),
                           new GroupFolder(context)
                           //!<# } else { #>
                           new UsersFolder(context)
                           //!<# } #>
                       };
        }
        //$>

        //$<IAclHierarchyItem.ResolveWellKnownPrincipalAsync
        /// <summary>
        /// Retrieves user or group which correspond to a well known principal
        /// (defined in <see cref="WellKnownPrincipal"/>.)
        /// </summary>
        /// <param name="wellKnownPrincipal">Well known principal type.</param>
        /// <returns>Instance of corresponding user/group or <c>null</c> if corresponding user/group
        /// is not supported.</returns>
        public async Task<IPrincipal> ResolveWellKnownPrincipalAsync(WellKnownPrincipal wellKnownPrincipal)
        {
            //!<# if ( Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm) ) { #>
            switch (wellKnownPrincipal)
            {
                case WellKnownPrincipal.All:
                    var sid1 = new SecurityIdentifier(WellKnownSidType.WorldSid, null);
                    return AclFactory.GetPrincipalFromSid(sid1.Value, context);
                case WellKnownPrincipal.Authenticated:
                    var sid2 = new SecurityIdentifier(WellKnownSidType.AuthenticatedUserSid, null);
                    return AclFactory.GetPrincipalFromSid(sid2.Value, context);
                case WellKnownPrincipal.Unauthenticated:
                    return context.AnonymousUser != null ? AclFactory.GetPrincipalFromSid(context.AnonymousUser.User.Value, context) : null;
                case WellKnownPrincipal.Self:
                    return null;
            }

            return null;
            //!<# } else { #>
            throw new DavException("Not implemented.", DavStatus.NOT_IMPLEMENTED);
            //!<# } #>
        }
        //$>

        //$<IAclHierarchyItem.GetItemsByPropertyAsync
        /// <summary>
        /// Find all resources in the subtree which have either Group or Owner (defined by <paramref name="matchBy"/>
        /// parameter) which identifies current user or group current user belongs to.
        /// For example, this report can return all of the resources in a collection
        /// hierarchy that are owned by the current user.
        /// We don't implement it here as all implementatino is in <see cref="DavFolder.GetItemsByProperty"/>.
        /// </summary>
        public virtual async Task<IEnumerable<IAclHierarchyItem>> GetItemsByPropertyAsync(MatchBy matchBy, IList<PropertyName> props)
        {
            return new List<IAclHierarchyItem>();
        }
        //$>

        /// <summary>
        /// Throws <see cref="NeedPrivilegesException"/> if current user doesn't have Read permission on this
        /// file/folder.
        /// </summary>
        protected void RequireReadPrivilege()
        {
            //!<# if ( Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm) ) { #>
            var acl = File.GetAccessControl(
                fileSystemInfo.FullName,
                AccessControlSections.Access | AccessControlSections.Owner | AccessControlSections.Group);

            var rights = EffectivePermissions.GetEffectivePermissions(context.WindowsIdentity, acl);
            if ((rights & FileSystemRights.Read) != FileSystemRights.Read)
            {
                throw new NeedPrivilegesException("Missing read privilege", Path, Privilege.Read);
            }
            //!<# } #>
        }
        //!<# } #>

        /// <summary>
        /// Gets element's parent path. 
        /// </summary>
        /// <param name="path">Element's path.</param>
        /// <returns>Path to parent element.</returns>
        protected static string GetParentPath(string path)
        {
            string parentPath = string.Format("/{0}", path.Trim('/'));
            int index = parentPath.LastIndexOf("/");
            parentPath = parentPath.Substring(0, index);
            return parentPath;
        }

        /// <summary>
        /// Returns WebSocket client ID.
        /// </summary>
        /// <returns>Client ID.</returns>
        protected string GetWebSocketID()
        {
            return context.Request.Headers.ContainsKey("InstanceId") ?
                context.Request.Headers["InstanceId"] : string.Empty;
        }
    }
}
