//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (!Params.CalDav && !Params.CardDav)  return string.Empty; #>
//!<# if (Params.Storage != StorageType.Ntfs)  return string.Empty; #>
using System;
using System.Collections.Generic;
using System.Threading.Tasks;

using ITHit.WebDAV.Server;
//!<# if (Params.CalDav) { #>
using ITHit.WebDAV.Server.CalDav;
//!<# } #>
//!<# if (Params.CardDav) { #>
using ITHit.WebDAV.Server.CardDav;
//!<# } #>

//!<# if (Params.CalDav) { #>
using ITHit.WebDav.Server.SampleModel.__NtfsStorage__.CalDav;
//!<# } #>
//!<# if (Params.CardDav) { #>
using ITHit.WebDav.Server.SampleModel.__NtfsStorage__.CardDav;
//!<# } #>
//!<# if (false) { #>
using ITHit.WebDav.Server.SampleModel.__NtfsStorage__;
//!<# } #>


namespace ITHit.WebDav.Server.SampleModel.__NtfsStorage__
{
    /// <summary>
    /// Assists in finding folders that contain calendars and address books.
    /// </summary>
    public class Discovery :
        /*!<# if (Params.CalDav) { #>!*/ ICalendarDiscovery/*!<# } #>!*/
        /*!<# if (Params.CalDav && Params.CardDav) { #>!*/ ,/*!<# } #>!*//*!<# if (Params.CardDav) { #>!*/ IAddressbookDiscovery/*!<# } #>!*/
    {
        /// <summary>
        /// Instance of <see cref="DavContext"/>.
        /// </summary>
        private DavContext context;

        public Discovery(DavContext context)
        {
            this.context = context;
        }
        //!<# if (Params.CalDav) { #>

        /// <summary>
        /// Returns list of folders that contain calendars owned by this principal.
        /// </summary>
        /// <remarks>This enables calendars discovery owned by current loged-in principal.</remarks>
        public async Task<IEnumerable<IItemCollection>> GetCalendarHomeSetAsync()
        {
            string calendarsUserFolder = string.Format("{0}{1}/", CalendarsRootFolder.CalendarsRootFolderPath, context.UserName);
            return new[] { await DavFolder.GetFolderAsync(context, calendarsUserFolder) };
        }

        /// <summary>
        /// Returns <b>true</b> if <b>calendar-home-set</b> feature is enabled, <b>false</b> otherwise.
        /// </summary>
        /// <remarks>
        /// <para>
        /// In this method you can analyze User-Agent header to find out the client application used for accessing the server
        /// and enable/disable <b>calendar-home-set</b> feature for specific client. 
        /// </para>
        /// <para>
        /// iOS and OS X does require <b>calendar-home-set</b> feature to be always enabled. On the other hand it may consume extra 
        /// resources especially with iOS CalDAV client. iOS starts immediate synchronization of all calendars found on the server 
        /// via home-set request. Typically you will always enable home-set for iOS and OS X CalDAV clients, but may disable it for other clients.
        /// </para>
        /// </remarks>
        public bool CalendarHomeSetEnabled
        {
            get
            {
                return true;
            }
        }
        //!<# } #>
        //!<# if (Params.CardDav) { #>

        /// <summary>
        /// Returns list of folders that contain address books owned by this principal.
        /// </summary>
        /// <remarks>This enables address books discovery owned by current loged-in principal.</remarks>
        public async Task<IEnumerable<IItemCollection>> GetAddressbookHomeSetAsync()
        {
            string addressbooksUserFolder = string.Format("{0}{1}/", AddressbooksRootFolder.AddressbooksRootFolderPath, context.UserName);
            return new[] { await DavFolder.GetFolderAsync(context, addressbooksUserFolder) };
        }

        /// <summary>
        /// Returns <b>true</b> if <b>addressbook-home-set</b> feature is enabled, <b>false</b> otherwise.
        /// </summary>
        /// <remarks>
        /// <para>
        /// In this method you can analyze User-Agent header and enable/disable <b>addressbook-home-set</b> feature for specific client. 
        /// </para>
        /// <para>
        /// iOS and OS X does require <b>addressbook-home-set</b> feature to be always enabled. On the other hand it may consume extra 
        /// resources especially with iOS CardDAV client. iOS starts immediate synchronization of all address books found on the server 
        /// via home-set request. Typically you will always enable heome-set for iOS and OS X CardDAV clients, but may disable it for other clients.
        /// </para>
        /// </remarks>
        public bool AddressbookHomeSetEnabled
        {
            get
            {
                return true;
            }
        }
        //!<# } #>
    }
}