﻿//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (!(Params.SynchronizationCollection && Params.NetCoreVersionNum < 6))  return string.Empty; #>
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace ITHit.WebDav.Server.SampleModel.__NtfsStorage__.Extensions
{
    internal class SemaphoreSlimExtensions
    {
        /// <summary>
        /// Executes a foreach operation in which iterations may run in parallel. Waits untill all operations are completed.
        /// </summary>
        public static async Task ForEachAsync<TSource>(IEnumerable<TSource> sourceItems, Func<TSource, Task> body,
                                                       Action<TSource, Exception> exceptionBody, int maxDegreeOfParallelism = 1000, CancellationToken cancellationToken = default)
        {
            using (SemaphoreSlim concurrencySemaphore = new SemaphoreSlim(maxDegreeOfParallelism))
            {
                List<Task> tasks = new List<Task>();

                foreach (TSource sourceItem in sourceItems)
                {
                    try
                    {
                        await concurrencySemaphore.WaitAsync(cancellationToken);
                    }
                    catch (OperationCanceledException)
                    {

                    }

                    if (cancellationToken.IsCancellationRequested)
                    {
                        return;
                    }

                    var task = Task.Run(async () =>
                    {
                        try
                        {
                            await body(sourceItem);
                        }
                        catch (Exception ex)
                        {
                            exceptionBody(sourceItem, ex);
                        }
                        finally
                        {
                            concurrencySemaphore.Release();
                        }
                    }, cancellationToken);

                    tasks.Add(task);
                }

                try
                {
                    await Task.WhenAll(tasks.ToArray());
                }
                catch (TaskCanceledException)
                {

                }
            }
        }
    }
}
