//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (!(Params.CalDav || Params.CardDav)) return string.Empty; #>
//!<# if (Params.Storage != StorageType.Ntfs)  return string.Empty; #>
using System;
using System.IO;
//!<# if (Params.Hosting == HostingEnvironment.IIS) { #>
using System.Web;
using System.Configuration;
//!<# } #>
//!<# if ( Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm) ) { #>
using System.Security.Principal;
using System.Security.AccessControl;
//!<# } #>
using System.Threading.Tasks;

//!<# if (Params.CalDav) { #>
using ITHit.WebDav.Server.SampleModel.__NtfsStorage__.CalDav;
//!<# } #>
//!<# if (Params.CardDav) { #>
using ITHit.WebDav.Server.SampleModel.__NtfsStorage__.CardDav;
//!<# } #>
//!<# if (false) { #>
// only required to compile project at dev time
using ITHit.WebDav.Server.SampleModel.__NtfsStorage__;
//!<# } #>

namespace ITHit.WebDav.Server.SampleModel.__NtfsStorage__
{
    /// <summary>
    /// This class creates initial calendar(s) and address book(s) for user during first log-in.
    /// </summary>
    /// <remarks>
    /// In case of windows authentication methods in this class are using impersonation. In 
    /// case you run IIS Express and log-in as the user that is different from the one running 
    /// IIS Express, the IIS Express must run with Administrative permissions.
    /// </remarks>
    public class Provisioning /*!<# if (Params.Hosting == HostingEnvironment.IIS) { #>!*/: IHttpModule /*!<# } #>!*/
    {
        //!<# if (Params.Hosting != HostingEnvironment.NetCore) { #>
        /// <summary>
        /// Path to the folder which stores WebDAV files.
        /// </summary>
        private static readonly string repositoryPath = ConfigurationManager.AppSettings["RepositoryPath"] ?? string.Empty;
        //!<# } #>
        //!<# if (Params.Hosting == HostingEnvironment.IIS) { #>

        public void Dispose()
        {
        }

        public void Init(HttpApplication application)
        {
            //!<# if (Params.TargetFramework45OrLater) { #>
            EventHandlerTaskAsyncHelper postAuthAsyncHelper = new EventHandlerTaskAsyncHelper(App_OnPostAuthenticateRequestAsync);
            application.AddOnPostAuthenticateRequestAsync(postAuthAsyncHelper.BeginEventHandler, postAuthAsyncHelper.EndEventHandler);
            //!<# } else { #>
            application.PostAuthenticateRequest += App_OnPostAuthenticateRequest;
            //!<# } #>
        }

        private async Task App_OnPostAuthenticateRequestAsync(object source, EventArgs eventArgs)
        {
            HttpContext httpContext = HttpContext.Current;
            if ((httpContext.User == null) || !httpContext.User.Identity.IsAuthenticated)
                return;

            DavContext context = new DavContext(httpContext);
            //!<# if (Params.CalDav) { #>

            // Create calendars for the user during first log-in.
            await CreateCalendarFoldersAsync(context);
            //!<# } #>
            //!<# if (Params.CardDav) { #>

            // Create addressboks for the user during first log-in.
            await CreateAddressbookFoldersAsync(context);
            //!<# } #>
        }
        //!<# } #>

        //!<# if (Params.CalDav) { #>

        /// <summary>
        /// Creates initial calendars for user as well as inbox and outbox folders.
        /// </summary>
        internal static async Task CreateCalendarFoldersAsync(DavContext context)
        {
            //!<# if (Params.Hosting == HostingEnvironment.NetCore) { #>
            string physicalRepositoryPath = context.RepositoryPath;
            //!<# } else { #>
            string physicalRepositoryPath = /*!<# if (Params.Hosting == HostingEnvironment.IIS) { #>!*/ repositoryPath.StartsWith("~") ? HttpContext.Current.Server.MapPath(repositoryPath) : /*!<# } #>!*/repositoryPath;
            //!<# } #>

            // Get path to user folder /calendars/[user_name]/ and check if it exists.
            string calendarsUserFolder = string.Format("{0}{1}", CalendarsRootFolder.CalendarsRootFolderPath.Replace('/', Path.DirectorySeparatorChar), context.UserName);
            string pathCalendarsUserFolder = Path.Combine(physicalRepositoryPath, calendarsUserFolder.TrimStart(Path.DirectorySeparatorChar));
            if (!Directory.Exists(pathCalendarsUserFolder))
            {
                Directory.CreateDirectory(pathCalendarsUserFolder);
                //!<# if ( Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm) ) { #>

                // Grant full control to loged-in user.
                GrantFullControl(pathCalendarsUserFolder, context);

                // Create all subfolders under the logged-in user account
                // so all folders has logged-in user as the owner.
                context.FileOperation(
                    () =>
                    {
                        // Make the loged-in user the owner of the new folder.
                        MakeOwner(pathCalendarsUserFolder, context);
                //!<# } #>

                        // Create user calendars, such as /calendars/[user_name]/Calendar/.
                        string pathCalendar = Path.Combine(pathCalendarsUserFolder, "Calendar1");
                        Directory.CreateDirectory(pathCalendar);
                        pathCalendar = Path.Combine(pathCalendarsUserFolder, "Home1");
                        Directory.CreateDirectory(pathCalendar);
                        pathCalendar = Path.Combine(pathCalendarsUserFolder, "Work1");
                        Directory.CreateDirectory(pathCalendar);
                        //!<# if (false) { #>

                        // Create outbox folder /calendars/[user_name]/outbox/.
                        string pathOutbox = Path.Combine(pathCalendarsUserFolder, ScheduleOutboxFolder.ScheduleOutboxFolderName);
                        Directory.CreateDirectory(pathOutbox);

                        // Create inbox folder /calendars/[user_name]/inbox/.
                        string pathInbox = Path.Combine(pathCalendarsUserFolder, ScheduleInboxFolder.ScheduleInboxFolderName);
                        Directory.CreateDirectory(pathInbox);
                        //!<# } #>
                        //!<# if ( Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm) ) { #>
                    });
                //!<# } #>
            }
        }
        //!<# } #>
        //!<# if (Params.CardDav) { #>

        /// <summary>
        /// Creates initial address books for user.
        /// </summary>
        internal static async Task CreateAddressbookFoldersAsync(DavContext context)
        {
            //!<# if (Params.Hosting == HostingEnvironment.NetCore) { #>
            string physicalRepositoryPath = context.RepositoryPath;
            //!<# } else { #>
            string physicalRepositoryPath = /*!<# if (Params.Hosting == HostingEnvironment.IIS) { #>!*/ repositoryPath.StartsWith("~") ? HttpContext.Current.Server.MapPath(repositoryPath) : /*!<# } #>!*/repositoryPath;
            //!<# } #>

            // Get path to user folder /addrsessbooks/[user_name]/ and check if it exists.
            string addressbooksUserFolder = string.Format("{0}{1}", AddressbooksRootFolder.AddressbooksRootFolderPath.Replace('/', Path.DirectorySeparatorChar), context.UserName);
            string pathAddressbooksUserFolder = Path.Combine(physicalRepositoryPath, addressbooksUserFolder.TrimStart(Path.DirectorySeparatorChar));
            if (!Directory.Exists(pathAddressbooksUserFolder))
            {
                Directory.CreateDirectory(pathAddressbooksUserFolder);
                //!<# if ( Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm) ) { #>

                // Grant full control to loged-in user.
                GrantFullControl(pathAddressbooksUserFolder, context);

                // Create all subfolders under the loged-in user account
                // so all folders has loged-in user as the owner.
                context.FileOperation(
                    () =>
                    {
                        // Make the loged-in user the owner of the new folder.
                        MakeOwner(pathAddressbooksUserFolder, context);
                //!<# } #>

                        // Create user address books, such as /addressbooks/[user_name]/Addressbook/.
                        string pathAddressbook = Path.Combine(pathAddressbooksUserFolder, "Addressbook1");
                        Directory.CreateDirectory(pathAddressbook);
                        pathAddressbook = Path.Combine(pathAddressbooksUserFolder, "Business1");
                        Directory.CreateDirectory(pathAddressbook);
                        //!<# if ( Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm) ) { #>
                    });
                //!<# } #>
            }
        }
        //!<# } #>
        //!<# if ( Params.AuthenticationType.HasFlag(AuthenticationType.Ntlm) ) { #>

        /// <summary>
        /// Makes the loged-in user the owner of the folder.
        /// </summary>
        /// <param name="folderPath">folder path in file system</param>
        private static void MakeOwner(string folderPath, DavContext context)
        {
            DirectorySecurity securityOwner = Directory.GetAccessControl(folderPath, AccessControlSections.Owner);
            securityOwner.SetOwner(context.WindowsIdentity.User);
            Directory.SetAccessControl(folderPath, securityOwner);
        }

        /// <summary>
        /// Grants full controll to currently loged-in user.
        /// </summary>
        /// <param name="folderPath">folder path in file system</param>
        private static void GrantFullControl(string folderPath, DavContext context)
        {
            DirectoryInfo folder = new DirectoryInfo(folderPath);
            DirectorySecurity security = folder.GetAccessControl();
            security.AddAccessRule(new FileSystemAccessRule(context.WindowsIdentity.User, FileSystemRights.FullControl, InheritanceFlags.ObjectInherit | InheritanceFlags.ContainerInherit, PropagationFlags.None, AccessControlType.Allow));
            folder.SetAccessControl(security);
        }
        //!<# } #>
    }
}
