//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (Params.CalDav || Params.CardDav)  return string.Empty; #>
//!<# if (Params.Storage != StorageType.Db)  return string.Empty; #>
using System;
using System.Collections.Generic;
//!<# if (Params.Hosting != HostingEnvironment.NetCore) { #>
using System.Configuration;
//!<# } #>
using System.Data;
using System.Data.SqlClient;
using System.Diagnostics;
using System.IO;
using System.Linq;
//!<# if (Params.Hosting != HostingEnvironment.NetCore) { #>
using System.Web;
//!<# } #>
using System.Threading.Tasks;

using ITHit.WebDAV.Server;
using ITHit.WebDAV.Server.Class1;
//!<# if (Params.Class2) { #>
using ITHit.WebDAV.Server.Class2;
//!<# } #>
//!<# if (Params.DeltaV) { #>
using ITHit.WebDAV.Server.DeltaV;
//!<# } #>
//!<# if (Params.ResumableUpload || Params.IEUpload) { #>
using ITHit.WebDAV.Server.ResumableUpload;
//!<# } #>

namespace ITHit.WebDav.Server.SampleModel.__SqlStorage__
{
    /// <summary>
    /// Represents file in WebDAV repository.
    /// </summary>
    public class DavFile : DavHierarchyItem, IFile/*!<# if (Params.DeltaV) { #>!*/, IVersionableItem/*!<# } #>!*//*!<# if (Params.ResumableUpload || Params.IEUpload) { #>!*/, IResumableUpload, IUploadProgress/*!<# } #>!*/
    {
        //!<# if (Params.DeltaV && Params.Hosting != HostingEnvironment.NetCore) { #>
        /// <summary>
        /// Autoversioning mode as specified in configuration.
        /// </summary>
        private static readonly AutoVersion autoVersionMode =
            (AutoVersion)Enum.Parse(typeof(AutoVersion), ConfigurationManager.AppSettings["AutoVersionMode"]);
        //!<# } #>

        /// <summary>
        /// Initializes a new instance of the DavFile class.
        /// </summary>
        /// <param name="context">Instance of <see cref="DavContext"/>.</param>
        /// <param name="itemId">Item identifier.</param>
        /// <param name="parentId">Parent identifier.</param>
        /// <param name="name">Item name.</param>
        /// <param name="path">Item path (encoded)</param>
        /// <param name="created">Item creation date.</param>
        /// <param name="modified">Item modification date.</param>
        /// <param name="fileAttributes">File attributes for the item.</param>
        public DavFile(
            DavContext context,
            Guid itemId,
            Guid parentId,
            string name,
            string path,
            DateTime created,
            DateTime modified /*!<# if (Params.MicrosoftExtensions) { #>!*/, /*!<#}#>!*/
            /*!<# if (Params.MicrosoftExtensions) { #>!*/ FileAttributes fileAttributes/*!<#}#>!*/)
            : base(context, itemId, parentId, name, path, created, modified/*!<# if (Params.MicrosoftExtensions) { #>!*/, fileAttributes/*!<#}#>!*/)
        {
        }

        /// <summary>
        /// Gets item's content type.
        /// </summary>
        public string ContentType
        {
            get
            {
                string itemContentType = null;

                int extIndex = Name.LastIndexOf('.');
                if (extIndex != -1)
                {
                    itemContentType = MimeType.GetMimeType(Name.Substring(extIndex + 1));
                }

                if (string.IsNullOrEmpty(itemContentType))
                {
                    itemContentType = getDbField<string>("ContentType", null);
                }

                if (string.IsNullOrEmpty(itemContentType))
                {
                    itemContentType = "application/octet-stream";
                }

                return itemContentType;
            }
        }

        /// <summary>
        /// Gets file length.
        /// </summary>
        public long ContentLength
        {
            get
            {
                object result = Context.ExecuteScalar<object>(
                    "SELECT DATALENGTH(Content) FROM Item WHERE ItemId = @ItemId",
                    "@ItemId", ItemId);

                return DBNull.Value.Equals(result) ? 0 : Convert.ToInt64(result);
            }
        }

        /// <summary>
        /// Gets Etag.
        /// </summary>
        public string Etag
        {
            get
            {
                int serialNumber = getDbField("SerialNumber", 0);
                return string.Format("{0}-{1}", Modified.ToBinary(), serialNumber);
            }
        }

        //!<# if (Params.DASL) { #>
        /// <summary>
        /// Gets or Sets snippet of file content that matches search conditions.
        /// </summary>
        public string Snippet { get; set; }
        //!<# } #>

        //!<# if (Params.DeltaV) { #>
        //$<IVersionableItem.VersionHistory
        /// <summary>
        /// Gets version history item for this file.
        /// </summary>
        public IHistory VersionHistory
        {
            get
            {
                string command =
                    @"SELECT VersionId
                      FROM Version
                      WHERE ItemId = @ItemId";

                if (Context.ExecuteScalar<object>(command, "@ItemId", ItemId) != null)
                {
                    return new VersionHistory(Context, ItemId, Path + "?history");
                }

                return null;
            }
        }
        //$>

        //$<IVersionableItem.IsCheckedOut
        /// <summary>
        /// Gets a value indicating whether the file is checked out.
        /// </summary>
        public bool IsCheckedOut
        {
            get { return getDbField("CheckedOut", false); }
        }
        //$>

        //$<IVersionableItem.IsAutoCheckedOut
        /// <summary>
        /// Gets or sets a value indicating whether the file was automatically checked out
        /// by the engine as result of autoversioning.
        /// </summary>
        public bool IsAutoCheckedOut
        {
            get { return getDbField("AutoCheckedOut", false); }
            set { SetDbField("AutoCheckedOut", value); }
        }
        //$>
        //!<# } #>

        /// <summary>
        /// Reads file's content from storage (to send to client).
        /// </summary>
        /// <param name="output">Stream to read body to.</param>
        /// <param name="byteStart">Number of first byte to write.</param>
        /// <param name="count">Number of bytes to be written.</param>
        public async Task ReadAsync(Stream output, long byteStart, long count)
        {
            //!<# if (Params.Hosting == HostingEnvironment.IIS) { #>
            //Set timeout to maximum value to be able to download large files.
            HttpContext.Current.Server.ScriptTimeout = int.MaxValue;

            //!<# } #>
            if (ContainsDownloadParam(Context.Request.RawUrl))
            {
                AddContentDisposition(Context, Name);
            }
            
            await using (SqlDataReader reader = await Context.ExecuteReaderAsync(
                CommandBehavior.SequentialAccess,
                @"SELECT Content FROM Item WHERE ItemId = @ItemId",
                "@ItemId", ItemId))                        
            {
                await reader.ReadAsync();

                long bufSize = 1048576; // 1Mb
                var buf = new byte[bufSize];
                long retval;
                long startIndex = byteStart;

                //!<# if (Params.Hosting != HostingEnvironment.NetCore) { #>
                try
                {
                //!<# } #>
                    while ((retval = reader.GetBytes(
                        reader.GetOrdinal("Content"),
                        startIndex,
                        buf,
                        0,
                        (int)(count > bufSize ? bufSize : count))) > 0)
                    {
                        await output.WriteAsync(buf, 0, (int)retval);
                        startIndex += retval;
                        count -= retval;
                    }
                //!<# if (Params.Hosting != HostingEnvironment.NetCore) { #>
                }
                //!<# if (Params.Hosting == HostingEnvironment.IIS) { #>
                catch (HttpException)
                {
                    // The remote host closed the connection (for example Cancel or Pause pressed).
                }
                //!<# } else { #>
                catch (System.Net.HttpListenerException)
                {
                    // The remote host closed the connection (for example Cancel or Pause pressed).
                }
                //!<# } #>
                //!<# } #>
            }
        }

        internal static bool ContainsDownloadParam(string url)
        {
            int ind = url.IndexOf('?');
            if (ind > 0 && ind < url.Length - 1)
            {
                string[] param = url.Substring(ind + 1).Split('&');
                return param.Any(p => p.StartsWith("download"));
            }
            return false;
        }

        internal static void AddContentDisposition(DavContext context, string name)
        {
            if (context.Request.UserAgent.Contains("MSIE")) // problem with file extensions in IE
            {
                var fileName = EncodeUtil.EncodeUrlPart(name);
                context.Response.AddHeader("Content-Disposition", "attachment; filename=\"" + fileName + "\"");
            }
            else
                context.Response.AddHeader("Content-Disposition", "attachment;");
        }

        /// <summary>
        /// Stores file contents to storage (when client updates it).
        /// </summary>
        /// <param name="segment">Stream with new file content.</param>
        /// <param name="contentType">New content type.</param>
        /// <param name="startIndex">Index of first byte in the file where update shall be applied.</param>
        /// <param name="totalContentLength">Length of the file after it will be updated with the new content.</param>
        /// <returns>Boolean value indicating if entire stream was written.</returns>
        public async Task<bool> WriteAsync(Stream segment, string contentType, long startIndex, long totalContentLength)
        {
            //!<# if (Params.Class2) { #>
            await RequireHasTokenAsync();

            //!<# } #>
            //!<# if (Params.Hosting == HostingEnvironment.IIS) { #>
            //Set timeout to maximum value to be able to upload large files.
            HttpContext.Current.Server.ScriptTimeout = int.MaxValue;

            //!<# } #>
            await WriteInternalAsync(segment, contentType, startIndex, totalContentLength);
            //!<# if(Params.WebSockets) { #>
            await Context.socketService.NotifyUpdatedAsync(Path, GetWebSocketID());
            //!<# } #>
            return true;
        }

        /// <summary>
        /// Stores file contents to storage (when client updates it).
        /// </summary>
        /// <param name="segment">Stream with new file content.</param>
        /// <param name="contentType">New content type.</param>
        /// <param name="startIndex">Index of first byte in the file where update shall be applied.</param>
        /// <param name="totalContentLength">Length of the file after it will be updated with the new content.</param>
        /// <returns>Boolean value indicating if entire stream was written.</returns>
        public async Task<bool> WriteInternalAsync(Stream segment, string contentType, long startIndex, long totalContentLength)
        {

            string commandText =
                @"UPDATE Item
                  SET
                      Modified = GETUTCDATE(),
                      //!<#if (Params.ResumableUpload || Params.IEUpload) { #>
                      TotalContentLength = CASE WHEN @TotalContentLength >= 0 THEN @TotalContentLength ELSE 0 END,
                      //!<# } #>
                      ContentType = @ContentType,
                      Content = CASE WHEN @ResetContent = 1 THEN 0x ELSE Content END,
                      SerialNumber = ISNULL(SerialNumber, 0) + 1
                  WHERE ItemId = @ItemId";

            await Context.ExecuteNonQueryAsync(
                 commandText,
                 "@ContentType", (object)contentType ?? DBNull.Value,
                 "@ItemId", ItemId,
                 //!<#if (Params.ResumableUpload || Params.IEUpload) { #>
                 "@TotalContentLength", totalContentLength,
                 //!<# } #>
                 "@ResetContent", startIndex == 0 ? 1 : 0);

            const int bufSize = 1048576; // 1Mb
            byte[] buffer = new byte[bufSize];

            long bytes = 0;

            int lastBytesRead;
            //!<# if (Params.Hosting != HostingEnvironment.NetCore) { #>
            try
            {
                //!<# } #>
                while ((lastBytesRead = await segment.ReadAsync(buffer, 0, bufSize)) > 0)
                {
                    SqlParameter dataParm = new SqlParameter("@Data", SqlDbType.VarBinary, bufSize);
                    SqlParameter offsetParm = new SqlParameter("@Offset", SqlDbType.Int);
                    SqlParameter bytesParm = new SqlParameter("@Bytes", SqlDbType.Int, bufSize);
                    SqlParameter itemIdParm = new SqlParameter("@ItemId", ItemId);

                    dataParm.Value = buffer;
                    dataParm.Size = lastBytesRead;
                    offsetParm.Value = bytes + startIndex;
                    bytesParm.Value = lastBytesRead;

                    string updateItemCommand =
                        @"UPDATE Item SET 
                        //!<#if (Params.ResumableUpload || Params.IEUpload) { #>
                            LastChunkSaved = GetUtcDate(),
                        //!<# } #>
                            Content .WRITE(@Data, @Offset, @Bytes)
                          WHERE ItemId = @ItemId";

                    await Context.ExecuteNonQueryAsync(
                        updateItemCommand,
                        dataParm,
                        offsetParm,
                        bytesParm,
                        itemIdParm);

                    bytes += lastBytesRead;
                }
                //!<# if (Params.Hosting != HostingEnvironment.NetCore) { #>
            }
            //!<# if (Params.Hosting == HostingEnvironment.IIS) { #>
            catch (HttpException)
            {
                // The remote host closed the connection (for example Cancel or Pause pressed).
            }
            //!<# } else { #>
            catch (System.Net.HttpListenerException)
            {
                // The remote host closed the connection (for example Cancel or Pause pressed).
            }
            //!<# } #>
            //!<# } #>
            return true;
        }

        /// <summary>
        /// Copies this file to another folder.
        /// </summary>
        /// <param name="destFolder">Destination folder.</param>
        /// <param name="destName">New file name in destination folder.</param>
        /// <param name="deep">Is not used.</param>
        /// <param name="multistatus">Container for errors with items other than this file.</param>
        public override async Task CopyToAsync(
            IItemCollection destFolder,
            string destName,
            bool deep,
            MultistatusException multistatus)
        {
            await CopyToInternalAsync(destFolder, destName, deep, multistatus, 0);
        }

        /// <summary>
        /// Called when this file is being copied.
        /// </summary>
        /// <param name="destFolder">Destination folder.</param>
        /// <param name="destName">New file name.</param>
        /// <param name="deep">Whether children items shall be copied. Ignored for files.</param>
        /// <param name="multistatus">Information about items that failed to copy.</param>
        /// <param name="recursionDepth">Recursion depth.</param>
        public override async Task CopyToInternalAsync(
            IItemCollection destFolder, 
            string destName, 
            bool deep, 
            MultistatusException multistatus, 
            int recursionDepth)
        {
            DavFolder destDavFolder = destFolder as DavFolder;
            if (destFolder == null)
            {
                throw new DavException("Destination folder doesn't exist.", DavStatus.CONFLICT);
            }

            //!<# if (Params.Class2) { #>
            if (!await destDavFolder.ClientHasTokenAsync())
            {
                throw new LockedException("Doesn't have token for destination folder.");
            }
            //!<# } #>

            DavHierarchyItem destItem = await destDavFolder.FindChildAsync(destName);
            if (destItem != null)
            {
                try
                {
                    await destItem.DeleteAsync(multistatus);
                }
                catch (DavException ex)
                {
                    multistatus.AddInnerException(destItem.Path, ex);
                    return;
                }
            }

            await CopyThisItemAsync(destDavFolder, null, destName);

            //!<# if(Params.WebSockets) { #>
            if (recursionDepth == 0)
            {
                await Context.socketService.NotifyCreatedAsync(destFolder.Path + EncodeUtil.EncodeUrlPart(destName), GetWebSocketID());
            }
            //!<# } #>
        }

        /// <summary>
        /// Deletes this file.
        /// </summary>
        /// <param name="multistatus">Is not used.</param>
        public override async Task DeleteAsync(MultistatusException multistatus)
        {
            await DeleteInternalAsync(multistatus, 0);
        }

        /// <summary>
        /// Called whan this file is being deleted.
        /// </summary>
        /// <param name="multistatus">Information about items that failed to delete.</param>
        /// <param name="recursionDepth">Recursion depth.</param>
        public override async Task DeleteInternalAsync(MultistatusException multistatus, int recursionDepth)
        {
            DavFolder parent = await GetParentAsync();
            if (parent == null)
            {
                throw new DavException("Parent is null.", DavStatus.CONFLICT);
            }

            //!<# if (Params.Class2) { #>
            if (!await parent.ClientHasTokenAsync())
            {
                throw new LockedException();
            }

            if (!await ClientHasTokenAsync())
            {
                throw new LockedException();
            }
            //!<# } #>

            await DeleteThisItemAsync(parent);

            //!<# if(Params.WebSockets) { #>
            if (recursionDepth == 0)
            {
                await Context.socketService.NotifyDeletedAsync(Path, GetWebSocketID());
            }
            //!<# } #>
        }

        /// <summary>
        /// Moves this file to different folder and renames it.
        /// </summary>
        /// <param name="destFolder">Destination folder.</param>
        /// <param name="destName">New file name.</param>
        /// <param name="multistatus">Container for errors with items other than this file.</param>
        public override async Task MoveToAsync(IItemCollection destFolder, string destName, MultistatusException multistatus)
        {
            await MoveToInternalAsync(destFolder, destName, multistatus, 0);
        }

        /// <summary>
        /// Called when this file is being moved or renamed.
        /// </summary>
        /// <param name="destFolder">Destination folder.</param>
        /// <param name="destName">New name of this file.</param>
        /// <param name="multistatus">Information about items that failed to move.</param>
        /// <param name="recursionDepth">Recursion depth.</param>
        public override async Task MoveToInternalAsync(IItemCollection destFolder, string destName, MultistatusException multistatus, int recursionDepth)
        {
            DavFolder destDavFolder = destFolder as DavFolder;
            if (destFolder == null)
            {
                throw new DavException("Destination folder doesn't exist.", DavStatus.CONFLICT);
            }

            DavFolder parent = await GetParentAsync();
            if (parent == null)
            {
                throw new DavException("Cannot move root.", DavStatus.CONFLICT);
            }

            //!<# if (Params.Class2) { #>
            if (!await ClientHasTokenAsync() || !await destDavFolder.ClientHasTokenAsync() || !await parent.ClientHasTokenAsync())
            {
                throw new LockedException();
            }
            //!<# } #>

            DavHierarchyItem destItem = await destDavFolder.FindChildAsync(destName);
            if (destItem != null)
            {
                try
                {
                    await destItem.DeleteAsync(multistatus);
                }
                catch (DavException ex)
                {
                    multistatus.AddInnerException(destItem.Path, ex);
                    return;
                }
            }

            await MoveThisItemAsync(destDavFolder, destName, parent);

            //!<# if(Params.WebSockets) { #>
            // Refresh client UI.
            if (recursionDepth == 0)
            {
                await Context.socketService.NotifyMovedAsync(Path, destDavFolder.Path, GetWebSocketID());
            }
            //!<# } #>
        }

        //!<# if (Params.DeltaV) { #>
        //$<IVersionableItem.CheckInAsync
        /// <summary>
        /// Checkins the file. This creates new version and marks the file as checked in.
        /// </summary>
        /// <returns>Path to the version created.</returns>
        public async Task<string> CheckInAsync()
        {
            string newVersionUrl;
            //!<# if (Params.Class2) { #>
            await RequireHasTokenAsync();
            //!<# } #>

            if (this.VersionHistory != null)
            {
                // Create new version. Copy content and properties from this item to new version.

                IVersion v = await this.VersionHistory.GetCurrentVersionAsync();
                Version version = v as Version;
                Debug.Assert(version != null);
                int newVersionNumber = version.VersionNumber + 1;
                string newVersionPath = Version.CreateVersionPath(Path, newVersionNumber);

                Guid newId = Guid.NewGuid();
                // Create new version.
                string command =
                    @"INSERT INTO Version
                         (ItemId,
                          VersionId,
                          VersionNumber,
                          Name,
                          Comment,
                          CreatorDisplayName,
                          Content,
                          ContentType,
                          Created,
                          SerialNumber)
                      SELECT
                          @ItemId,
                          @Identity,
                          @VersionNumber,
                          Name,
                          Comment,
                          IsNull(@CreatorDisplayName, CreatorDisplayName),
                          Content,
                          ContentType,
                          GETUTCDATE(),
                          COALESCE(SerialNumber, 1)
                      FROM Item
                      WHERE ItemId = @ItemId";
                // COALESCE returns the first nonnull expression among its arguments.

               await  Context.ExecuteNonQueryAsync(
                    command,
                    "@ItemId", ItemId,
                    "@VersionNumber", newVersionNumber,
                    "@CreatorDisplayName",
                    string.IsNullOrEmpty(CurrentUserName) ? (object)DBNull.Value : CurrentUserName,
                    "@Identity", newId);

               await Context.ExecuteNonQueryAsync(
                    "UPDATE Item SET Comment = '' WHERE ItemId = @ItemId",
                    "@ItemId", ItemId);

                // Copy properties to new version
                string copyCommand =
                    @"INSERT INTO VersionProperty( VersionId, Name, Namespace, PropVal)
                      SELECT @VersionId, Name, Namespace, PropVal
                      FROM Property
                      WHERE ItemID = @ItemID";

                await Context.ExecuteNonQueryAsync(
                    copyCommand,
                    "@VersionId", newId,
                    "@ItemId", ItemId);

                // Copy content to new version
                newVersionUrl = newVersionPath;
            }
            else
            {
                newVersionUrl = this.Path;
            }

            await setFileCheckedOutAsync(false, false);
            return newVersionUrl;
        }
        //$>

        //$<IVersionableItem.CheckOutAsync
        /// <summary>
        /// Marks the item as checkedout.
        /// </summary>
        /// <param name="autoCheckout">Whether this checkout is performed by the engine itself as result of
        /// autoversioning. We blindly store the value, so it can be retrieved by the engine later.</param>
        public async Task CheckOutAsync(bool autoCheckout)
        {
            //!<# if (Params.Class2) { #>
            await RequireHasTokenAsync();
            //!<# } #>

            await setFileCheckedOutAsync(true, autoCheckout);
        }
        //$>

        //$<IVersionableItem.UnCheckOutAsync
        /// <summary>
        /// Cancels last checkout.
        /// </summary>
        public async Task UnCheckOutAsync()
        {
            //!<# if (Params.Class2) { #>
            await RequireHasTokenAsync();
            //!<# } #>

            if (this.VersionHistory != null)
            {
                // Discard changes.
                // Copy content and properties from current version to this item. Mark item as checked in.

                Version version = (Version)await VersionHistory.GetCurrentVersionAsync();

                // Restore properties.
                await Context.ExecuteNonQueryAsync(
                    "DELETE FROM Property WHERE ItemID = @ItemID",
                    "@ItemId", ItemId);

                string command =
                     @"INSERT INTO Property(ItemId, Name, Namespace, PropVal)
                       SELECT @ItemId, Name, Namespace, PropVal
                       FROM VersionProperty
                       WHERE VersionId = @VersionId";

                await Context.ExecuteNonQueryAsync(
                    command,
                    "@VersionId", version.VersionId,
                    "@ItemId", ItemId);

                string updateItemCommand =
                     @"UPDATE Item
                       SET Content = (SELECT Content FROM [Version] WHERE VersionId = @versionID)
                       WHERE ItemId = @ItemID";

                await Context.ExecuteNonQueryAsync(
                    updateItemCommand,
                    "@ItemId", ItemId,
                    "@VersionId", version.VersionId);
            }
            // Mark item as checked in.
            await setFileCheckedOutAsync(false, false);
        }
        //$>

        //$<IVersionableItem.UpdateToVersionAsync
        /// <summary>
        /// Updates content of the file to the content of the version specified.
        /// </summary>
        /// <param name="version">Version to get content from.</param>
        public async Task UpdateToVersionAsync(IVersion version)
        {
            //!<# if (Params.Class2) { #>
            await RequireHasTokenAsync();
            //!<# } #>
            // Create a new version and copy content and properties from IVersion passed as a parameter. 
            // Replace content and properties of this item.
            Version ver = version as Version;
            if (ver.ItemId != this.ItemId)
            {
                throw new DavException("The version must be from the same item.", DavStatus.CONFLICT);
            }

            IVersion v = await this.VersionHistory.GetCurrentVersionAsync();
            Version currVersion = v as Version;
            int newVersionNumber = currVersion.VersionNumber + 1;

            Guid newID = Guid.NewGuid();

            string command =
                 @"INSERT INTO Version(
                       ItemId,
                       VersionId,
                       VersionNumber,
                       Name,
                       Comment,
                       CreatorDisplayName,
                       Content,
                       ContentType,
                       Created,
                       SerialNumber)
                   SELECT 
                       @ItemId,
                       @Identity,
                       @VersionNumber,
                       i.Name,
                       i.Comment,
                       i.CreatorDisplayName,
                       v.Content,
                       i.ContentType,
                       GETUTCDATE(),
                       v.SerialNumber
                   FROM Item i
                   INNER JOIN [Version] v ON v.ItemId = i.ItemId
                   WHERE i.ItemId = @ItemId AND v.VersionId = @VersionId";

            await Context.ExecuteNonQueryAsync(
                command,
                "@ItemId", ItemId,
                "@VersionNumber", newVersionNumber,
                "@Identity", newID,
                "@VersionID", ver.VersionId);

            string updateContentCommand =
                @"UPDATE Item
                  SET
                      Content = v.Content,
                      SerialNumber = v.SerialNumber
                  FROM Item i
                      INNER JOIN [Version] v ON i.ItemId = v.ItemId
                  WHERE
                      i.ItemId = @ItemId AND v.VersionId = @versionID";

            await Context.ExecuteNonQueryAsync(
                updateContentCommand,
                "@ItemId", ItemId,
                "@VersionId", ver.VersionId);

            // Copy properties to this item
            await Context.ExecuteNonQueryAsync(
                "DELETE FROM Property WHERE ItemID = @ItemID",
                "@ItemId", ItemId);

            string insertPropertyCommand =
                 @"INSERT INTO Property(ItemId, Name, Namespace, PropVal)
                   SELECT @ItemId, Name, Namespace, PropVal
                   FROM VersionProperty
                   WHERE VersionId = @VersionId";

            await Context.ExecuteNonQueryAsync(
                insertPropertyCommand,
                "@ItemId", ItemId,
                "@VersionId", ver.VersionId);

            string insertVersionPropertyCommand =
                @"INSERT INTO VersionProperty(VersionId, Name, Namespace, PropVal)
                  SELECT @NewVerId, Name, Namespace, PropVal
                  FROM VersionProperty
                  WHERE VersionId = @VersionId";

            await Context.ExecuteNonQueryAsync(
                insertVersionPropertyCommand,
                "@NewVerId", newID,
                "@VersionId", ver.VersionId);
        }
        //$>

        //$<IVersionableItem.GetAutoVersionAsync
        /// <summary>
        /// Gets autoversioning mode for the file.
        /// </summary>
        /// <returns>Autoversioning mode for the file.</returns>
        public async Task<AutoVersion> GetAutoVersionAsync()
        {
            return getDbField<AutoVersion>("AutoVersion", AutoVersion.NoAutoVersioning);
        }
        //$>

        //$<IVersionableItem.SetAutoVersionAsync
        /// <summary>
        /// Sets autoversioning mode for the file.
        /// </summary>
        /// <param name="value">Autoversioning mode.</param>
        public async Task SetAutoVersionAsync(AutoVersion value)
        {
            await SetDbFieldAsync("AutoVersion", value);
        }
        //$>

        //$<IVersionableItem.PutUnderVersionControlAsync
        /// <summary>
        /// Enables/disables versioning for this file.
        /// </summary>
        /// <param name="enable">Whether versioning shall be enabled or disabled.</param>
        public async Task PutUnderVersionControlAsync(bool enable)
        {
            if (enable && this.VersionHistory == null)
            {
                // Create new version. The content and properties of the new version is being copied from this item.
                //!<# if (Params.DeltaV && Params.Hosting != HostingEnvironment.NetCore) { #>
                await SetAutoVersionAsync(autoVersionMode);
                //!<# } else { #>
                await SetAutoVersionAsync(Context.ContextConfig.AutoVersionMode);
                //!<# } #>

                Guid newID = Guid.NewGuid();
                string insertVersionCommand =
                    @"INSERT INTO Version(
                          ItemId,
                          VersionId,
                          VersionNumber,
                          Name,
                          Comment,
                          CreatorDisplayName,
                          Content,
                          ContentType,
                          Created,
                          SerialNumber)
                       SELECT
                          @ItemId,
                          @Identity,
                          1,
                          Name,
                          Comment,
                          @CreatorDisplayName,
                          Content,
                          ContentType,
                          GETUTCDATE(),
                          0
                        FROM Item
                        WHERE ItemId = @ItemId";

                await Context.ExecuteNonQueryAsync(
                    insertVersionCommand,
                    "@ItemId", ItemId,
                    "@CreatorDisplayName", CurrentUserName ?? string.Empty,
                    "@Identity", newID);

                string insertVersionPropertyCommand =
                    @"INSERT INTO VersionProperty(VersionId, Name, Namespace, PropVal)
                      SELECT @VersionId, Name, Namespace, PropVal
                      FROM Property
                      WHERE ItemID = @ItemId";

                await Context.ExecuteNonQueryAsync(
                    insertVersionPropertyCommand,
                    "@VersionId", newID,
                    "@ItemId", ItemId);

                await setFileCheckedOutAsync(false, false);
            }
            else if (!enable)
            {
                // Delete all versions
                await Context.ExecuteNonQueryAsync(
                    "DELETE FROM Version WHERE ItemId = @ItemId",
                    "@ItemId", ItemId);
            }
        }
        //$>

        /// <summary>
        /// Retrieves a brief comment about a file that is suitable for presentation to a user.
        /// </summary>
        /// <returns>Brief comment about a file.</returns>
        public async Task<string> GetCommentAsync()
        {
            return getDbField<string>("Comment", null);
        }

        /// <summary>
        /// Sets a brief comment about a file that is suitable for presentation to a user.
        /// </summary>
        /// <param name="comment">Comment string.</param>
        public async Task SetCommentAsync(string comment)
        {
            await SetDbFieldAsync("Comment", comment);
        }

        /// <summary>
        /// Gets creator display name.
        /// </summary>
        /// <returns>Name of person who created this item.</returns>
        public async Task<string> GetCreatorDisplayNameAsync()
        {
            return getDbField<string>("CreatorDisplayName", null);
        }

        /// <summary>
        /// Sets name of person who created the version.
        /// </summary>
        /// <param name="name">Creator name.</param>
        public async Task SetCreatorDisplayNameAsync(string name)
        {
            await SetDbFieldAsync("CreatorDisplayName", name);
        }
        //!<# } #>

        //!<# if (Params.ResumableUpload || Params.IEUpload) { #>
        /// <summary>
        /// Cancels incomplete upload.
        /// </summary>
        public async Task CancelUploadAsync()
        {
            //!<# if (!Params.DeltaV) { #>
            await DeleteAsync(null);
            //!<# } else { #>
            // UnCheckOut();
            //!<# } #>
        }

        /// <summary>
        /// Gets time when last upload piece was saved to disk.
        /// </summary>
        public DateTime LastChunkSaved
        {
            get { return getDbField("LastChunkSaved", DateTime.MinValue); }
        }

        /// <summary>
        /// Gets bytes uploaded sofar.
        /// </summary>
        public long BytesUploaded
        {
            get { return ContentLength; }
        }

        /// <summary>
        /// Gets length of the file which it will have after upload finishes.
        /// </summary>
        public long TotalContentLength
        {
            get { return getDbField<long>("TotalContentLength", -1); }
        }

        /// <summary>
        /// Returns instance of <see cref="IResumableUpload"/> interface for this item.
        /// </summary>
        /// <returns>Instance of <see cref="IResumableUpload"/> interface.</returns>
        public async Task<IEnumerable<IResumableUpload>> GetUploadProgressAsync()
        {
            return new[] { this };
        }
        //!<# } #>

        //!<# if (Params.DeltaV) { #>
        /// <summary>
        /// Sets checked out flag.
        /// </summary>
        /// <param name="value">Checked out or not.</param>
        /// <param name="autoCheckOut">Whether this is invoked by the enging as result of autoversioning.</param>
        private async Task setFileCheckedOutAsync(bool value, bool autoCheckOut)
        {
            await SetDbFieldAsync("CheckedOut", value);
            await SetDbFieldAsync("AutoCheckedOut", autoCheckOut);
        }
        //!<# } #>

        /// <summary>
        /// Returns database field of the file.
        /// </summary>
        /// <typeparam name="T">Type of field value.</typeparam>
        /// <param name="columnName">DB columen in which field is stored.</param>
        /// <param name="defaultValue">Default value to return.</param>
        /// <returns>Value from database or default value if it is null.</returns>
        private T getDbField<T>(string columnName, T defaultValue)
        {
            string commandText = string.Format("SELECT {0} FROM Item WHERE ItemId = @ItemId", columnName);
            object obj = Context.ExecuteScalar<object>(commandText, "@ItemId", ItemId);

            return obj != null ? (T)obj : defaultValue;
        }
    }
}
